@@ -89,7 +89,6 @@ my @minimal_build_configure_args =
 	 '-DWITH_GLOBUS:BOOL'		 => 'OFF',
 	 '-DWITH_GSOAP:BOOL'		 => 'OFF',
 	 '-DWITH_HADOOP:BOOL'		 => 'OFF',
-	 '-DWITH_KRB5:BOOL'			 => 'OFF',
 	 '-DWITH_LIBDELTACLOUD:BOOL' => 'OFF',
 	 '-DWITH_LIBVIRT:BOOL'		 => 'OFF',
 	 '-DWITH_LIBXML2:BOOL'		 => 'OFF',
@@ -203,7 +202,7 @@ our %submit_info = (
 		'build' => {
 			'configure_args' => { 
 			  '-DCMAKE_SUPPRESS_REGENERATION:BOOL' => 'TRUE', # because the windows VM doesn't keep time very well.
-            },
+		},
 			'prereqs'	=> undef,
 			'xtests'	=> undef,
 		},
@@ -216,6 +215,7 @@ our %submit_info = (
 	},
 	
 	'x86_64_Windows8'   => 'x86_64_Windows7',
+	'x86_64_Windows10'  => 'x86_64_Windows7',
 	'x86_WindowsXP'		=> 'x86_64_Windows7',
 	'x86_64_winnt_6.1'	=> 'x86_64_Windows7',
 	'x86_winnt_5.1'		=> 'x86_64_Windows7',
@@ -333,6 +333,8 @@ our %submit_info = (
 	# Add the SWAMP's (temporary) platform name
 	'swamp:rhel-6.4-64'	=> 'x86_64_RedHat6',
 
+	'x86_64_SL7'	=> 'x86_64_SL6',
+
 	# for now SL6 is the same as RedHat6
 	'x86_64_SL6'	=> 'x86_64_RedHat6',
 	'x86_64_sl_6.0' => 'x86_64_SL6',
@@ -449,6 +451,7 @@ our %submit_info = (
 	'x86_64_MacOSX7',	=> 'x86_64_MacOSX',
 	'x86_64_MacOSX8',	=> 'x86_64_MacOSX',
 	'x86_64_MacOSX9',	=> 'x86_64_MacOSX',
+	'x86_64_MacOSX10',	=> 'x86_64_MacOSX',
 
 	#
 	# The SWAMP platforms.
@@ -511,6 +514,8 @@ our %submit_info = (
 	'x86_64_fedora_20'				=> 'x86_64_Fedora',
 	'x86_64_fedora_21'				=> 'x86_64_Fedora',
 	'x86_64_fedora_22'				=> 'x86_64_Fedora',
+	'x86_64_fedora_23'				=> 'x86_64_Fedora',
+	'x86_64_fedora_24'				=> 'x86_64_Fedora',
 
 	##########################################################################
 	# Platform Solaris 11 on x86_64
@@ -524,7 +529,6 @@ our %submit_info = (
 			# we can use ssh_to_job on solaris if we use the proper kerberose
 			# this is OK since we build kerberose only for batlab.
 			'configure_args' => { @default_build_configure_args,
-								  '-DWITH_KRB5:BOOL' => 'OFF',
 								  '-DWITH_GSOAP:BOOL' => 'OFF', 
 								  '-DWITH_CURL:BOOL' => 'OFF',
 								  #'-DHAVE_SSH_TO_JOB:BOOL' => 'OFF',
