@@ -103,97 +103,89 @@ long timezone_offset( time_t clock, bool no_dst )
     return tz_offset;
 }
 
+// convert escapes in-place
+// the string can only shrink while converting escapes so we can safely convert in-place.
 void convert_escapes(string &text, bool &validStr)
 {
-	char *copy;
-	int  length;
-	int  source, dest;
+	validStr = true;
+	if (text.empty())
+		return;
 
-	// We now it will be no longer than the original.
-	length = text.length();
-	copy = new char[length + 1];
-	
-	// We scan up to one less than the length, because we ignore
-	// a terminating slash: it can't be an escape. 
-	dest = 0;
-	for (source = 0; source < length - 1; source++) {
-		if (text[source] != '\\' || source == length - 1) {
-			copy[dest++]= text[source]; 
-		}
-		else {
-			source++;
+	int length = text.length();
+	int dest = 0;
 
-			char new_char;
-			switch(text[source]) {
-			case 'a':	new_char = '\a'; break;
-			case 'b':	new_char = '\b'; break;
-			case 'f':	new_char = '\f'; break;
-			case 'n':	new_char = '\n'; break;
-			case 'r':	new_char = '\r'; break;
-			case 't':	new_char = '\t'; break;
-			case 'v':	new_char = '\v'; break;
-			case '\\':	new_char = '\\'; break;
-			case '\?':	new_char = '\?'; break;
-			case '\'':	new_char = '\''; break;
-			case '\"':	new_char = '\"'; break;
-			default:   
-				if (isodigit(text[source])) {
-					unsigned int  number;
-					// There are three allowed ways to have octal escape characters:
-					//  \[0..3]nn or \nn or \n. We check for them in that order.
-					if (   source <= length - 3
-						&& text[source] >= '0' && text[source] <= '3'
-						&& isodigit(text[source+1])
-						&& isodigit(text[source+2])) {
+	for (int source = 0; source < length; ++source) {
+		char ch = text[source];
+		// scan for escapes, a terminating slash cannot be an escape
+		if (ch == '\\' && source < length - 1) {
+			++source; // skip the \ character
+			ch = text[source];
 
-						// We have the \[0..3]nn case
-						char octal[4];
-						octal[0] = text[source];
-						octal[1] = text[source+1];
-						octal[2] = text[source+2];
-						octal[3] = 0;
-						sscanf(octal, "%o", &number);
-						new_char = number;
-						source += 2; // to account for the two extra digits
-					} else if (   source <= length -2
-							   && isodigit(text[source+1])) {
-
-						// We have the \nn case
-						char octal[3];
-						octal[0] = text[source];
-						octal[1] = text[source+1];
-						octal[2] = 0;
-						sscanf(octal, "%o", &number);
-						new_char = number;
-						source += 1; // to account for the extra digit
-					} else if (source <= length - 1) {
-						char octal[2];
-						octal[0] = text[source];
-						octal[1] = 0;
-						sscanf(octal, "%o", &number);
-						new_char = number;
-					} else {
-						number = new_char = text[source];
+			switch(ch) {
+			case '\"':	ch = '\"'; break;
+			case '\'':	ch = '\''; break;
+			case '\?':	ch = '\?'; break;
+			case 'a':	ch = '\a'; break;
+			case 'b':	ch = '\b'; break;
+			case 'f':	ch = '\f'; break;
+			case 'n':	ch = '\n'; break;
+			case 'r':	ch = '\r'; break;
+			case 't':	ch = '\t'; break;
+			case 'v':	ch = '\v'; break;
+			case '\\':	ch = '\\'; break;
+			default:
+				if (isodigit(ch)) {
+					unsigned int  number = ch - '0';
+					// There can be up to 3 octal digits in an octal escape
+					//  \[0..3]nn or \nn or \n. We quit at 3 characters or
+					// at the first non-octal character.
+					if (source+1 < length) {
+						char digit = text[source+1]; // is the next digit also 
+						if (isodigit(digit)) {
+							++source;
+							number = (number << 3) + digit - '0';
+							if (number < 0x20 && source+1 < length) {
+								digit =  text[source+1];
+								if (isodigit(digit)) {
+									++source;
+									number = (number << 3) + digit - '0';
+								}
+							}
+						}
 					}
-					if(number == 0) { // "\\0" is an invalid substring within a string literal
-					  validStr = false;
-					  delete [] copy;
-					  return;
+					if(ch == 0) { // "\\0" is an invalid substring within a string literal
+						validStr = false;
 					}
 				} else {
-					new_char = text[source];
+					// pass char after \ unmodified.
 				}
 				break;
 			}
-			copy[dest++] = new_char;
+		}
+
+		if (dest == source) {
+			// no need to assign ch to text when we haven't seen any escapes yet.
+			// text[dest] = ch;
+			++dest;
+		} else {
+			text[dest] = ch;
+			++dest;
 		}
 	}
-	copy[dest] = 0;
-	text = copy;
-	delete [] copy;
-	return;
+
+	if (dest < length) {
+		text.erase(dest, std::string::npos);
+		length = dest;
+	}
+	// silly, but to fulfull the original contract for this function
+	// we need to remove the last character in the string if it is a '\0'
+	// (earlier logic guaranteed that a '\0' can ONLY be the last character)
+	if (length > 0 && ! text[length-1]) {
+		text.erase(length-1, std::string::npos);
+	}
 }
 
+
 void 
 getLocalTime(time_t *now, struct tm *localtm) 
 {
