@@ -73,6 +73,9 @@ main_init( int argc, char ** const argv )
 			// don't check parent for schedd addr. use this one instead
 			if ( argc <= i + 1 )
 				usage();
+			if (ScheddAddr) {
+				free(ScheddAddr);
+			}
 			ScheddAddr = strdup( argv[i + 1] );
 			i++;
 			break;
@@ -80,6 +83,9 @@ main_init( int argc, char ** const argv )
 			// specify what pool (i.e. collector) to lookup the schedd name
 			if ( argc <= i + 1 )
 				usage();
+			if (ScheddPool) {
+				free(ScheddPool);
+			}
 			ScheddPool = strdup( argv[i + 1] );
 			i++;
 			break;
@@ -154,8 +160,6 @@ Reconfig()
 	contact_schedd_interval = 
 		param_integer ("C_GAHP_CONTACT_SCHEDD_DELAY", 5);
 
-	useXMLClassads = param_boolean( "GAHP_USE_XML_CLASSADS", false );
-
 		// When GSI authentication is used, we're willing to trust schedds
 		// which have the same credential as the job
 	if ( proxySubjectName ) {
