@@ -72,7 +72,8 @@ CollectorEngine::CollectorEngine (CollectorStats *stats ) :
 	HadAds        (LESSER_TABLE_SIZE , &adNameHashFunction),
 	LeaseManagerAds(LESSER_TABLE_SIZE , &adNameHashFunction),
 	GridAds       (LESSER_TABLE_SIZE , &adNameHashFunction),
-	GenericAds    (LESSER_TABLE_SIZE , &stringHashFunction)
+	GenericAds    (LESSER_TABLE_SIZE , &stringHashFunction),
+	__self_ad__(0)
 {
 	clientTimeout = 20;
 	machineUpdateInterval = 30;
@@ -506,7 +507,7 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 	{
 	  case UPDATE_STARTD_AD:
 	  case UPDATE_STARTD_AD_WITH_ACK:
-#if !defined(WANT_OLD_CLASSADS)
+#if defined(ADD_TARGET_SCOPING)
 		  clientAd->AddTargetRefs( TargetJobAttrs );
 #endif
 		if ( repeatStartdAds > 0 ) {
@@ -578,7 +579,7 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 				fakeAd = new ClassAd(*clientAdToRepeat);
 				snprintf(newname,sizeof(newname),
 						 "Name=\"fake%d-%s\"",n,oldname);
-				fakeAd->InsertOrUpdate(newname);
+				fakeAd->Insert(newname);
 				makeStartdAdHashKey (hk, fakeAd);
 				hashString.Build( hk );
 				if (! updateClassAd (StartdAds, "StartdAd     ", "Start",
@@ -594,7 +595,7 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 		break;
 
 	  case MERGE_STARTD_AD:
-#if !defined(WANT_OLD_CLASSADS)
+#if defined(ADD_TARGET_SCOPING)
 		  clientAd->AddTargetRefs( TargetJobAttrs );
 #endif
 		if (!makeStartdAdHashKey (hk, clientAd))
@@ -646,14 +647,6 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 			retVal = 0;
 			break;
 		}
-		// CRUFT: Before 7.3.2, submitter ads had a MyType of
-		//   "Scheduler". The only way to tell the difference
-		//   was that submitter ads didn't have ATTR_NUM_USERS.
-		//   Coerce MyStype to "Submitter" for ads coming from
-		//   these older schedds.
-		//   Before 7.7.3, submitter ads for parallel universe
-		//   jobs had a MyType of "Scheduler".
-		SetMyTypeName( *clientAd, SUBMITTER_ADTYPE );
 		// since submittor ads always follow a schedd ad, and a master check is
 		// performed for schedd ads, we don't need a master check in here
 		hashString.Build( hk );
@@ -973,8 +966,13 @@ remove (AdTypes adType, AdNameHashKey &hk)
 	}
 	return !table->remove(hk);
 }
-	
-				
+
+void CollectorEngine::
+identifySelfAd(ClassAd * ad)
+{
+	__self_ad__ = (void*)ad;
+}
+
 ClassAd * CollectorEngine::
 updateClassAd (CollectorHashTable &hashTable,
 			   const char *adType,
@@ -1040,6 +1038,8 @@ updateClassAd (CollectorHashTable &hashTable,
 			EXCEPT( "Error inserting ad" );
 		}
 
+		if (isSelfAd(old_ad)) { __self_ad__ = new_ad; }
+
 		delete old_ad;
 
 		insert = 0;
