@@ -29,8 +29,6 @@
 void Init();
 void Register();
 
-DECL_SUBSYSTEM( "CREDD", SUBSYSTEM_TYPE_DAEMON );	// used by Daemon Core
-
 char * myUserName = NULL;
 
 // this appears at the bottom of this file
@@ -141,14 +139,16 @@ main_shutdown_graceful()
 	DC_Exit(0);
 }
 
-void
-main_pre_dc_init( int argc, char* argv[] )
+int
+main( int argc, char **argv )
 {
-}
+	set_mySubSystem( "CREDD", SUBSYSTEM_TYPE_DAEMON );	// used by Daemon Core
 
-void
-main_pre_command_sock_init( )
-{
+	dc_main_init = main_init;
+	dc_main_config = main_config;
+	dc_main_shutdown_fast = main_shutdown_fast;
+	dc_main_shutdown_graceful = main_shutdown_graceful;
+	return dc_main( argc, argv );
 }
 
 // This function is called by dprintf - always display our pid in our
