@@ -30,14 +30,12 @@
 #include "condor_adtypes.h"
 #include "condor_query.h"
 #include "get_daemon_name.h"
-#include "get_full_hostname.h"
 #include "internet.h"
 #include "HashTable.h"
 #include "condor_daemon_core.h"
 #include "dc_collector.h"
 #include "time_offset.h"
 #include "condor_netdb.h"
-#include "daemon_core_sock_adapter.h"
 #include "subsystem_info.h"
 #include "condor_sinful.h"
 
@@ -600,6 +598,11 @@ Daemon::startCommand( int cmd, Stream::stream_type st,Sock **sock,int timeout, C
 	// Also, there's no one to delete the Sock.
 	ASSERT(!nonblocking || callback_fn);
 
+	if (IsDebugLevel(D_COMMAND)) {
+		const char * addr = this->addr();
+		dprintf (D_COMMAND, "Daemon::startCommand(%s,...) making connection to %s\n", getCommandStringSafe(cmd), addr ? addr : "NULL");
+	}
+
 	*sock = makeConnectedSocket(st,timeout,0,errstack,nonblocking);
 	if( ! *sock ) {
 		if ( callback_fn ) {
@@ -814,6 +817,11 @@ Daemon::sendCACmd( ClassAd* req, ClassAd* reply, ReliSock* cmd_sock,
 		cmd_sock->timeout( timeout );
 	}
 
+	if (IsDebugLevel(D_COMMAND)) {
+		dprintf (D_COMMAND, "Daemon::sendCACmd(%s,...) making connection to %s\n",
+			getCommandStringSafe(CA_CMD), _addr ? _addr : "NULL");
+	}
+
 	if( ! connectSock(cmd_sock) ) {
 		std::string err_msg = "Failed to connect to ";
 		err_msg += daemonString(_type);
@@ -1448,14 +1456,14 @@ Daemon::findCmDaemon( const char* cm_name )
 	std::string buf;
 	condor_sockaddr saddr;
 
-	dprintf( D_HOSTNAME, "Using name \"%s\" to find daemon\n", cm_name ); 
+	dprintf( D_HOSTNAME, "Using name \"%s\" to find daemon\n", cm_name );
 
 	Sinful sinful( cm_name );
 
 	if( !sinful.valid() || !sinful.getHost() ) {
 		dprintf( D_ALWAYS, "Invalid address: %s\n", cm_name );
 		formatstr( buf, "%s address or hostname not specified in config file",
-				 _subsys ); 
+				 _subsys );
 		newError( CA_LOCATE_FAILED, buf.c_str() );
 		_is_configured = false;
 		return false;
@@ -1695,10 +1703,13 @@ Daemon::getDefaultPort( void )
 {
 	switch( _type ) {
 	case DT_COLLECTOR:
-		return COLLECTOR_PORT;
+	{
+		int port = param_integer("COLLECTOR_PORT", COLLECTOR_PORT);
+		return port;
 		break;
+	}
 	case DT_VIEW_COLLECTOR:
-		return CONDOR_VIEW_PORT;
+		return param_integer("COLLECTOR_PORT", COLLECTOR_PORT);
 		break;
 	default:
 		return 0;
@@ -2029,9 +2040,6 @@ Daemon::New_addr( char* str )
 						// replace address with private address
 						std::string buf;
 						if( *priv_addr != '<' ) {
-								// [TODO]
-								// if priv address is an IPv6 address,
-								// it should be <[%s]> form
 							formatstr(buf,"<%s>",priv_addr);
 							priv_addr = buf.c_str();
 						}
@@ -2295,6 +2303,11 @@ Daemon::getTimeOffset( long &offset )
 		//
 	offset = TIME_OFFSET_DEFAULT;
 
+	if (IsDebugLevel(D_COMMAND)) {
+		dprintf (D_COMMAND, "Daemon::getTimeOffset(%s,...) making connection to %s\n",
+			getCommandStringSafe(DC_TIME_OFFSET), _addr ? _addr : "NULL");
+	}
+
 		//
 		// First establish a socket to the other daemon
 		//
@@ -2344,6 +2357,10 @@ Daemon::getTimeOffsetRange( long &min_range, long &max_range )
 		//
 	min_range = max_range = TIME_OFFSET_DEFAULT;
 
+	if (IsDebugLevel(D_COMMAND)) {
+		dprintf (D_COMMAND, "Daemon::getTimeOffsetRange(%s,...) making connection to %s\n",
+			getCommandStringSafe(DC_TIME_OFFSET), _addr ? _addr : "NULL");
+	}
 		//
 		// First establish a socket to the other daemon
 		//
