@@ -678,7 +678,7 @@ protected:
 		  the <subsys>_NAME parameter is set in the config file, we
 		  use that, and pass it to build_valid_daemon_name() to make
 		  sure we have a fully-qualified hostname.  If not, we just
-		  use my_full_hostname().  The string we return is newly
+		  use get_local_fqdn().  The string we return is newly
 		  allocated and should be deallocated with delete []. 
 		  */
 	char* localName( void );
@@ -691,6 +691,12 @@ protected:
 		*/
 	bool readAddressFile( const char* subsys );
 
+		/** Should we try to access the daemon via the super port?
+		    @return true if we are a client tool running as root or
+			via the condor_super tool, false if not
+		*/
+	bool useSuperPort();
+
 		/** Code for parsing a locally-written classad, which should
 			contain everything about the daemon
 			@return true if we found everthing in the ad, false if not
