@@ -23,7 +23,8 @@
 #include "condor_daemon_core.h"
 #include "daemon.h"
 #include "dc_message.h"
-#include "daemon_core_sock_adapter.h"
+#include "stopwatch.h"
+#include "condor_config.h"
 
 DCMsg::DCMsg(int cmd):
 	m_cmd( cmd ),
@@ -209,10 +210,13 @@ DCMsg::reportFailure( DCMessenger *messenger )
 	if( m_delivery_status == DELIVERY_CANCELED ) {
 		debug_level = m_msg_cancel_debug_level;
 	}
-	dprintf( debug_level, "Failed to send %s to %s: %s\n",
+	if (debug_level)
+	{
+		dprintf( debug_level, "Failed to send %s to %s: %s\n",
 			 name(),
 			 messenger->peerDescription(),
 			 m_errstack.getFullText().c_str() );
+	}
 }
 
 void
@@ -254,6 +258,7 @@ DCMessenger::DCMessenger( classy_counted_ptr<Daemon> daemon )
 	m_callback_msg = NULL;
 	m_callback_sock = NULL;
 	m_pending_operation = NOTHING_PENDING;
+	m_receive_messages_duration_ms = param_integer("RECEIVE_MSGS_DURATION",0,0);
 }
 
 DCMessenger::DCMessenger( classy_counted_ptr<Sock> sock ):
@@ -262,6 +267,7 @@ DCMessenger::DCMessenger( classy_counted_ptr<Sock> sock ):
 	m_callback_msg = NULL;
 	m_callback_sock = NULL;
 	m_pending_operation = NOTHING_PENDING;
+	m_receive_messages_duration_ms = param_integer("RECEIVE_MSGS_DURATION",0,0);
 }
 
 DCMessenger::~DCMessenger()
@@ -306,7 +312,7 @@ void DCMessenger::startCommand( classy_counted_ptr<DCMsg> msg )
 		// the SafeSock and another for a ReliSock to establish the
 		// security session.
 	Stream::stream_type st = msg->getStreamType();
-	if( daemonCoreSockAdapter.TooManyRegisteredSockets(-1,&error,st==Stream::safe_sock?2:1) ) {
+	if( daemonCore->TooManyRegisteredSockets(-1,&error,st==Stream::safe_sock?2:1) ) {
 			// Try again in a sec
 			// Eventually, it would be better to queue this centrally
 			// (i.e. in DaemonCore) rather than having an independent
@@ -327,6 +333,13 @@ void DCMessenger::startCommand( classy_counted_ptr<DCMsg> msg )
 	m_callback_msg = msg;
 	m_callback_sock = m_sock.get();
 	if( !m_callback_sock ) {
+
+		if (IsDebugLevel(D_COMMAND)) {
+			const char * addr = m_daemon->addr();
+			const int cmd = msg->m_cmd;
+			dprintf (D_COMMAND, "DCMessenger::startCommand(%s,...) making non-blocking connection to %s\n", getCommandStringSafe(cmd), addr ? addr : "NULL");
+		}
+
 		const bool nonblocking = true;
 		m_callback_sock = m_daemon->makeConnectedSocket(st,msg->getTimeout(),msg->getDeadline(),&msg->m_errstack,nonblocking);
 		if( !m_callback_sock ) {
@@ -465,7 +478,7 @@ void DCMessenger::startReceiveMsg( classy_counted_ptr<DCMsg> msg, Sock *sock )
 
 	incRefCount();
 
-	int reg_rc = daemonCoreSockAdapter.
+	int reg_rc = daemonCore->
 		Register_Socket( sock, peerDescription(),
 						 (SocketHandlercpp)&DCMessenger::receiveMsgCallback,
 						 name.c_str(), this, ALLOW );
@@ -488,19 +501,56 @@ void DCMessenger::startReceiveMsg( classy_counted_ptr<DCMsg> msg, Sock *sock )
 int
 DCMessenger::receiveMsgCallback(Stream *sock)
 {
-	classy_counted_ptr<DCMsg> msg = m_callback_msg;
-	ASSERT(msg.get());
+	Stopwatch watch; watch.start();
+	int passes = 0;
+	pending_operation_enum cur_state = m_pending_operation;
+	do
+	{
+		// If we've gone through the loop once and there's not another ready message,
+		// wait for DC to call us back.
+		if (passes)
+		{
+			if (!static_cast<Sock*>(sock)->msgReady())
+			{
+				dprintf(D_NETWORK, "No messages left to process (done %d).\n", passes);
+				break;
+			}
+			dprintf(D_NETWORK, "DC Messenger is processing message %d.\n", passes+1);
+		}
+		passes++;
 
-	m_callback_msg = NULL;
-	m_callback_sock = NULL;
-	m_pending_operation = NOTHING_PENDING;
+		classy_counted_ptr<DCMsg> msg = m_callback_msg;
+		ASSERT(msg.get());
 
-	daemonCoreSockAdapter.Cancel_Socket( sock );
+		m_callback_msg = NULL;
+		m_callback_sock = NULL;
+		m_pending_operation = NOTHING_PENDING;
 
-	ASSERT( sock );
-	readMsg( msg, (Sock *)sock );
+		daemonCore->Cancel_Socket( sock );
+
+		ASSERT( sock );
+
+			// Invoke readMsg() to read and process the message.
+			// Note that in some cases, the callback to process the
+			// message will invoke startReceiveMsg() if it wants to
+			// receive another message; this is the case
+			// in class ScheddNegotiate for instance.  When this
+			// happens, m_pending_operation will get reset to
+			// RECEIVE_MSG_PENDING, and we may end up looping
+			// again via the do/while block below to read the
+			// next message if it is ready without going back to DC.
+			// See gt #4928.
+		readMsg( msg, (Sock *)sock );
+
+		cur_state = m_pending_operation;
+		decRefCount();
+	}
+		// Note that we do not access m_pending_operation after
+		// decRefCount is called - that may have deleted our object.
+	while ((cur_state == RECEIVE_MSG_PENDING) &&
+		   (m_receive_messages_duration_ms > 0) &&
+		   (watch.get_ms() < m_receive_messages_duration_ms));
 
-	decRefCount();
 	return KEEP_STREAM;
 }
 
@@ -569,7 +619,7 @@ DCMessenger::cancelMessage( classy_counted_ptr<DCMsg> msg )
 	else if( m_callback_sock && m_callback_sock->get_file_desc() != INVALID_SOCKET) {
 		m_callback_sock->close();
 			// force callback now so everything gets cleaned up properly
-		daemonCoreSockAdapter.CallSocketHandler( m_callback_sock );
+		daemonCore->CallSocketHandler( m_callback_sock );
 	}
 }
 
@@ -586,18 +636,18 @@ DCMessenger::startCommandAfterDelay( unsigned int delay, classy_counted_ptr<DCMs
 	qc->msg = msg;
 
 	incRefCount();
-	qc->timer_handle = daemonCoreSockAdapter.Register_Timer(
+	qc->timer_handle = daemonCore->Register_Timer(
 		delay,
 		(TimerHandlercpp)&DCMessenger::startCommandAfterDelay_alarm,
 		"DCMessenger::startCommandAfterDelay",
 		this );
 	ASSERT(qc->timer_handle != -1);
-	daemonCoreSockAdapter.Register_DataPtr( qc );
+	daemonCore->Register_DataPtr( qc );
 }
 
 void DCMessenger::startCommandAfterDelay_alarm()
 {
-	QueuedCommand *qc = (QueuedCommand *)daemonCoreSockAdapter.GetDataPtr();
+	QueuedCommand *qc = (QueuedCommand *)daemonCore->GetDataPtr();
 	ASSERT(qc);
 
 	startCommand(qc->msg);
