@@ -1123,7 +1123,7 @@ handle_fetch_log_history(ReliSock *stream, char *name) {
 	free(name);
 
 	int numHistoryFiles = 0;
-	char **historyFiles = 0;
+	const char **historyFiles = 0;
 
 	historyFiles = findHistoryFiles(history_file_param, &numHistoryFiles);
 
@@ -1140,9 +1140,8 @@ handle_fetch_log_history(ReliSock *stream, char *name) {
 	for (int f = 0; f < numHistoryFiles; f++) {
 		filesize_t size;
 		stream->put_file(&size, historyFiles[f]);
-		free(historyFiles[f]);
 	}
-	free(historyFiles);
+	freeHistoryFilesList(historyFiles);
 
 	stream->end_of_message();
 
