@@ -51,10 +51,6 @@ void parseCommandLine(SubmitDagDeepOptions &deepOpts,
 			const char * const argv[]);
 bool parsePreservedArgs(const MyString &strArg, int &argNum, int argc,
 			const char * const argv[], SubmitDagShallowOptions &shallowOpts);
-// Note: doRecursion() should eventually be torn out completely (see
-// gittrac #4189).
-int doRecursion( SubmitDagDeepOptions &deepOpts,
-			SubmitDagShallowOptions &shallowOpts );
 int doRecursionNew( SubmitDagDeepOptions &deepOpts,
 			SubmitDagShallowOptions &shallowOpts );
 int parseJobOrDagLine( const char *dagLine, StringList &tokens,
@@ -108,13 +104,11 @@ int main(int argc, char *argv[])
 		// depth-first so all of the lower-level .condor.sub files already
 		// exist when we check for log files.
 	if ( deepOpts.recurse ) {
-		bool useOldDagReader = param_boolean( "DAGMAN_USE_OLD_DAG_READER",
-					false );
-		if ( useOldDagReader ) {
-			tmpResult = doRecursion( deepOpts, shallowOpts );
-		} else {
-			tmpResult = doRecursionNew( deepOpts, shallowOpts );
+		if ( param_boolean( "DAGMAN_USE_OLD_DAG_READER", false ) ) {
+			fprintf( stderr, "Warning: DAGMAN_USE_OLD_DAG_READER "
+						"is no longer supported\n" );
 		}
+		tmpResult = doRecursionNew( deepOpts, shallowOpts );
 		if ( tmpResult != 0) {
 			fprintf( stderr, "Recursive submit(s) failed; exiting without "
 						"attempting top-level submit\n" );
@@ -148,103 +142,6 @@ int main(int argc, char *argv[])
 	return submitDag( shallowOpts );
 }
 
-//---------------------------------------------------------------------------
-/** Recursively call condor_submit_dag on nested DAGs.
-	@param deepOpts: the condor_submit_dag deep options
-	@return 0 if successful, 1 if failed
-*/
-int
-doRecursion( SubmitDagDeepOptions &deepOpts,
-			SubmitDagShallowOptions &shallowOpts )
-{
-	int result = 0;
-
-	shallowOpts.dagFiles.rewind();
-
-		// Go through all DAG files specified on the command line...
-	StringList submitFiles;
-	const char *dagFile;
-	while ( (dagFile = shallowOpts.dagFiles.next()) ) {
-
-			// Get logical lines from this DAG file.
-		StringList logicalLines;
-		MyString error = MultiLogFiles::fileNameToLogicalLines(
-					dagFile, logicalLines );
-		if ( error != "" ) {
-			fprintf( stderr, "Error reading DAG file: %s\n",
-						error.Value() );
-			return 1;
-		}
-
-			// Find and parse JOB and SUBDAG lines.
-		logicalLines.rewind();
-		const char *dagLine;
-		while ( (dagLine = logicalLines.next()) ) {
-			StringList tokens( dagLine, " \t" );
-			tokens.rewind();
-			const char *first = tokens.next();
-
-			if ( first && !strcasecmp( first, "JOB" ) ) {
-
-					// Get the submit file and directory from the DAG
-					// file line.
-				const char *subFile;
-				const char *directory;
-				if ( parseJobOrDagLine( dagLine, tokens, "submit",
-							subFile, directory ) != 0 ) {
-					return 1;
-				}
-
-					// Now figure out whether JOB line is a nested DAG.
-				MyString submitFile( subFile );
-
-					// If submit file ends in ".condor.sub", we assume it
-					// refers to a sub-DAG.
-				int start = submitFile.find( DAG_SUBMIT_FILE_SUFFIX );
-				if ( start >= 0 &&
-							start + (int)strlen( DAG_SUBMIT_FILE_SUFFIX) ==
-							submitFile.Length() ) {
-
-						// Change submit file name to DAG file name.
-					submitFile.replaceString( DAG_SUBMIT_FILE_SUFFIX, "" );
-
-						// Now run condor_submit_dag on the DAG file.
-					if ( runSubmitDag( deepOpts, submitFile.Value(),
-								directory, false ) != 0 ) {
-						result = 1;
-					}
-				}
-
-			} else if ( first && !strcasecmp( first, "SUBDAG" ) ) {
-
-				const char *inlineOrExt = tokens.next();
-				if ( strcasecmp( inlineOrExt, "EXTERNAL" ) ) {
-					fprintf( stderr, "ERROR: only SUBDAG EXTERNAL is supported "
-								"at this time (line: <%s>)\n", dagLine );
-					return 1;
-				}
-
-					// Get the nested DAG file and directory from the DAG
-					// file line.
-				const char *nestedDagFile;
-				const char *directory;
-				if ( parseJobOrDagLine( dagLine, tokens, "DAG",
-							nestedDagFile, directory ) != 0 ) {
-					return 1;
-				}
-
-					// Now run condor_submit_dag on the DAG file.
-				if ( runSubmitDag( deepOpts, nestedDagFile, directory,
-							false ) != 0 ) {
-					result = 1;
-				}
-			}
-		}
-	}
-
-	return result;
-}
-
 //---------------------------------------------------------------------------
 /** Recursively call condor_submit_dag on nested DAGs.
 	@param deepOpts: the condor_submit_dag deep options
@@ -655,19 +552,17 @@ getOldSubmitFlags(SubmitDagShallowOptions &shallowOpts)
 {
 		// It's not an error for the submit file to not exist.
 	if ( fileExists( shallowOpts.strSubFile ) ) {
-		StringList logicalLines;
-		MyString error = MultiLogFiles::fileNameToLogicalLines(
-					shallowOpts.strSubFile, logicalLines );
+		MultiLogFiles::FileReader reader;
+		MyString error = reader.Open( shallowOpts.strSubFile );
 		if ( error != "" ) {
 			fprintf( stderr, "Error reading submit file: %s\n",
 						error.Value() );
 			return 1;
 		}
 
-		logicalLines.rewind();
-		const char *subLine;
-		while ( (subLine = logicalLines.next()) ) {
-			StringList tokens( subLine, " \t" );
+		MyString subLine;
+		while ( reader.NextLogicalLine( subLine ) ) {
+			StringList tokens( subLine.Value(), " \t" );
 			tokens.rewind();
 			const char *first = tokens.next();
 			if ( first && !strcasecmp( first, "arguments" ) ) {
@@ -676,6 +571,8 @@ getOldSubmitFlags(SubmitDagShallowOptions &shallowOpts)
 				}
 			}
 		}
+
+		reader.Close();
 	}
 
 	return 0;
@@ -716,8 +613,10 @@ parseArgumentsLine( const MyString &subLine,
 	for ( int argNum = 0; argNum < arglist.Count(); argNum++ ) {
 		MyString strArg = arglist.GetArg( argNum );
 		strArg.lower_case();
+		char **args = arglist.GetStringArray();
 		(void)parsePreservedArgs( strArg, argNum, arglist.Count(),
-					arglist.GetStringArray(), shallowOpts);
+					args, shallowOpts);
+		deleteStringArray(args);
 	}
 
 	return 0;
@@ -824,6 +723,9 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		args.AppendArg(deepOpts.strDagmanPath.Value());
 	}
 
+		// -p 0 causes DAGMan to run w/o a command socket (see gittrac #4987).
+	args.AppendArg("-p");
+	args.AppendArg("0");
 	args.AppendArg("-f");
 	args.AppendArg("-l");
 	args.AppendArg(".");
@@ -837,9 +739,6 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 	args.AppendArg(deepOpts.autoRescue);
 	args.AppendArg("-DoRescueFrom");
 	args.AppendArg(deepOpts.doRescueFrom);
-	if(!deepOpts.always_use_node_log) {
-		args.AppendArg("-dont_use_default_node_log");
-	}
 
 	shallowOpts.dagFiles.rewind();
 	while ( (dagFile = shallowOpts.dagFiles.next()) != NULL ) {
@@ -852,39 +751,47 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		args.AppendArg("-MaxIdle");
 		args.AppendArg(shallowOpts.iMaxIdle);
     }
+
     if(shallowOpts.iMaxJobs != 0) 
 	{
 		args.AppendArg("-MaxJobs");
 		args.AppendArg(shallowOpts.iMaxJobs);
     }
+
     if(shallowOpts.iMaxPre != 0) 
 	{
 		args.AppendArg("-MaxPre");
 		args.AppendArg(shallowOpts.iMaxPre);
     }
+
     if(shallowOpts.iMaxPost != 0) 
 	{
 		args.AppendArg("-MaxPost");
 		args.AppendArg(shallowOpts.iMaxPost);
     }
+
 	if(shallowOpts.bNoEventChecks)
 	{
 		// strArgs += " -NoEventChecks";
 		printf( "Warning: -NoEventChecks is ignored; please use "
 					"the DAGMAN_ALLOW_EVENTS config parameter instead\n");
 	}
+
 	if(!shallowOpts.bPostRun)
 	{
 		args.AppendArg("-DontAlwaysRunPost");
 	}
+
 	if(deepOpts.bAllowLogError)
 	{
 		args.AppendArg("-AllowLogError");
 	}
+
 	if(deepOpts.useDagDir)
 	{
 		args.AppendArg("-UseDagDir");
 	}
+
 	if(deepOpts.suppress_notification)
 	{
 		args.AppendArg("-Suppress_notification");
@@ -894,11 +801,17 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		args.AppendArg("-Dont_Suppress_notification");
 	}
 
+	if ( shallowOpts.doRecovery ) {
+		args.AppendArg( "-DoRecov" );
+	}
+
 	args.AppendArg("-CsdVersion");
 	args.AppendArg(CondorVersion());
+
 	if(deepOpts.allowVerMismatch) {
 		args.AppendArg("-AllowVersionMismatch");
 	}
+
 	if(shallowOpts.dumpRescueDag) {
 		args.AppendArg("-DumpRescue");
 	}
@@ -1000,7 +913,8 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		}
 
 		char *line;
-		while ((line = getline(aFile)) != NULL) {
+		int lineno = 0;
+		while ((line = getline_trim(aFile, lineno)) != NULL) {
     		fprintf(pSubFile, "%s\n", line);
 		}
 
@@ -1232,7 +1146,8 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 			}
 			else if ( (strArg.find("-dont_use_default_node_log") != -1) )
 			{
-				deepOpts.always_use_node_log = false;
+				fprintf( stderr, "Error: -dont_use_default_node_log is no longer allowed\n" );
+				printUsage();
 			}
 			else if ( (strArg.find("-suppress_notification") != -1) )
 			{
@@ -1242,11 +1157,6 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 			{
 				deepOpts.suppress_notification = false;
 			}
-			else if ( parsePreservedArgs( strArg, iArg, argc, argv,
-						shallowOpts) )
-			{
-				// No-op here
-			}
 			else if( (strArg.find("-prio") != -1) ) // -priority
 			{
 				if(iArg + 1 >= argc) {
@@ -1255,6 +1165,15 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 				}
 				deepOpts.priority = atoi(argv[++iArg]);
 			}
+			else if ( (strArg.find("-dorecov") != -1) )
+			{
+				shallowOpts.doRecovery = true;
+			}
+			else if ( parsePreservedArgs( strArg, iArg, argc, argv,
+						shallowOpts) )
+			{
+				// No-op here
+			}
 			else
 			{
 				fprintf( stderr, "ERROR: unknown option %s\n", strArg.Value() );
@@ -1358,6 +1277,7 @@ int printUsage(int iExitCode)
     printf("    -notification <value> (Determines how much email you get from Condor.\n");
     printf("        See the condor_submit man page for values.)\n");
     printf("    -NoEventChecks      (Now ignored -- use DAGMAN_ALLOW_EVENTS)\n"); 
+    printf("    -DontAlwaysRunPost  (Don't run POST script if PRE script fails)\n");
     printf("    -AllowLogError      (Allows the DAG to attempt execution even if the log\n");
     printf("        reading code finds errors when parsing the submit files)\n"); 
 	printf("    -UseDagDir          (Run DAGs in directories specified in DAG file paths)\n");
@@ -1382,7 +1302,9 @@ int printUsage(int iExitCode)
 	printf("    -valgrind           (create submit file to run valgrind on DAGMan)\n");
 	printf("    -priority <priority> (jobs will run with this priority by default)\n");
 	printf("    -dont_use_default_node_log (Restore pre-7.9.0 behavior of using UserLog only)\n");
+	printf("        (-dont_use_default_node_log is no longer allowed as of 8.3.1)\n");
 	printf("    -suppress_notification (Set \"notification = never\" in all jobs submitted by this DAGMan)\n");
 	printf("    -dont_suppress_notification (Allow jobs to specify notification)\n");
+	printf("    -DoRecov            (run in recovery mode)\n");
 	exit(iExitCode);
 }
