@@ -61,53 +61,58 @@ static void Usage() {
             "\t\t-Lockfile <NAME.dag.lock>\n"
             "\t\t-Dag <NAME.dag>\n"
             "\t\t-CsdVersion <version string>\n"
-            "\t\t[-Debug <level>]\n"
+            "\t\t[-Help]\n"
+            "\t\t[-Version]\n"
+	    "\t\t[-Debug <level>]\n"
             "\t\t[-MaxIdle <int N>]\n"
             "\t\t[-MaxJobs <int N>]\n"
             "\t\t[-MaxPre <int N>]\n"
             "\t\t[-MaxPost <int N>]\n"
-            "\t\t[-DontAlwaysRunPost]\n"
-            "\t\t[-WaitForDebug]\n"
             "\t\t[-NoEventChecks]\n"
             "\t\t[-AllowLogError]\n"
+            "\t\t[-DontAlwaysRunPost]\n"
+            "\t\t[-WaitForDebug]\n"
             "\t\t[-UseDagDir]\n"
             "\t\t[-AutoRescue <0|1>]\n"
             "\t\t[-DoRescueFrom <int N>]\n"
-            "\t\t[-Priority <int N>]\n"
 			"\t\t[-AllowVersionMismatch]\n"
 			"\t\t[-DumpRescue]\n"
 			"\t\t[-Verbose]\n"
 			"\t\t[-Force]\n"
 			"\t\t[-Notification <never|always|complete|error>]\n"
+			"\t\t[-Suppress_notification]\n"
+			"\t\t[-Dont_Suppress_notification]\n"
 			"\t\t[-Dagman <dagman_executable>]\n"
 			"\t\t[-Outfile_dir <directory>]\n"
 			"\t\t[-Update_submit]\n"
 			"\t\t[-Import_env]\n"
-			"\t\t[-Suppress_notification]\n"
-			"\t\t[-Dont_Suppress_notification]\n"
+            "\t\t[-Priority <int N>]\n"
+			"\t\t[-dont_use_default_node_log] (no longer allowed)\n"
+			"\t\t[-DoRecov]\n"
             "\twhere NAME is the name of your DAG.\n"
-            "\tdefault -Debug is -Debug %d\n", DEBUG_NORMAL);
+            "\tdefault -Debug is -Debug %d\n", DEBUG_VERBOSE );
 	DC_Exit( EXIT_ERROR );
 }
 
 //---------------------------------------------------------------------------
 
+#define MAX_IDLE_DEFAULT 1000
+#define MAX_SUBMITS_PER_INT_DEFAULT 5
+#define LOG_SCAN_INT_DEFAULT 5
 
 Dagman::Dagman() :
 	dag (NULL),
-	maxIdle (0),
+	maxIdle (MAX_IDLE_DEFAULT),
 	maxJobs (0),
-	maxPreScripts (0),
-	maxPostScripts (0),
+	maxPreScripts (20),
+	maxPostScripts (20),
 	paused (false),
 	condorSubmitExe (NULL),
 	condorRmExe (NULL),
-	storkSubmitExe (NULL),
-	storkRmExe (NULL),
 	submit_delay (0),
 	max_submit_attempts (6),
-	max_submits_per_interval (5), // so Coverity is happy
-	m_user_log_scan_interval (5),
+	max_submits_per_interval (MAX_SUBMITS_PER_INT_DEFAULT), // so Coverity is happy
+	m_user_log_scan_interval (LOG_SCAN_INT_DEFAULT),
 	primaryDagFile (""),
 	multiDags (false),
 	startup_cycle_detect (false), // so Coverity is happy
@@ -135,6 +140,8 @@ Dagman::Dagman() :
 	_runPost(true),
 	_defaultPriority(0),
 	_claim_hold_time(20),
+	_doRecovery(false),
+	_suppressJobLogs(false),
 	_dagmanClassad(NULL)
 {
     debug_level = DEBUG_VERBOSE;  // Default debug level is verbose output
@@ -234,10 +241,11 @@ Dagman::Config()
 	debug_printf( DEBUG_NORMAL, "DAGMAN_DEFAULT_PRIORITY setting: %d\n",
 				_defaultPriority );
 
-	_submitDagDeepOpts.always_use_node_log = param_boolean(
-				"DAGMAN_ALWAYS_USE_NODE_LOG", true);
-	debug_printf( DEBUG_NORMAL, "DAGMAN_ALWAYS_USE_NODE_LOG setting: %s\n",
-				_submitDagDeepOpts.always_use_node_log ? "True" : "False" );
+	if ( !param_boolean( "DAGMAN_ALWAYS_USE_NODE_LOG", true ) ) {
+       	debug_printf( DEBUG_QUIET,
+					"Error: setting DAGMAN_ALWAYS_USE_NODE_LOG to false is no longer allowed\n" );
+		DC_Exit( EXIT_ERROR );
+	}
 
 	_submitDagDeepOpts.suppress_notification = param_boolean(
 		"DAGMAN_SUPPRESS_NOTIFICATION",
@@ -353,18 +361,16 @@ Dagman::Config()
 		ASSERT( condorRmExe );
 	}
 
-	free( storkSubmitExe );
-	storkSubmitExe = param( "DAGMAN_STORK_SUBMIT_EXE" );
-	if( !storkSubmitExe ) {
-		storkSubmitExe = strdup( "stork_submit" );
-		ASSERT( storkSubmitExe );
+	if ( param_boolean( "DAGMAN_STORK_SUBMIT_EXE", false ) ) {
+		debug_printf( DEBUG_NORMAL, "Warning: DAGMAN_STORK_SUBMIT_EXE is "
+					"no longer supported\n" );
+		check_warning_strictness( DAG_STRICT_1 );
 	}
 
-	free( storkRmExe );
-	storkRmExe = param( "DAGMAN_STORK_RM_EXE" );
-	if( !storkRmExe ) {
-		storkRmExe = strdup( "stork_rm" );
-		ASSERT( storkRmExe );
+	if ( param_boolean( "DAGMAN_STORK_RM_EXE", false ) ) {
+		debug_printf( DEBUG_NORMAL, "Warning: DAGMAN_STORK_RM_EXE is "
+					"no longer supported\n" );
+		check_warning_strictness( DAG_STRICT_1 );
 	}
 
 	abortDuplicates = param_boolean( "DAGMAN_ABORT_DUPLICATES",
@@ -402,7 +408,7 @@ Dagman::Config()
 	debug_printf( DEBUG_NORMAL, "DAGMAN_WRITE_PARTIAL_RESCUE setting: %s\n",
 				_writePartialRescueDag ? "True" : "False" );
 
-	_defaultNodeLog = param( "DAGMAN_DEFAULT_NODE_LOG" );
+	param( _defaultNodeLog, "DAGMAN_DEFAULT_NODE_LOG" );
 	if ( _defaultNodeLog == "" ) {
 		_defaultNodeLog = "@(DAG_DIR)/@(DAG_FILE).nodes.log";
 	}
@@ -439,6 +445,12 @@ Dagman::Config()
 		free( debugSetting );
 	}
 
+	_suppressJobLogs = 
+				param_boolean( "DAGMAN_SUPPRESS_JOB_LOGS",
+				_suppressJobLogs );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_SUPPRESS_JOB_LOGS setting: %s\n",
+				_suppressJobLogs ? "True" : "False" );
+
 	// enable up the debug cache if needed
 	if (debug_cache_enabled) {
 		debug_cache_set_size(debug_cache_size);
@@ -481,7 +493,8 @@ void main_shutdown_graceful() {
 	DC_Exit( EXIT_RESTART );
 }
 
-void main_shutdown_rescue( int exitVal, Dag::dag_status dagStatus ) {
+void main_shutdown_rescue( int exitVal, Dag::dag_status dagStatus,
+			bool removeCondorJobs ) {
 		// Avoid possible infinite recursion if you hit a fatal error
 		// while writing a rescue DAG.
 	static bool inShutdownRescue = false;
@@ -516,10 +529,14 @@ void main_shutdown_rescue( int exitVal, Dag::dag_status dagStatus ) {
 
 		debug_printf( DEBUG_DEBUG_1, "We have %d running jobs to remove\n",
 					dagman.dag->NumJobsSubmitted() );
-		if( dagman.dag->NumJobsSubmitted() > 0 ) {
-			debug_printf( DEBUG_NORMAL, "Removing submitted jobs...\n" );
-			dagman.dag->RemoveRunningJobs(dagman);
-		}
+			// We just go ahead and do a condor_rm here even if we don't
+			// think we have any jobs running, because if we're aborting
+			// because of DAGMAN_PROHIBIT_MULTI_JOBS getting triggered,
+			// we may have jobs in the queue even if we think we don't.
+			// (See gittrac #4960.) wenger 2015-04-22
+		debug_printf( DEBUG_NORMAL, "Removing submitted jobs...\n" );
+		dagman.dag->RemoveRunningJobs( dagman.DAGManJobId,
+					removeCondorJobs, false );
 		if ( dagman.dag->NumScriptsRunning() > 0 ) {
 			debug_printf( DEBUG_NORMAL, "Removing running scripts...\n" );
 			dagman.dag->RemoveRunningScripts();
@@ -550,7 +567,9 @@ void main_shutdown_rescue( int exitVal, Dag::dag_status dagStatus ) {
 // the schedd will send if the DAGMan job is removed from the queue
 int main_shutdown_remove(Service *, int) {
     debug_printf( DEBUG_QUIET, "Received SIGUSR1\n" );
-	main_shutdown_rescue( EXIT_ABORT, Dag::DAG_STATUS_RM );
+	// We don't remove Condor node jobs here because the schedd will
+	// automatically remove them itself.
+	main_shutdown_rescue( EXIT_ABORT, Dag::DAG_STATUS_RM, false );
 	return FALSE;
 }
 
@@ -800,14 +819,21 @@ void main_init (int argc, char ** const argv) {
 			dagman._submitDagDeepOpts.importEnv = true;
 
         } else if( !strcasecmp( "-priority", argv[i] ) ) {
-		++i;
-		if( i >= argc || strcmp( argv[i], "" ) == 0 ) {
-			debug_printf( DEBUG_NORMAL, "No priority value specified\n");
-			Usage();
-		}
-		dagman._submitDagDeepOpts.priority = atoi(argv[i]);
+			++i;
+			if( i >= argc || strcmp( argv[i], "" ) == 0 ) {
+				debug_printf( DEBUG_NORMAL, "No priority value specified\n");
+				Usage();
+			}
+			dagman._submitDagDeepOpts.priority = atoi(argv[i]);
+
 		} else if( !strcasecmp( "-dont_use_default_node_log", argv[i] ) ) {
-			dagman._submitDagDeepOpts.always_use_node_log = false;
+       		debug_printf( DEBUG_QUIET,
+						"Error: -dont_use_default_node_log is no longer allowed\n" );
+			DC_Exit( EXIT_ERROR );
+
+		} else if ( !strcasecmp( "-dorecov", argv[i] ) ) {
+			dagman._doRecovery = true;
+
         } else {
     		debug_printf( DEBUG_SILENT, "\nUnrecognized argument: %s\n",
 						argv[i] );
@@ -916,15 +942,73 @@ void main_init (int argc, char ** const argv) {
         Usage();
     }
 
-	if ( !dagman._submitDagDeepOpts.always_use_node_log ) {
-        debug_printf( DEBUG_QUIET, "Warning: setting DAGMAN_ALWAYS_USE_NODE_LOG to false is no longer recommended and will probably be disabled in a future version\n" );
-		check_warning_strictness( DAG_STRICT_1 );
+		// This is kind of a guess at adjusting
+		// DAGMAN_MAX_SUBMITS_PER_INTERVAL and DAGMAN_USER_LOG_SCAN_INTERVAL
+		// so that they don't goof up DAGMAN_MAX_JOBS_IDLE too much...
+		// wenger 2015-11-05
+	if ( dagman.maxIdle != MAX_IDLE_DEFAULT ) {
+    	debug_printf( DEBUG_QUIET,
+					"Note:  DAGMAN_MAX_JOBS_IDLE has been changed from "
+					"the default setting; if your submit files create "
+					"multiple procs, you should probably set "
+					"DAGMAN_MAX_SUBMITS_PER_INTERVAL to 1\n" );
+
+		int submitsLimit = MAX( 1, dagman.maxIdle / 10 );
+		if ( dagman.max_submits_per_interval > submitsLimit ) {
+			if ( dagman.max_submits_per_interval ==
+						MAX_SUBMITS_PER_INT_DEFAULT ) {
+					// The user hasn't changed DAGMAN_MAX_SUBMITS_PER_INTERVAL,
+					// so change it to our best guess at something that will
+					// work with DAGMAN_MAX_JOBS_IDLE.
+				dagman.max_submits_per_interval = submitsLimit;
+    			debug_printf( DEBUG_QUIET,
+							"Note:  DAGMAN_MAX_SUBMITS_PER_INTERVAL has been "
+							"changed to %d because of your "
+							"DAGMAN_MAX_JOBS_IDLE setting of %d\n",
+							dagman.max_submits_per_interval, dagman.maxIdle );
+			} else {
+					// If the user has changed this from the default, leave
+					// their setting alone.
+				debug_printf( DEBUG_QUIET,
+							"Warning: your DAGMAN_MAX_SUBMITS_PER_INTERVAL "
+							"setting of %d may interfere with your "
+							"DAGMAN_MAX_JOBS_IDLE setting of %d\n",
+							dagman.max_submits_per_interval, dagman.maxIdle );
+				check_warning_strictness( DAG_STRICT_2 );
+			}
+		}
+
+		if ( dagman.m_user_log_scan_interval >
+					dagman.max_submits_per_interval ) {
+			if ( dagman.m_user_log_scan_interval == LOG_SCAN_INT_DEFAULT ) {
+					// The user hasn't changed DAGMAN_USER_LOG_SCAN_INTERVAL,
+					// so change it to our best guess at something that will
+					// work with DAGMAN_MAX_SUBMITS_PER_INTERVAL.
+				dagman.m_user_log_scan_interval =
+							dagman.max_submits_per_interval;
+    			debug_printf( DEBUG_QUIET,
+							"Note:  DAGMAN_USER_LOG_SCAN_INTERVAL has been "
+							"changed to %d because of the "
+							"DAGMAN_MAX_SUBMITS_PER_INTERVAL setting of %d\n",
+							dagman.m_user_log_scan_interval,
+							dagman.max_submits_per_interval );
+			} else {
+					// If the user has changed this from the default, leave
+					// their setting alone.
+				debug_printf( DEBUG_QUIET,
+							"Warning: your DAGMAN_USER_LOG_SCAN_INTERVAL "
+							"setting of %d may interfere with the "
+							"DAGMAN_MAX_SUBMITS_PER_INTERVAL setting of %d\n",
+							dagman.m_user_log_scan_interval,
+							dagman.max_submits_per_interval );
+				check_warning_strictness( DAG_STRICT_2 );
+			}
+		}
 	}
 
 	//
 	// ...done checking arguments.
 	//
-
     debug_printf( DEBUG_VERBOSE, "DAG Lockfile will be written to %s\n",
                    lockFileName );
 	if ( dagman.dagFiles.number() == 1 ) {
@@ -1004,7 +1088,7 @@ void main_init (int argc, char ** const argv) {
 						  dagman.allowLogError, dagman.useDagDir,
 						  dagman.maxIdle, dagman.retrySubmitFirst,
 						  dagman.retryNodeFirst, dagman.condorRmExe,
-						  dagman.storkRmExe, &dagman.DAGManJobId,
+						  &dagman.DAGManJobId,
 						  dagman.prohibitMultiJobs, dagman.submitDepthFirst,
 						  dagman._defaultNodeLog.Value(),
 						  dagman._generateSubdagSubmits,
@@ -1057,7 +1141,12 @@ void main_init (int argc, char ** const argv) {
 							false, true, false );
 			}
 			
-			dagman.dag->RemoveRunningJobs(dagman, true);
+			// I guess we're setting bForce to true here in case we're
+			// in recovery mode and we have any leftover jobs from
+			// before (e.g., user did condor_hold, modified DAG file
+			// introducing a syntax error, and then did condor_release).
+			// (wenger 2014-10-28)
+			dagman.dag->RemoveRunningJobs( dagman.DAGManJobId, true, true );
 			tolerant_unlink( lockFileName );
 			dagman.CleanUp();
 			
@@ -1113,7 +1202,12 @@ void main_init (int argc, char ** const argv) {
 							true, false );
 			}
 			
-			dagman.dag->RemoveRunningJobs(dagman, true);
+			// I guess we're setting bForce to true here in case we're
+			// in recovery mode and we have any leftover jobs from
+			// before (e.g., user did condor_hold, modified DAG (or
+			// rescue DAG) file introducing a syntax error, and then
+			// did condor_release). (wenger 2014-10-28)
+			dagman.dag->RemoveRunningJobs( dagman.DAGManJobId, true, true );
 			tolerant_unlink( lockFileName );
 			dagman.CleanUp();
 			
@@ -1200,14 +1294,14 @@ void main_init (int argc, char ** const argv) {
 				}
 			}
 
-				// Not using the default node log is the backward
-				// compatible thing to do, so if using the default
-				// log file is already disabled, we don't have to
-				// do any checking.
-			if ( dagman._submitDagDeepOpts.always_use_node_log ) { 
-				dagman.CheckLogFileMode( submitFileVersion );
-			}
-        }
+        } else if ( dagman._doRecovery ) {
+            debug_printf( DEBUG_VERBOSE, "Running in recovery mode because -DoRecovery flag was specified\n" );
+			recovery = true;
+		}
+
+        if ( recovery ) {
+			dagman.CheckLogFileMode( submitFileVersion );
+		}
 
 			//
 			// If this DAGMan continues, it should overwrite the lock
@@ -1254,8 +1348,8 @@ Dagman::CheckLogFileMode( const CondorVersionInfo &submitFileVersion )
 				// Pre-7.9.0 -- default log wasn't implemented yet, so
 				// we need to use individual logs from submit files.
 			debug_printf( DEBUG_QUIET, "Submit file version indicates submit is too old. "
-				"Falling back to 7.8 behavior of not using the default node log\n");
-			DisableDefaultLog();
+				"DAGMan no longer supports individual per-job log files.\n" );
+			DC_Exit( EXIT_ERROR );
 		}
 
 	} else {
@@ -1276,25 +1370,12 @@ Dagman::CheckLogFileMode( const CondorVersionInfo &submitFileVersion )
 				// We are in recovery, but the default log does not exist.
 				// Fall back to 7.8 behavior
 			debug_printf( DEBUG_QUIET, "Default node log does not exist. "
-						"Falling back to 7.8 behavior of not using the default node log\n");
-			DisableDefaultLog();
+				"DAGMan no longer supports individual per-job log files.\n" );
+			DC_Exit( EXIT_ERROR );
 		}
 	}
 }
 
-//---------------------------------------------------------------------------
-void
-Dagman::DisableDefaultLog()
-{
-	dagman._submitDagDeepOpts.always_use_node_log = false;
-		// Note:  we have to explicitly turn off the default
-		// log file here because
-		// _submitDagDeepOpts.always_use_node_log is
-		// referenced in the Dag constructor, so just
-		// changing that here won't do us any good.
-	dagman.dag->UseDefaultNodeLog(false);
-}
-
 //---------------------------------------------------------------------------
 void
 Dagman::ResolveDefaultLog()
@@ -1302,19 +1383,24 @@ Dagman::ResolveDefaultLog()
 	char *dagDir = condor_dirname( primaryDagFile.Value() );
 	const char *dagFile = condor_basename( primaryDagFile.Value() );
 
+	MyString owner;
+	MyString nodeName;
+	dagman._dagmanClassad->GetInfo( owner, nodeName );
+
 	_defaultNodeLog.replaceString( "@(DAG_DIR)", dagDir );
 	_defaultNodeLog.replaceString( "@(DAG_FILE)", dagFile );
 	MyString cluster( DAGManJobId._cluster );
 	_defaultNodeLog.replaceString( "@(CLUSTER)", cluster.Value() );
 	free( dagDir );
+	_defaultNodeLog.replaceString( "@(OWNER)", owner.Value() );
+	_defaultNodeLog.replaceString( "@(NODE_NAME)", nodeName.Value() );
 
 	if ( _defaultNodeLog.find( "@" ) >= 0 ) {
 		debug_printf( DEBUG_QUIET, "Warning: "
 					"default node log file %s contains an '@' character -- "
 					"unresolved macro substituion?\n",
 					_defaultNodeLog.Value() );
-		check_warning_strictness( _submitDagDeepOpts.always_use_node_log ?
-					DAG_STRICT_1 : DAG_STRICT_2 );
+		check_warning_strictness( DAG_STRICT_1 );
 	}
 
 		// Force default log file path to be absolute so it works
@@ -1332,8 +1418,29 @@ Dagman::ResolveDefaultLog()
 		debug_printf( DEBUG_QUIET, "Warning: "
 					"default node log file %s is in /tmp\n",
 					_defaultNodeLog.Value() );
-		check_warning_strictness( _submitDagDeepOpts.always_use_node_log ?
-					DAG_STRICT_1 : DAG_STRICT_2 );
+		check_warning_strictness( DAG_STRICT_1 );
+	}
+
+	bool nfsLogIsError = param_boolean( "DAGMAN_LOG_ON_NFS_IS_ERROR", true );
+	if ( nfsLogIsError ) {
+		bool userlog_locking = param_boolean( "ENABLE_USERLOG_LOCKING", false );
+		if ( userlog_locking ) {
+			bool locks_on_local = param_boolean( "CREATE_LOCKS_ON_LOCAL_DISK", true);
+			if ( locks_on_local ) {
+				debug_printf( DEBUG_QUIET, "Ignoring value of DAGMAN_LOG_ON_NFS_IS_ERROR because ENABLE_USERLOG_LOCKING and CREATE_LOCKS_ON_LOCAL_DISK are true.\n");
+				nfsLogIsError = false;
+			}
+		}
+	}
+
+		// This function returns true if the log file is on NFS and
+		// that is an error.  If the log file is on NFS, but nfsIsError
+		// is false, it prints a warning but returns false.
+	if ( MultiLogFiles::logFileNFSError( _defaultNodeLog.Value(),
+				nfsLogIsError ) ) {
+		debug_printf( DEBUG_QUIET, "Error: log file %s on NFS\n",
+					_defaultNodeLog.Value() );
+		DC_Exit( EXIT_ERROR );
 	}
 
 	debug_printf( DEBUG_NORMAL, "Default node log file is: <%s>\n",
@@ -1342,6 +1449,10 @@ Dagman::ResolveDefaultLog()
 
 void
 print_status() {
+	debug_printf( DEBUG_VERBOSE, "DAG status: %d (%s)\n",
+				dagman.dag->_dagStatus,
+				dagman.dag->GetStatusName() );
+
 	int total = dagman.dag->NumNodes( true );
 	int done = dagman.dag->NumNodesDone( true );
 	int pre = dagman.dag->PreRunNodeCount();
@@ -1398,6 +1509,8 @@ void condor_event_timer () {
     static int prevScriptRunNodes = 0;
     static int prevJobsHeld = 0;
 
+	dagman.dag->RunWaitingScripts();
+
 	int justSubmitted;
 	justSubmitted = dagman.dag->SubmitReadyJobs(dagman);
 	if( justSubmitted ) {
@@ -1409,19 +1522,9 @@ void condor_event_timer () {
 
 	// If the log has grown
 	if( dagman.dag->DetectCondorLogGrowth() ) {
-		if( dagman.dag->ProcessLogEvents( CONDORLOG ) == false ) {
+		if( dagman.dag->ProcessLogEvents() == false ) {
 			debug_printf( DEBUG_NORMAL,
-						"ProcessLogEvents(CONDORLOG) returned false\n" );
-			dagman.dag->PrintReadyQ( DEBUG_DEBUG_1 );
-			main_shutdown_rescue( EXIT_ERROR, Dag::DAG_STATUS_ERROR );
-			return;
-		}
-	}
-
-	if( dagman.dag->DetectDaPLogGrowth() ) {
-		if( dagman.dag->ProcessLogEvents( DAPLOG ) == false ) {
-			debug_printf( DEBUG_NORMAL,
-						"ProcessLogEvents(DAPLOG) returned false\n" );
+						"ProcessLogEvents() returned false\n" );
 			dagman.dag->PrintReadyQ( DEBUG_DEBUG_1 );
 			main_shutdown_rescue( EXIT_ERROR, Dag::DAG_STATUS_ERROR );
 			return;
@@ -1479,6 +1582,9 @@ void condor_event_timer () {
 	// DAG has failed -- dump rescue DAG.
 	//
     if( dagman.dag->DoneFailed( true ) ) {
+		debug_printf( DEBUG_QUIET,
+				  "ERROR: the following job(s) failed:\n" );
+		dagman.dag->PrintJobList( Job::STATUS_ERROR );
 		main_shutdown_rescue( EXIT_ERROR, dagman.dag->_dagStatus );
 		return;
 	}
@@ -1504,6 +1610,9 @@ void condor_event_timer () {
 			// if there is one.
 		debug_printf ( DEBUG_QUIET, "Exiting because DAG is halted "
 					"and no jobs or scripts are running\n" );
+		debug_printf( DEBUG_QUIET,
+				  "ERROR: the following job(s) failed:\n" );
+		dagman.dag->PrintJobList( Job::STATUS_ERROR );
 		main_shutdown_rescue( EXIT_ERROR, Dag::DAG_STATUS_HALTED );
 		return;
 	}
