@@ -25,6 +25,7 @@
 #include "condor_string.h"	// for strnewp and friends
 #include "condor_daemon_core.h"
 #include "spooled_job_files.h"
+#include "write_user_log.h"
 
 #include "gridmanager.h"
 #include "basejob.h"
@@ -736,11 +737,19 @@ int BaseJob::EvalPeriodicJobExpr()
 	bool old_run_time_dirty;
 	UserPolicy user_policy;
 
+#ifdef USE_NON_MUTATING_USERPOLICY
+	user_policy.Init();
+#else
 	user_policy.Init( jobAd );
+#endif
 
 	UpdateJobTime( &old_run_time, &old_run_time_dirty );
 
+#ifdef USE_NON_MUTATING_USERPOLICY
+	int action = user_policy.AnalyzePolicy( *jobAd, PERIODIC_ONLY );
+#else
 	int action = user_policy.AnalyzePolicy( PERIODIC_ONLY );
+#endif
 
 	RestoreJobTime( old_run_time, old_run_time_dirty );
 
@@ -787,7 +796,11 @@ int BaseJob::EvalOnExitJobExpr()
 	bool old_run_time_dirty;
 	UserPolicy user_policy;
 
+#ifdef USE_NON_MUTATING_USERPOLICY
+	user_policy.Init();
+#else
 	user_policy.Init( jobAd );
+#endif
 
 	// The user policy code expects an exit value to be set
 	// If the ON_EXIT attributes haven't been set at all, fake
@@ -806,7 +819,11 @@ int BaseJob::EvalOnExitJobExpr()
 	// TODO: We should just mark the job as done running
 	UpdateJobTime( &old_run_time, &old_run_time_dirty );
 
+#ifdef USE_NON_MUTATING_USERPOLICY
+	int action = user_policy.AnalyzePolicy( *jobAd, PERIODIC_THEN_EXIT );
+#else
 	int action = user_policy.AnalyzePolicy( PERIODIC_THEN_EXIT );
+#endif
 
 	RestoreJobTime( old_run_time, old_run_time_dirty );
 
