@@ -514,6 +514,7 @@ void CreamJob::doEvaluateState()
 		reevaluate_state = false;
 		old_gm_state = gmState;
 		old_remote_state = remoteState;
+		ASSERT ( gahp != NULL || gmState == GM_HOLD || gmState == GM_DELETE );
 
 		switch ( gmState ) {
 		  
@@ -1739,11 +1740,11 @@ TransferRequest *CreamJob::MakeStageInRequest()
 	result = true;
 	jobAd->LookupBool(ATTR_TRANSFER_EXECUTABLE, result);
 	if (result) {
-			//here, JOB_CMD = full path to executable
-		jobAd->LookupString(ATTR_JOB_CMD, tmp_str);
+		GetJobExecutable( jobAd, tmp_str );
 		tmp_str2 = "file://" + tmp_str;
 		local_urls.insert(tmp_str2.c_str());
 
+		jobAd->LookupString( ATTR_JOB_CMD, tmp_str );
 		formatstr( tmp_str2, "%s/%s", uploadUrl,
 				 condor_basename( tmp_str.c_str() ) );
 		remote_urls.insert( tmp_str2.c_str() );
