@@ -22,7 +22,7 @@
 #include "condor_attributes.h"
 #include "condor_debug.h"
 #include "condor_string.h"	// for strnewp and friends
-#include "../condor_daemon_core.V6/condor_daemon_core.h"
+#include "condor_daemon_core.h"
 #include "basename.h"
 #include "nullfile.h"
 #include "filename_tools.h"
@@ -113,8 +113,6 @@ BaseJob* DCloudJobCreate( ClassAd *jobad )
 int DCloudJob::gahpCallTimeout = 600;
 int DCloudJob::submitInterval = 300;
 int DCloudJob::maxConnectFailures = 3;
-int DCloudJob::funcRetryInterval = 15;
-int DCloudJob::pendingWaitTime = 15;
 int DCloudJob::maxRetryTimes = 3;
 
 DCloudJob::DCloudJob( ClassAd *classad )
