@@ -47,7 +47,12 @@ public:
 
 	void Reconfig();
 	void doEvaluateState();
+	void NotifyResourceDown();
+	void NotifyResourceUp();
 	BaseResource *GetResource();
+
+	void ResourceLeaseExpired();
+
 	void SetKeypairId( const char *keypair_id );
 	void SetInstanceId( const char *instance_id );
 	void SetClientToken( const char *client_token );
@@ -59,7 +64,6 @@ public:
 	static int gahpCallTimeout;
 	static int maxConnectFailures;
 	static int funcRetryInterval;
-	static int pendingWaitTime;
 	static int maxRetryTimes;
 	
 	static void setSubmitInterval( int new_interval )	{ submitInterval = new_interval; }
@@ -79,7 +83,12 @@ public:
 	EC2Resource *myResource;
 	GahpClient *gahp;
 
-    void StatusUpdate( const char * newStatus );
+    void StatusUpdate( const char * instanceID,
+                       const char * status,
+                       const char * stateReasonCode,
+                       const char * publicDNSName );
+
+	friend class EC2Resource;
 
 private:
 	// create dynamic input parameters
@@ -89,7 +98,9 @@ private:
 	StringList* build_groupnames();
 
 	std::string m_serviceUrl;
-	
+
+	std::string m_iam_profile_arn;
+	std::string m_iam_profile_name;
 	std::string m_public_key_file;
 	std::string m_private_key_file;
 	std::string m_user_data;
@@ -104,18 +115,24 @@ private:
 	std::string m_key_pair_file;
 	bool m_should_gen_key_pair;
 	bool m_keypair_created;
+	bool m_was_job_completion;
 
-	int m_vm_check_times;
+	int m_retry_times;
 	
 	std::string m_ami_id;
 	std::string m_client_token;
+	std::string m_block_device_mapping;
 	StringList* m_group_names;
+	StringList* m_group_ids;
+	StringList* m_parameters_and_values;
 	
 	std::string m_spot_price;
 	std::string m_spot_request_id;
 	// This is actually a global.
 	const char * m_failure_injection;
 	
+	std::string m_state_reason_code;
+	
 	// remove created temporary keypair file
 	bool remove_keypair_file(const char* filename);
 	
@@ -123,12 +140,14 @@ private:
 	 * associate_n_attach - sends the gahp commands to associate addresses and volumes
 	 * with a running instance.
 	 */ 
-	void associate_n_attach(StringList & returnStatus);
+	void associate_n_attach();
 
 	// print out error codes returned from grid_manager
 	void print_error_code( const char* error_code, const char* function_name );
 
-    bool probeNow;
+	bool probeNow;
+
+	int resourceLeaseTID;
 };
 
 #endif
