@@ -22,7 +22,7 @@
 #include "condor_attributes.h"
 #include "condor_debug.h"
 #include "condor_string.h"	// for strnewp and friends
-#include "../condor_daemon_core.V6/condor_daemon_core.h"
+#include "condor_daemon_core.h"
 #include "basename.h"
 #include "nullfile.h"
 #include "filename_tools.h"
@@ -84,6 +84,12 @@ void GCEJobInit()
 
 void GCEJobReconfig()
 {
+    int gct = param_integer( "GRIDMANAGER_GAHP_CALL_TIMEOUT", 10 * 60 );
+	GCEJob::setGahpCallTimeout( gct );
+
+	int cfrc = param_integer("GRIDMANAGER_CONNECT_FAILURE_RETRY_COUNT", 3);
+	GCEJob::setConnectFailureRetry( cfrc );
+
 	// Tell all the resource objects to deal with their new config values
 	GCEResource *next_resource;
 
@@ -121,7 +127,6 @@ int GCEJob::gahpCallTimeout = 600;
 int GCEJob::submitInterval = 300;
 int GCEJob::maxConnectFailures = 3;
 int GCEJob::funcRetryInterval = 15;
-int GCEJob::pendingWaitTime = 15;
 int GCEJob::maxRetryTimes = 3;
 
 GCEJob::GCEJob( ClassAd *classad ) :
@@ -385,6 +390,7 @@ void GCEJob::doEvaluateState()
 
 		reevaluate_state = false;
 		old_gm_state = gmState;
+		ASSERT ( gahp != NULL || gmState == GM_HOLD || gmState == GM_DELETE );
 
 		switch ( gmState )
 		{
