@@ -63,6 +63,9 @@ main_init( int argc, char ** const argv )
 		case 'C':
 			if ( argc <= i + 1 )
 				usage( argv[0] );
+			if (ScheddJobConstraint) {
+				free(ScheddJobConstraint);
+			}
 			ScheddJobConstraint = strdup( argv[i + 1] );
 			i++;
 			break;
@@ -70,12 +73,18 @@ main_init( int argc, char ** const argv )
 			// don't check parent for schedd addr. use this one instead
 			if ( argc <= i + 1 )
 				usage( argv[0] );
+			if (ScheddAddr) {
+				free(ScheddAddr);
+			}
 			ScheddAddr = strdup( argv[i + 1] );
 			i++;
 			break;
 		case 'S':
 			if ( argc <= i + 1 )
 				usage( argv[0] );
+			if (GridmanagerScratchDir) {
+				free(GridmanagerScratchDir);
+			}
 			GridmanagerScratchDir = strdup( argv[i + 1] );
 			i++;
 			break;
@@ -157,7 +166,8 @@ main_pre_dc_init( int argc, char* argv[] )
 		// We can't call daemonCore->Register_Priv_State() here because
 		// there's no daemonCore object yet. We'll call it in main_init().
 
-		free( owner );
+		free( myUserName );
+		myUserName = owner;
 	} else if ( is_root() ) {
 		dprintf( D_ALWAYS, "Don't know what user to run as!\n" );
 		DC_Exit( 1 );
