@@ -56,7 +56,7 @@ class NordugridResource : public BaseResource
 	static HashTable <HashKey, NordugridResource *> ResourcesByName;
 
  private:
-	void DoPing( time_t& ping_delay, bool& ping_complete,
+	void DoPing( unsigned& ping_delay, bool& ping_complete,
 				 bool& ping_succeeded  );
 
 	void DoJobStatus();
