@@ -43,7 +43,12 @@ using std::string;
 // to be considered distinct values
 static const float PriorityDelta = 0.5;
 
-class ClassAdLog;
+// local working attribute to stash match-cost on resource ads that have a 
+// request matched against them
+#define CP_MATCH_COST "_cp_match_cost"
+
+template <typename K, typename AltK, typename AD> class ClassAdLog;
+class HashKey;
 struct GroupEntry;
 
 class Accountant {
@@ -149,7 +154,7 @@ private:
   // Data members
   //--------------------------------------------------------
 
-  ClassAdLog* AcctLog;
+  ClassAdLog<HashKey, const char*, ClassAd*> * AcctLog;
   int LastUpdateTime;
 
   HashTable<MyString, double> concurrencyLimits;
