@@ -29,13 +29,9 @@
 #define START_PORT						9611
 #define START_UDP_PORT					9611
 
-#define COLLECTOR_PORT					9618
-#define COLLECTOR_UDP_PORT				9618
-#define COLLECTOR_COMM_PORT				9618
-#define COLLECTOR_UDP_COMM_PORT				9618
-#define CREDD_PORT						9620
+#define COLLECTOR_PORT					9618	
 
-#define CONDOR_VIEW_PORT				COLLECTOR_PORT
+#define CREDD_PORT						9620
 
 #define	DEFAULT_CONFIG_SERVER_PORT		    9600
 
@@ -43,7 +39,6 @@
 extern "C" {
 #endif
 
-int do_connect ( const char *host, const char *service, u_short port );
 int tcp_accept_timeout( int ConnectionSock, struct sockaddr *sinful,
 						int *len, int timeout );
 
