@@ -28,6 +28,8 @@ int set_seed(int seed);
 int get_random_int(void);
 unsigned int get_random_uint(void);
 float get_random_float(void);
+double get_random_double( void );
+int timer_fuzz(int period);
 
 #if defined(__cplusplus)
 }
