@@ -241,6 +241,57 @@ typedef enum
 } globus_gram_protocol_job_signal_t;
 
 
+// Pointers to Globus symbols used in Condor_Auth_X509
+#ifdef HAVE_EXT_GLOBUS
+extern int (*globus_thread_set_model_ptr)(
+	const char *);
+extern OM_uint32 (*globus_gss_assist_display_status_str_ptr)(
+	char **, char *, OM_uint32, OM_uint32, int);
+extern globus_result_t (*globus_gss_assist_map_and_authorize_ptr)(
+	gss_ctx_id_t, char *, char *, char *, unsigned int);
+extern OM_uint32 (*globus_gss_assist_acquire_cred_ptr)(
+	OM_uint32 *, gss_cred_usage_t, gss_cred_id_t *);
+extern OM_uint32 (*globus_gss_assist_init_sec_context_ptr)(
+	OM_uint32 *, const gss_cred_id_t, gss_ctx_id_t *, char *, OM_uint32,
+	OM_uint32 *, int *, int (*)(void *, void **, size_t *), void *,
+	int (*)(void *, void *, size_t), void *);
+extern OM_uint32 (*gss_accept_sec_context_ptr)(
+	OM_uint32 *, gss_ctx_id_t *, const gss_cred_id_t, const gss_buffer_t,
+	const gss_channel_bindings_t, gss_name_t *, gss_OID *, gss_buffer_t,
+	OM_uint32 *, OM_uint32 *, gss_cred_id_t *);
+extern OM_uint32 (*gss_compare_name_ptr)(
+	OM_uint32 *, const gss_name_t, const gss_name_t, int *);
+extern OM_uint32 (*gss_context_time_ptr)(
+	OM_uint32 *, const gss_ctx_id_t, OM_uint32 *);
+extern OM_uint32 (*gss_delete_sec_context_ptr)(
+	OM_uint32 *, gss_ctx_id_t *, gss_buffer_t);
+extern OM_uint32 (*gss_display_name_ptr)(
+	OM_uint32 *, const gss_name_t, gss_buffer_t, gss_OID *);
+extern OM_uint32 (*gss_import_cred_ptr)(
+	OM_uint32 *, gss_cred_id_t *, const gss_OID, OM_uint32,
+	const gss_buffer_t, OM_uint32, OM_uint32 *);
+extern OM_uint32 (*gss_import_name_ptr)(
+	OM_uint32 *, const gss_buffer_t, const gss_OID, gss_name_t *);
+extern OM_uint32 (*gss_inquire_context_ptr)(
+	OM_uint32 *, const gss_ctx_id_t, gss_name_t *, gss_name_t *,
+	OM_uint32 *, gss_OID *, OM_uint32 *, int *, int *);
+extern OM_uint32 (*gss_release_buffer_ptr)(
+	OM_uint32 *, gss_buffer_t);
+extern OM_uint32 (*gss_release_cred_ptr)(
+	OM_uint32 *, gss_cred_id_t *);
+extern OM_uint32 (*gss_release_name_ptr)(
+	OM_uint32 *, gss_name_t *);
+extern OM_uint32 (*gss_unwrap_ptr)(
+	OM_uint32 *, const gss_ctx_id_t, const gss_buffer_t, gss_buffer_t, int *,
+	gss_qop_t *);
+extern OM_uint32 (*gss_wrap_ptr)(
+	OM_uint32 *, const gss_ctx_id_t, int, gss_qop_t, const gss_buffer_t,
+	int *, gss_buffer_t);
+extern gss_OID_desc **gss_nt_host_ip_ptr;
+#endif
+
+int activate_globus_gsi();
+
 const char *GlobusJobStatusName( int status );
 
 char *get_x509_proxy_filename( void );
