@@ -21,7 +21,6 @@
 #ifndef __UTIL_LIB_PROTO_H
 #define __UTIL_LIB_PROTO_H
 
-#include "condor_status.h"		/* For STATUS_LINE */
 #include "condor_config.h"
 #include "condor_getmnt.h"
 #include "condor_types.h"
@@ -46,38 +45,17 @@ int copy_file(const char *old_filename, const char *new_filename);
 
 int hardlink_or_copy_file(const char *old_filename, const char *new_filename);
 
-void schedule_event ( int month, int day, int hour, int minute, int second, void (*func)(void) );
-void event_mgr ( void );
-/*
-void StartRecording ( void );
-void CompleteRecording ( long numberOfBytes );
-*/
-void ProcessLogging ( int request, int extraInteger );
 void detach ( void );
 int do_connect ( const char *host, const char *service, u_short port );
 int udp_connect ( char *host, u_short port );
 void dprintf ( int flags, const char* fmt, ... ) CHECK_PRINTF_FORMAT(2,3);
-FILE *debug_lock(int debug_level, const char *mode, int force_lock);
-void debug_unlock ( int debug_level );
 void _EXCEPT_ ( const char *fmt, ... ) CHECK_PRINTF_FORMAT(1,2);
 void condor_except_should_dump_core( int flag );
 int getdtablesize ( void );
 
-char * ltrunc ( register char *str );
-int set_machine_status ( int status );
-int get_machine_status ( void );
 int mkargv ( int *argc, char *argv[], char *line );
 char * format_time ( float fp_secs );
-
-int display_status_line ( STATUS_LINE *line, FILE *fp );
-char * shorten ( char *state );
-int free_status_line ( STATUS_LINE *line );
-int print_header ( FILE *fp );
-char * format_seconds ( int t_sec );
-const char * substr ( char *string, char *pattern );
 void update_rusage( register struct rusage *ru1, register struct rusage *ru2 );
-int sysapi_swap_space ( void );
-int sysapi_disk_space(const char *filename);
 
 #if defined(__cplusplus)
 }		/* End of extern "C" declaration */
