@@ -21,8 +21,7 @@
 #include "condor_config.h"
 #include "shared_port_endpoint.h"
 #include "subsystem_info.h"
-#include "../condor_daemon_core.V6/condor_daemon_core.h"
-#include "daemon_core_sock_adapter.h"
+#include "condor_daemon_core.h"
 #include "counted_ptr.h"
 #include "basename.h"
 
@@ -46,10 +45,15 @@
 static char const *WINDOWS_DAEMON_SOCKET_DIR = "\\\\.\\pipe\\condor";
 #endif
 
+bool SharedPortEndpoint::m_initialized_socket_dir = false;
+bool SharedPortEndpoint::m_created_shared_port_dir = false;
+
 SharedPortEndpoint::SharedPortEndpoint(char const *sock_name):
+	m_is_file_socket(true),
 	m_listening(false),
 	m_registered_listener(false),
 	m_retry_remote_addr_timer(-1),
+	m_max_accepts(8),
 	m_socket_check_timer(-1)
 {
 
@@ -133,19 +137,29 @@ SharedPortEndpoint::GetSharedPortID()
 void
 SharedPortEndpoint::InitAndReconfig()
 {
-	MyString socket_dir;
-	paramDaemonSocketDir(socket_dir);
+	std::string socket_dir;
+#ifdef USE_ABSTRACT_DOMAIN_SOCKET
+	m_is_file_socket = false;
+#endif
+	if (!GetDaemonSocketDir(socket_dir)) {
+		m_is_file_socket = true;
+		if (!GetAltDaemonSocketDir(socket_dir)) {
+			EXCEPT("Unable to determine an appropriate DAEMON_SOCKET_DIR to use.");
+		}
+	}
 
 	if( !m_listening ) {
 		m_socket_dir = socket_dir;
 	}
 	else if( m_socket_dir != socket_dir ) {
 		dprintf(D_ALWAYS,"SharedPortEndpoint: DAEMON_SOCKET_DIR changed from %s to %s, so restarting.\n",
-				m_socket_dir.Value(), socket_dir.Value());
+				m_socket_dir.Value(), socket_dir.c_str());
 		StopListener();
 		m_socket_dir = socket_dir;
 		StartListener();
 	}
+	m_max_accepts = param_integer("SHARED_ENDPOINT_MAX_ACCEPTS_PER_CYCLE",
+						param_integer("MAX_ACCEPTS_PER_CYCLE", 8));
 }
 
 void
@@ -213,8 +227,8 @@ SharedPortEndpoint::StopListener()
 		DeleteCriticalSection(&received_lock);
 	}
 #else
-	if( m_registered_listener && daemonCoreSockAdapter.isEnabled() ) {
-		daemonCoreSockAdapter.Cancel_Socket( &m_listener_sock );
+	if( m_registered_listener && daemonCore ) {
+		daemonCore->Cancel_Socket( &m_listener_sock );
 	}
 	m_listener_sock.close();
 	if( !m_full_name.IsEmpty() ) {
@@ -222,7 +236,7 @@ SharedPortEndpoint::StopListener()
 	}
 
 	if( m_retry_remote_addr_timer != -1 ) {
-		daemonCoreSockAdapter.Cancel_Timer( m_retry_remote_addr_timer );
+		daemonCore->Cancel_Timer( m_retry_remote_addr_timer );
 		m_retry_remote_addr_timer = -1;
 	}
 #endif
@@ -242,8 +256,9 @@ SharedPortEndpoint::CreateListener()
 		return true;
 	}
 
-	m_full_name.formatstr(
-		"%s%c%s",m_socket_dir.Value(),DIR_DELIM_CHAR,m_local_id.Value());
+	std::stringstream ss;
+	ss << m_socket_dir.Value() << DIR_DELIM_CHAR << m_local_id.Value();
+	m_full_name = ss.str();
 
 	pipe_end = CreateNamedPipe(
 		m_full_name.Value(),
@@ -275,16 +290,27 @@ SharedPortEndpoint::CreateListener()
 	}
 
 	m_listener_sock.close();
-	m_listener_sock.assign(sock_fd);
+	m_listener_sock.assignDomainSocket( sock_fd );
 
-	m_full_name.formatstr(
-		"%s%c%s",m_socket_dir.Value(),DIR_DELIM_CHAR,m_local_id.Value());
+	std::stringstream ss;
+	ss << m_socket_dir.Value() << DIR_DELIM_CHAR << m_local_id.Value();
+	m_full_name = ss.str();
 
 	struct sockaddr_un named_sock_addr;
 	memset(&named_sock_addr, 0, sizeof(named_sock_addr));
 	named_sock_addr.sun_family = AF_UNIX;
-	strncpy(named_sock_addr.sun_path,m_full_name.Value(),sizeof(named_sock_addr.sun_path)-1);
-	if( strcmp(named_sock_addr.sun_path,m_full_name.Value()) ) {
+	unsigned named_sock_addr_len;
+	bool is_no_good;
+	if (m_is_file_socket) {
+		strncpy(named_sock_addr.sun_path, m_full_name.Value(), sizeof(named_sock_addr.sun_path)-1);
+		named_sock_addr_len = SUN_LEN(&named_sock_addr);
+		is_no_good = strcmp(named_sock_addr.sun_path,m_full_name.Value());
+	} else {
+		strncpy(named_sock_addr.sun_path+1, m_full_name.Value(), sizeof(named_sock_addr.sun_path)-2);
+		named_sock_addr_len = sizeof(named_sock_addr) - sizeof(named_sock_addr.sun_path) + 1 + strlen(named_sock_addr.sun_path+1);
+		is_no_good = strcmp(named_sock_addr.sun_path+1,m_full_name.Value());;
+	}
+	if( is_no_good ) {
 		dprintf(D_ALWAYS,
 			"ERROR: SharedPortEndpoint: full listener socket name is too long."
 			" Consider changing DAEMON_SOCKET_DIR to avoid this:"
@@ -304,7 +330,7 @@ SharedPortEndpoint::CreateListener()
 			bind(
 				 sock_fd,
 				 (struct sockaddr *)&named_sock_addr,
-				 SUN_LEN(&named_sock_addr));
+				 named_sock_addr_len);
 
 		if( tried_priv_switch ) {
 			set_priv( orig_priv );
@@ -318,13 +344,13 @@ SharedPortEndpoint::CreateListener()
 
 			// bind failed: deal with some common sources of error
 
-		if( RemoveSocket(m_full_name.Value()) ) {
+		if( m_is_file_socket && RemoveSocket(m_full_name.Value()) ) {
 			dprintf(D_ALWAYS,
 				"WARNING: SharedPortEndpoint: removing pre-existing socket %s\n",
 				m_full_name.Value());
 			continue;
 		}
-		else if( MakeDaemonSocketDir() ) {
+		else if( m_is_file_socket && MakeDaemonSocketDir() ) {
 			dprintf(D_ALWAYS,
 				"SharedPortEndpoint: creating DAEMON_SOCKET_DIR=%s\n",
 				m_socket_dir.Value());
@@ -378,10 +404,10 @@ SharedPortEndpoint::StartListener()
 
 	return StartListenerWin32();
 #else
-	ASSERT( daemonCoreSockAdapter.isEnabled() );
+	ASSERT( daemonCore );
 
 	int rc;
-	rc = daemonCoreSockAdapter.Register_Socket(
+	rc = daemonCore->Register_Socket(
 		&m_listener_sock,
 		m_full_name.Value(),
 		(SocketHandlercpp)&SharedPortEndpoint::HandleListenerAccept,
@@ -395,7 +421,7 @@ SharedPortEndpoint::StartListener()
 			// from removing it (and to prevent tmpwatch accidents).
 		const int socket_check_interval = TouchSocketInterval();
 		int fuzz = timer_fuzz(socket_check_interval);
-		m_socket_check_timer = daemonCoreSockAdapter.Register_Timer(
+		m_socket_check_timer = daemonCore->Register_Timer(
 			socket_check_interval + fuzz,
 			socket_check_interval + fuzz,
 			(TimerHandlercpp)&SharedPortEndpoint::SocketCheck,
@@ -563,7 +589,7 @@ SharedPortEndpoint::PipeListenerThread()
 			if(!wake_select_dest)
 			{
 //				dprintf(D_ALWAYS, "SharedPortEndpoint: Registering timer.\n");
-				int status = daemonCoreSockAdapter.Register_Timer_TS(0, (TimerHandlercpp)&SharedPortEndpoint::PipeListenerHelper, "Received socket handler", this);
+				int status = daemonCore->Register_Timer_TS(0, (TimerHandlercpp)&SharedPortEndpoint::PipeListenerHelper, "Received socket handler", this);
 //				dprintf(D_ALWAYS, "SharedPortEndpoint: Timer registration status: %d\n", status);
 			}
 			else
@@ -608,7 +634,7 @@ SharedPortEndpoint::TouchSocketInterval()
 void
 SharedPortEndpoint::SocketCheck()
 {
-	if( !m_listening || m_full_name.IsEmpty() ) {
+	if( !m_listening || m_full_name.IsEmpty() || !m_is_file_socket) {
 		return;
 	}
 
@@ -695,6 +721,29 @@ SharedPortEndpoint::InitRemoteAddress()
 		sinful.setPrivateAddr( private_sinful.getSinful() );
 	}
 
+	// Next, look for alternate command strings
+	std::string commandStrings;
+	if (ad->EvaluateAttrString(ATTR_SHARED_PORT_COMMAND_SINFULS, commandStrings))
+	{
+		m_remote_addrs.clear();
+		StringList sl(commandStrings.c_str());
+		sl.rewind();
+		const char *commandSinfulStr;
+		while ((commandSinfulStr = sl.next()))
+		{
+			Sinful altsinful(commandSinfulStr);
+			altsinful.setSharedPortID(m_local_id.Value());
+			char const *private_addr = sinful.getPrivateAddr();
+			if (private_addr)
+			{
+				Sinful private_sinful(private_addr);
+				private_sinful.setSharedPortID(m_local_id.Value());
+				altsinful.setPrivateAddr(private_sinful.getSinful());
+			}
+			m_remote_addrs.push_back(altsinful);
+		}
+	}
+
 	m_remote_addr = sinful.getSinful();
 
 	return true;
@@ -723,12 +772,12 @@ SharedPortEndpoint::RetryInitRemoteAddress()
 			// Now set up a timer to periodically check for changes
 			// in SharedPortServer's address.
 
-		if( daemonCoreSockAdapter.isEnabled() ) {
+		if( daemonCore ) {
 				// Randomize time a bit so many daemons are unlikely to
 				// do it all at once.
 			int fuzz = timer_fuzz(remote_addr_retry_time);
 
-			m_retry_remote_addr_timer = daemonCoreSockAdapter.Register_Timer(
+			m_retry_remote_addr_timer = daemonCore->Register_Timer(
 				remote_addr_refresh_time + fuzz,
 				(TimerHandlercpp)&SharedPortEndpoint::RetryInitRemoteAddress,
 				"SharedPortEndpoint::RetryInitRemoteAddress",
@@ -740,19 +789,19 @@ SharedPortEndpoint::RetryInitRemoteAddress()
 					// for daemonCore's command socket.  If that isn't
 					// true, we may inform daemonCore more frequently
 					// than necessary, which isn't the end of the world.
-				daemonCoreSockAdapter.daemonContactInfoChanged();
+				daemonCore->daemonContactInfoChanged();
 			}
 		}
 
 		return;
 	}
 
-	if( daemonCoreSockAdapter.isEnabled() ) {
+	if( daemonCore ) {
 		dprintf(D_ALWAYS,
 			"SharedPortEndpoint: did not successfully find SharedPortServer address."
 			" Will retry in %ds.\n",remote_addr_retry_time);
 
-		m_retry_remote_addr_timer = daemonCoreSockAdapter.Register_Timer(
+		m_retry_remote_addr_timer = daemonCore->Register_Timer(
 			remote_addr_retry_time,
 			(TimerHandlercpp)&SharedPortEndpoint::RetryInitRemoteAddress,
 			"SharedPortEndpoint::RetryInitRemoteAddress",
@@ -773,15 +822,23 @@ SharedPortEndpoint::ClearSharedPortServerAddr()
 void
 SharedPortEndpoint::ReloadSharedPortServerAddr()
 {
-	if( daemonCoreSockAdapter.isEnabled() ) {
+	if( daemonCore ) {
 		if( m_retry_remote_addr_timer != -1 ) {
-			daemonCoreSockAdapter.Cancel_Timer( m_retry_remote_addr_timer );
+			daemonCore->Cancel_Timer( m_retry_remote_addr_timer );
 			m_retry_remote_addr_timer = -1;
 		}
 	}
 	RetryInitRemoteAddress();
 }
 
+void
+SharedPortEndpoint::EnsureInitRemoteAddress()
+{
+	if( m_remote_addr.IsEmpty() && m_retry_remote_addr_timer==-1 ) {
+		RetryInitRemoteAddress();
+	}
+}
+
 char const *
 SharedPortEndpoint::GetMyRemoteAddress()
 {
@@ -789,9 +846,7 @@ SharedPortEndpoint::GetMyRemoteAddress()
 		return NULL;
 	}
 
-	if( m_remote_addr.IsEmpty() && m_retry_remote_addr_timer==-1 ) {
-		RetryInitRemoteAddress();
-	}
+	EnsureInitRemoteAddress();
 
 	if( m_remote_addr.IsEmpty() ) {
 		return NULL;
@@ -799,6 +854,13 @@ SharedPortEndpoint::GetMyRemoteAddress()
 	return m_remote_addr.Value();
 }
 
+const std::vector<Sinful> &
+SharedPortEndpoint::GetMyRemoteAddresses()
+{
+	EnsureInitRemoteAddress(); // Initializes the addresses if necessary.
+	return m_remote_addrs;
+}
+
 char const *
 SharedPortEndpoint::GetMyLocalAddress()
 {
@@ -830,7 +892,19 @@ SharedPortEndpoint::HandleListenerAccept( Stream * stream )
 #ifndef WIN32
 	ASSERT( stream == &m_listener_sock );
 #endif
-	DoListenerAccept(NULL);
+	Selector selector;
+	selector.set_timeout( 0, 0 );
+	selector.add_fd( static_cast<Sock*>(stream)->get_file_desc(), Selector::IO_READ );
+
+	for (int idx=0; (idx<m_max_accepts) || (m_max_accepts <= 0); idx++)
+	{
+		DoListenerAccept(NULL);
+		selector.execute();
+		if (!selector.has_ready())
+		{
+			break;
+		}
+	}
 	return KEEP_STREAM;
 }
 
@@ -854,7 +928,7 @@ SharedPortEndpoint::DoListenerAccept(ReliSock *return_remote_sock)
 		remote_sock->enter_connected_state();
 		remote_sock->isClient(false);
 		if(!return_remote_sock)
-			daemonCoreSockAdapter.HandleReqAsync(remote_sock);
+			daemonCore->HandleReqAsync(remote_sock);
 		HeapFree(GetProcessHeap(), NULL, received_socket);
 	}
 	else
@@ -998,7 +1072,7 @@ SharedPortEndpoint::ReceiveSocket( ReliSock *named_sock, ReliSock *return_remote
 	if( !remote_sock ) {
 		remote_sock = new ReliSock();
 	}
-	remote_sock->assign( passed_fd );
+	remote_sock->assignSocket( passed_fd );
 	remote_sock->enter_connected_state();
 	remote_sock->isClient(false);
 
@@ -1018,8 +1092,8 @@ SharedPortEndpoint::ReceiveSocket( ReliSock *named_sock, ReliSock *return_remote
 
 
 	if( !return_remote_sock ) {
-		ASSERT( daemonCoreSockAdapter.isEnabled() );
-		daemonCoreSockAdapter.HandleReqAsync(remote_sock);
+		ASSERT( daemonCore );
+		daemonCore->HandleReqAsync(remote_sock);
 		remote_sock = NULL; // daemonCore took ownership of remote_sock
 	}
 	free(buf);
@@ -1080,7 +1154,7 @@ SharedPortEndpoint::deserialize(char *inherit_buf)
 	*/
 	sscanf_s(inherit_buf, "%d", (int*)&pipe_end);
 
-	//m_pipe_out = daemonCoreSockAdapter.Inherit_Pipe_Handle(out_pipe, false, true, true, 4096);
+	//m_pipe_out = daemonCore->Inherit_Pipe_Handle(out_pipe, false, true, true, 4096);
 #else
 	inherit_buf = m_listener_sock.serialize(inherit_buf);
 #endif
@@ -1103,7 +1177,7 @@ SharedPortEndpoint::UseSharedPort(MyString *why_not,bool already_open)
 {
 #ifndef HAVE_SHARED_PORT
 	if( why_not ) {
-		why_not->sprintf("shared ports not supported on this platform");
+		*why_not = "shared ports not supported on this platform";
 	}
 	return false;
 #else
@@ -1115,9 +1189,13 @@ SharedPortEndpoint::UseSharedPort(MyString *why_not,bool already_open)
 		// to decide whether to create a shared port for our child.
 		// In the latter case, other methods are used to determine
 		// that a shared port should not be used.
+		// The HAD and REPLICATION daemons can't properly handle a
+		// shared port address for themselves or their peers.
 
 	bool never_use_shared_port =
-		get_mySubSystem()->isType(SUBSYSTEM_TYPE_SHARED_PORT);
+		get_mySubSystem()->isType(SUBSYSTEM_TYPE_SHARED_PORT) ||
+		get_mySubSystem()->nameMatch("HAD") ||
+		get_mySubSystem()->nameMatch("REPLICATION");
 
 	if( never_use_shared_port ) {
 		if( why_not ) {
@@ -1160,17 +1238,34 @@ SharedPortEndpoint::UseSharedPort(MyString *why_not,bool already_open)
 
 	time_t now = time(NULL);
 	if( abs((int)now-(int)cached_time) > 10 || cached_time==0 || why_not ) {
-		MyString socket_dir;
-		paramDaemonSocketDir(socket_dir);
-
 		cached_time = now;
-		cached_result = access(socket_dir.Value(),W_OK)==0;
+
+		std::string socket_dir;
+		bool is_file_socket = true;
+#ifdef USE_ABSTRACT_DOMAIN_SOCKET
+		is_file_socket = false;
+#endif
+		if (!GetDaemonSocketDir(socket_dir)) {
+			is_file_socket = true;
+			if (!GetAltDaemonSocketDir(socket_dir)) {
+				why_not->formatstr("No DAEMON_SOCKET_DIR is available.\n");
+				cached_result = false;
+				return cached_result;
+			}
+		}
+
+		if (!is_file_socket) {
+			cached_result = true;
+			return cached_result;
+		}
+
+		cached_result = access(socket_dir.c_str(),W_OK)==0;
 
 		if( !cached_result && errno == ENOENT )
 		{
 				// if socket_dir doesn't exist, see if we are allowed to
 				// create it
-			char *parent_dir = condor_dirname( socket_dir.Value() );
+			char *parent_dir = condor_dirname( socket_dir.c_str() );
 			if( parent_dir ) {
 				cached_result = access(parent_dir,W_OK)==0;
 				free( parent_dir );
@@ -1179,7 +1274,7 @@ SharedPortEndpoint::UseSharedPort(MyString *why_not,bool already_open)
 
 		if( !cached_result && why_not ) {
 			why_not->formatstr("cannot write to %s: %s",
-						   socket_dir.Value(),
+						   socket_dir.c_str(),
 						   strerror(errno));
 		}
 	}
@@ -1311,14 +1406,106 @@ SharedPortEndpoint::MakeDaemonSocketDir()
 }
 #endif
 
+
 void
-SharedPortEndpoint::paramDaemonSocketDir(MyString &result)
+SharedPortEndpoint::InitializeDaemonSocketDir()
+{
+	if ( m_initialized_socket_dir ) {
+		return;
+	}
+	m_initialized_socket_dir = true;
+
+	std::string result;
+#ifdef USE_ABSTRACT_DOMAIN_SOCKET
+		// Linux has some unique behavior.  We use a random cookie as a prefix to our
+		// shared port "directory" in the abstract Unix namespace.
+	char *keybuf = Condor_Crypt_Base::randomHexKey(32);
+	if (keybuf == NULL) {
+		EXCEPT("SharedPortEndpoint: Unable to create a secure shared port cookie.\n");
+	}
+	result = keybuf;
+	free(keybuf);
+	keybuf = NULL;
+#elif defined(WIN32)
+	return;
+#else
+	if( !param(result, "DAEMON_SOCKET_DIR") ) {
+		EXCEPT("DAEMON_SOCKET_DIR must be defined");
+	}
+		// If set to "auto", we want to make sure that $(DAEMON_SOCKET_DIR)/collector or $(DAEMON_SOCKET_DIR)/15337_9022_123456 isn't more than 108 characters
+		// Hence we assume the longest valid shared port ID is 18 characters.
+	if (result == "auto") {
+		struct sockaddr_un named_sock_addr;
+		const unsigned max_len = sizeof(named_sock_addr.sun_path)-1;
+		char * default_name = expand_param("$(LOCK)/daemon_sock");
+		if (strlen(default_name) + 18 > max_len) {
+			TemporaryPrivSentry tps(PRIV_CONDOR);
+				// NOTE we force the use of /tmp here - not using the HTCondor library routines;
+				// this is because HTCondor will look up the param TMP_DIR, which might also be
+				// a long directory path.  We really want /tmp.
+			char dirname_template[] = "/tmp/condor_shared_port_XXXXXX";
+			const char *dirname = mkdtemp(dirname_template);
+			if (dirname == NULL) {
+				EXCEPT("SharedPortEndpoint: Failed to create shared port directory: %s (errno=%d)\n", strerror(errno), errno);
+			}
+			m_created_shared_port_dir = true;
+			result = dirname;
+			dprintf(D_ALWAYS, "Default DAEMON_SOCKET_DIR too long; using %s instead.  Please shorten the length of $(LOCK)\n", dirname);
+		} else {
+			result = default_name;
+		}
+		free( default_name );
+	}
+#endif
+#ifndef WIN32
+	setenv("CONDOR_PRIVATE_SHARED_PORT_COOKIE", result.c_str(), 1);
+#endif
+}
+
+
+bool
+SharedPortEndpoint::GetAltDaemonSocketDir(std::string &result)
+{
+#ifndef WIN32
+	if (!param(result, "DAEMON_SOCKET_DIR") )
+	{
+		EXCEPT("DAEMON_SOCKET_DIR must be defined");
+	}
+		// If set to "auto", we want to make sure that $(DAEMON_SOCKET_DIR)/collector or $(DAEMON_SOCKET_DIR)/15337_9022_123456 isn't more than 108 characters
+	std::string default_name;
+	if (result == "auto") {
+		char *tmp = expand_param("$(LOCK)/daemon_sock");
+		default_name = tmp;
+		free(tmp);
+	} else {
+		default_name = result;
+	}
+	struct sockaddr_un named_sock_addr;
+	const unsigned max_len = sizeof(named_sock_addr.sun_path)-1;
+	if (strlen(default_name.c_str()) + 18 > max_len) {
+		dprintf(D_FULLDEBUG, "WARNING: DAEMON_SOCKET_DIR %s setting is too long.\n", default_name.c_str());
+		return false;
+	}
+	result = default_name;
+	return true;
+#endif
+	return false;
+}
+
+
+bool
+SharedPortEndpoint::GetDaemonSocketDir(std::string &result)
 {
 #ifdef WIN32
 	result = WINDOWS_DAEMON_SOCKET_DIR;
 #else
-	if( !param(result,"DAEMON_SOCKET_DIR") ) {
-		EXCEPT("DAEMON_SOCKET_DIR must be defined");
+	const char * known_dir = getenv("CONDOR_PRIVATE_SHARED_PORT_COOKIE");
+	if (known_dir == NULL) {
+		dprintf(D_FULLDEBUG, "No shared_port cookie available; will fall back to using on-disk $(DAEMON_SOCKET_DIR)\n");
+		return false;
 	}
+	result = known_dir;
 #endif
+	return true;
 }
+
