@@ -25,6 +25,10 @@
 #include "selector.h"
 #include <queue>
 
+#ifdef LINUX
+#define USE_ABSTRACT_DOMAIN_SOCKET 1
+#endif
+
 // SharedPortEndpoint receives connections forwarded from SharedPortServer.
 // This enables Condor daemons to share a single network port.
 
@@ -45,11 +49,15 @@ class SharedPortEndpoint: Service {
 
 	void StopListener();
 
-		// returns a contact string suitable for connecting to the
+		// returns the best contact string suitable for connecting to the
 		// SharedPortServer that will forward the connection to us.
 		// May return NULL if remote address cannot be determined.
 	char const *GetMyRemoteAddress();
 
+		// Returns a vector of contact strings suitable for connecting
+		// to the shared port server.  May be empty!
+	const std::vector<Sinful> &GetMyRemoteAddresses();
+
 		// Force an immediate reload of the shared port server's
 		// address.
 	void ReloadSharedPortServerAddr();
@@ -117,17 +125,26 @@ class SharedPortEndpoint: Service {
 
 	void DoListenerAccept(ReliSock *return_remote_sock);
 
-	static void paramDaemonSocketDir(MyString &result);
+	static void InitializeDaemonSocketDir();
+	static bool GetDaemonSocketDir(std::string &result);
+	static bool GetAltDaemonSocketDir(std::string &result);
+	static bool CreatedSharedPortDirectory() {return m_created_shared_port_dir;}
 
  private:
+	static bool m_created_shared_port_dir;
+	static bool m_initialized_socket_dir;
+
+	bool m_is_file_socket; // Set to false if we are using a Linux abstract socket.
 	bool m_listening;
 	bool m_registered_listener;
 	MyString m_socket_dir;// dirname of socket
 	MyString m_full_name; // full path of socket
 	MyString m_local_id;  // basename of socket
 	MyString m_remote_addr;  // SharedPortServer addr with our local_id inserted
+	std::vector<Sinful> m_remote_addrs;
 	MyString m_local_addr;
 	int m_retry_remote_addr_timer;
+	int m_max_accepts;
 #ifdef WIN32
 	//Lock for accessing the queue that holds onto the received data structures.
 	CRITICAL_SECTION received_lock;
@@ -153,6 +170,8 @@ class SharedPortEndpoint: Service {
 #endif
 	int m_socket_check_timer;
 
+	void EnsureInitRemoteAddress();
+
 	int HandleListenerAccept( Stream * stream );
 #ifndef WIN32
 	void ReceiveSocket( ReliSock *local_sock, ReliSock *return_remote_sock );
