@@ -34,12 +34,11 @@
 #include "condor_debug.h"
 #include "get_port_range.h"
 #include "condor_netdb.h"
-#include "daemon_core_sock_adapter.h"
 #include "selector.h"
 #include "authentication.h"
 #include "condor_sockfunc.h"
-#include "condor_ipv6.h"
 #include "condor_config.h"
+#include "condor_sinful.h"
 
 #if defined(WIN32)
 // <winsock2.h> already included...
@@ -66,8 +65,6 @@ void dprintf ( int flags, Sock & sock, const char *fmt, ... )
     va_end( args );
 }
 
-DaemonCoreSockAdapterClass daemonCoreSockAdapter;
-
 unsigned int Sock::m_nextUniqueId = 1;
 
 Sock::Sock() : Stream() {
@@ -148,7 +145,7 @@ Sock::Sock(const Sock & orig) : Stream() {
 	SOCKET DuplicateSock = INVALID_SOCKET;
 	WSAPROTOCOL_INFO sockstate;
 
-	dprintf(D_FULLDEBUG,"About to sock duplicate, old sock=%X new sock=%X state=%d\n",
+	dprintf(D_NETWORK,"About to sock duplicate, old sock=%X new sock=%X state=%d\n",
 		orig._sock,_sock,_state);
 
 	if (WSADuplicateSocket(orig._sock,GetCurrentProcessId(),&sockstate) == 0)
@@ -163,7 +160,7 @@ Sock::Sock(const Sock & orig) : Stream() {
 	}
 	// if made it here, successful duplication
 	_sock = DuplicateSock;
-	dprintf(D_FULLDEBUG,"Socket duplicated, old sock=%X new sock=%X state=%d\n",
+	dprintf(D_NETWORK,"Socket duplicated, old sock=%X new sock=%X state=%d\n",
 		orig._sock,_sock,_state);
 #else
 	// Unix
@@ -337,43 +334,109 @@ int Sock::assign(
 	int socket_fd = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, 
 					  FROM_PROTOCOL_INFO, pProtoInfo, 0, 0);
 
-	return assign( socket_fd );
+	return assignSocket( socket_fd );
 }
 #endif
 
 #ifdef WIN32
+
+#ifndef SIO_BASE_HANDLE
+  #define SIO_BASE_HANDLE _WSAIOR(IOC_WS2,34)
+#endif
+
 int Sock::set_inheritable( int flag )
 {
 	// on unix, all sockets are always inheritable by a child process.
 	// but on Win32, each individual socket has a flag that says if it can
 	// or cannot be inherited by a child.  this method effectively sets
-	// that flag (by duplicating the socket).  
+	// that flag.
 	// pass flag as "TRUE" to make this socket inheritable, "FALSE" to make
 	// it private to this process.
 	// Returns TRUE on sucess, FALSE on failure.
 
+	SOCKET RealKernelSock;
 	SOCKET DuplicateSock;
 
 	if ( (flag != TRUE) && (flag != FALSE) )
 		return FALSE;	// flag must be either TRUE or FALSE...
 
-	if (!DuplicateHandle(GetCurrentProcess(),
-        (HANDLE)_sock,
-        GetCurrentProcess(),
-        (HANDLE*)&DuplicateSock,
-        0,
-        flag, // inheritable flag
-        DUPLICATE_SAME_ACCESS)) {
-			// failed to duplicate
-			dprintf(D_ALWAYS,"ERROR: DuplicateHandle() failed "
-			                 "in Sock:set_inheritable(%d), error=%d\n"
-				  ,flag,GetLastError());
-			closesocket(DuplicateSock);
-			return FALSE;
+	DWORD BytesReturned = 0;  // useless pointer needed for winsock call
+
+	// If the Winsock stack on this machine is configured with a
+	// Layered Service Provider (LSP) extention, it is possible that
+	// _sock is not a "real" kernel socket handle but is instead an LSP
+	// handle.  LSPs implementations may be sloppy and fail to implement
+	// socket inheritance features. So our algorithm is as follows -
+	// First, we get the real kernel handle, which can be obtained on
+	// recent Windows versions (Vista and newer) via an ioctl call.
+	// If it turns out that _sock is a real kernel handle, we
+	// can likely just call SetHandleInformation() which is Microsoft's
+	// current recommendation for changing the inheritance of a socket.
+	// But if _sock is from an LSP, of SetHandleInformation() fails for
+	// some reason, then we duplicate the kernel handle into a new
+	// socket that has the inheritance we want, and close the original _sock.
+	// See gitrac ticklet #304.
+
+	int rc = ::WSAIoctl(_sock, SIO_BASE_HANDLE, NULL, 0, (void *) &RealKernelSock,
+		sizeof (RealKernelSock), &BytesReturned, NULL, NULL);
+	if (rc == SOCKET_ERROR)
+	{
+		int err = WSAGetLastError();
+		dprintf(D_NETWORK,
+			"Failed SIO_BASE_HANDLE in set_inheritable on sock %d to %d (err=%d, %s)",
+			_sock, flag, err, GetLastErrorString(err) );
+		// Well, we failed to get the "real" handle for whatever reason.
+		// We may as well try to move forward with the original _sock handle.
+		RealKernelSock = _sock;
+	}
+
+	// If _sock is a real kernel sock (not an LSB), we can likely do the deed with
+	// a quick call to SetHandleInformation().
+	bool sethandle_worked = false;
+	if (RealKernelSock == _sock) {   // if true, we are LSB free
+		if (!SetHandleInformation(
+				(HANDLE) _sock,
+				HANDLE_FLAG_INHERIT,
+				flag ? HANDLE_FLAG_INHERIT : 0) )
+		{
+			int err = WSAGetLastError();
+			dprintf(D_NETWORK,
+				"Failed to set_inheritable on sock %d to %d (err=%d, %s)",
+				_sock, flag, err, GetLastErrorString(err) );
+		} else {
+			sethandle_worked = true;
+		}
+	}
+
+	// Well, we failed to set the inheritable flag by using
+	// Microsoft's recommendaded API of SetHandleInformation().
+	// In this case we will fall back to using our old method
+	// of duplicating the handle.  This old method is no longer
+	// recommended by Microsoft, but hey, we tried and failed to do
+	// it the "right way" (maybe it failed cuz an old/customized Winsock,
+	// or it failed because we never attempted it because _sock is an LSP
+	// sock, not a kernel sock).
+	if (sethandle_worked == false) {
+		if (!DuplicateHandle(GetCurrentProcess(),
+			(HANDLE)RealKernelSock,  // dup kernel sock here, not LSB sock
+			GetCurrentProcess(),
+			(HANDLE*)&DuplicateSock,
+			0,
+			flag, // inheritable flag
+			DUPLICATE_SAME_ACCESS)) {
+				// failed to duplicate
+				dprintf(D_ALWAYS,"ERROR: DuplicateHandle() failed "
+								 "in Sock:set_inheritable(%d), error=%d\n"
+					  ,flag,GetLastError());
+				return FALSE;
+		}
+		// if made it here, successful duplication; replace original.
+		// note we close _sock, not RealKernelSock, since _sock is
+		// the handle we obtained via socket() - if it is from an LSP, then the
+		// LSP is responsible for closing RealKernelSock.
+		closesocket(_sock);
+		_sock = DuplicateSock;
 	}
-	// if made it here, successful duplication; replace original
-	closesocket(_sock);
-	_sock = DuplicateSock;
 
 	return TRUE;
 }
@@ -419,34 +482,97 @@ int Sock::move_descriptor_up()
 	return TRUE;
 }
 
-int Sock::assign(SOCKET sockd)
-{
-	condor_protocol proto = CP_IPV4;
-	if (_condor_is_ipv6_mode())
-		proto = CP_IPV6;
-	return assign(proto, sockd);
+//
+// Moving all assignments of INVALID_SOCKET into their own function
+// dramatically simplifies the logic for assign()ing sockets without
+// an explicit protoco argument.
+//
+
+int Sock::assignInvalidSocket() {
+	ABEND( _who.is_valid() );
+	return assignSocket( _who.get_protocol(), INVALID_SOCKET );
 }
 
-int Sock::assign(condor_protocol proto, SOCKET sockd)
-{
-	int		my_type = SOCK_DGRAM;
+int Sock::assignInvalidSocket( condor_protocol proto ) {
+	return assignSocket( proto, INVALID_SOCKET );
+}
 
-	if (_state != sock_virgin) return FALSE;
+int Sock::assignSocket( SOCKET sockd ) {
+	ABEND( sockd != INVALID_SOCKET );
 
-	if (sockd != INVALID_SOCKET){
-		_sock = sockd;		/* Could we check for correct protocol ? */
-		/* should we check that sockd matches to IPv6 mode? */
+	condor_sockaddr sockAddr;
+	ABEND( condor_getsockname( sockd, sockAddr ) == 0 );
+	condor_protocol sockProto = sockAddr.get_protocol();
+
+	if( _who.is_valid() ) {
+		condor_protocol objectProto = _who.get_protocol();
+		// If we're accepting a socket via shared port as part of a CCB
+		// callback, then it's legitimate for the object protocol and the
+		// socket protocol to differ.
+		if( sockProto == PF_UNIX && objectProto != PF_UNIX ) {
+			// dprintf( D_ALWAYS, "[tlm] assigning PF_UNIX socket to object connecting to %s.\n", get_connect_addr() );
+			Sinful s( get_connect_addr() );
+			ABEND( s.getCCBContact() != NULL && s.getSharedPortID() != NULL );
+		} else {
+			// dprintf( D_ALWAYS, "[tlm] assigning socket of protocol %d to object of protocol %d.\n", sockProto, objectProto );
+			ABEND( sockProto == objectProto );
+		}
+	}
+
+	return assignSocket( sockProto, sockd );
+}
+
+//
+// Domain sockets don't have a useful protocol or address.
+//
+int Sock::assignDomainSocket( SOCKET sockd ) {
+	ABEND( sockd != INVALID_SOCKET );
+
+	_sock = sockd;
+	_state = sock_assigned;
+	_who.clear();
+
+	if( _timeout > 0 ) {
+		timeout_no_timeout_multiplier( _timeout );
+	}
+
+	// This wasn't here originally, but it seems like it should be --
+	// clearing the caches is at worst a performance hit, but it may
+	// also be a correctness fix.
+	addr_changed();
+	return TRUE;
+}
+
+int Sock::assignSocket( condor_protocol proto, SOCKET sockd ) {
+	if( _state != sock_virgin ) { return FALSE; }
+
+	if( sockd != INVALID_SOCKET ) {
+		condor_sockaddr sockAddr;
+		ABEND( condor_getsockname( sockd, sockAddr ) == 0 );
+		condor_protocol sockProto = sockAddr.get_protocol();
+		ABEND( sockProto == proto );
+
+		_sock = sockd;
 		_state = sock_assigned;
-
 		_who.clear();
-		condor_getpeername(_sock, _who);
+		condor_getpeername( _sock, _who );
 
-		if ( _timeout > 0 ) {
+		if( _timeout > 0 ) {
 			timeout_no_timeout_multiplier( _timeout );
 		}
+
+		// This wasn't here originally, but it seems like it should be --
+		// clearing the caches is at worst a performance hit, but it may
+		// also be a correctness fix.
+		addr_changed();
 		return TRUE;
 	}
 
+	//
+	// At some point, we should probably rename assignInvalid*() to create()
+	// and move this implementation there.
+	//
+
 	int af_type = 0;
 	switch(proto) {
 		case CP_IPV4: af_type = AF_INET; break;
@@ -454,6 +580,7 @@ int Sock::assign(condor_protocol proto, SOCKET sockd)
 		default: ASSERT(false);
 	}
 
+	int my_type = 0;
 	switch(type()){
 		case safe_sock:
 			my_type = SOCK_DGRAM;
@@ -530,18 +657,23 @@ Sock::bindWithin(condor_protocol proto, const int low_port, const int high_port,
 	bool bind_all = (bool)_condor_bind_all_interfaces();
 
 	// Use hash function with pid to get the starting point
-    struct timeval curTime;
+	// int pid = (int) getpid();
+
+	// lets use time as the starting point instead...
+	long randomish_val;
 #ifndef WIN32
-    (void) gettimeofday(&curTime, NULL);
+	struct timeval curTime;
+	(void) gettimeofday(&curTime, NULL);
+	randomish_val = curTime.tv_usec;
 #else
-	// Win32 does not have gettimeofday, sigh.
-	curTime.tv_usec = ::GetTickCount() % 1000000;
+	LARGE_INTEGER li;
+	::QueryPerformanceCounter(&li);
+	randomish_val = (int)li.LowPart;
 #endif
 
-	// int pid = (int) getpid();
 	int range = high_port - low_port + 1;
 	// this line must be changed to use the hash function of condor
-	int start_trial = low_port + (curTime.tv_usec * 73/*some prime number*/ % range);
+	int start_trial = low_port + (randomish_val * 73/*some prime number*/ % range);
 
 	int this_trial = start_trial;
 	do {
@@ -553,11 +685,11 @@ Sock::bindWithin(condor_protocol proto, const int low_port, const int high_port,
 			addr.set_protocol(proto);
 			addr.set_addr_any();
 		} else {
-			addr = get_local_ipaddr();
-			// what if the socket type does not match?
-			// e.g. addr is ipv6 but ipv6 mode is not turned on?
-			if (addr.is_ipv4() && proto==CP_IPV6)
-				addr.convert_to_ipv6();
+			addr = get_local_ipaddr(proto);
+			if(!addr.is_valid()) {
+				dprintf(D_ALWAYS, "Asked to bind to a single %s interface, but cannot find a suitable interface\n", condor_protocol_to_str(proto).Value());
+				return FALSE;
+			}
 		}
 		addr.set_port((unsigned short)this_trial++);
 
@@ -603,17 +735,12 @@ Sock::bindWithin(condor_protocol proto, const int low_port, const int high_port,
 	return FALSE;
 }
 
-int Sock::bind(bool outbound, int port, bool loopback)
-{
-	condor_protocol proto = CP_IPV4;
-	if(_condor_is_ipv6_mode()) {
-		proto = CP_IPV6;
-	}
-	return bind(proto, outbound, port, loopback);
-}
-
 int Sock::bind(condor_protocol proto, bool outbound, int port, bool loopback)
 {
+	if( proto <= CP_INVALID_MIN || proto >= CP_INVALID_MAX ) {
+		EXCEPT( "Unknown protocol (%d) in Sock::bind(); aborting.", proto );
+	}
+
 	condor_sockaddr addr;
 	int bind_return_value;
 
@@ -626,7 +753,7 @@ int Sock::bind(condor_protocol proto, bool outbound, int port, bool loopback)
     }
 
 	// if stream not assigned to a sock, do it now	*/
-	if (_state == sock_virgin) assign(proto);
+	if (_state == sock_virgin) assignInvalidSocket( proto );
 
 	if (_state != sock_assigned) {
 		dprintf(D_ALWAYS, "Sock::bind - _state is not correct\n");
@@ -677,9 +804,11 @@ int Sock::bind(condor_protocol proto, bool outbound, int port, bool loopback)
 		} else if( (bool)_condor_bind_all_interfaces() ) {
 			addr.set_addr_any();
 		} else {
-			addr = get_local_ipaddr();
-			if (addr.is_ipv4() && proto==CP_IPV6)
-				addr.convert_to_ipv6();
+			addr = get_local_ipaddr(proto);
+			if(!addr.is_valid()) {
+				dprintf(D_ALWAYS, "Asked to bind to a single %s interface, but cannot find a suitable interface\n", condor_protocol_to_str(proto).Value());
+				return FALSE;
+			}
 		}
 		addr.set_port((unsigned short)port);
 
@@ -744,11 +873,6 @@ int Sock::bind(condor_protocol proto, bool outbound, int port, bool loopback)
 	return TRUE;
 }
 
-bool Sock::bind_to_loopback(bool outbound,int port)
-{
-	return bind(outbound,port,true) == TRUE;
-}
-
 bool Sock::set_keepalive()
 {
 	bool result = true;
@@ -852,7 +976,7 @@ bool Sock::set_keepalive()
 	alive.keepaliveinterval = 5 * 1000;
 
 	// if stream not assigned to a sock, do it now before WSAIoctl calls.
-	if (_state == sock_virgin) assign();
+	if (_state == sock_virgin) assignInvalidSocket();
 
 	DWORD BytesReturned = 0;  // useless pointer needed for winsock call
 
@@ -944,6 +1068,14 @@ int Sock::setsockopt(int level, int optname, const void* optval, int optlen)
 {
 	ASSERT(_state != sock_virgin); 
 
+		// Don't bother with TCP options on Unix sockets.
+#ifndef WIN32
+        if ((_who.to_storage().ss_family == AF_LOCAL) && (level == IPPROTO_TCP))
+	{
+		return true;
+	}
+#endif
+
 	/* cast optval from void* to char*, as some platforms (Windows!) require this */
 	if(::setsockopt(_sock, level, optname, static_cast<const char*>(optval), optlen) < 0)
 	{
@@ -978,6 +1110,85 @@ bool Sock::guess_address_string(char const* host, int port, condor_sockaddr& add
 	return true;
 }
 
+bool Sock::chooseAddrFromAddrs( char const * host, std::string & addr ) {
+	//
+	// If host is a Sinful string and contains an addrs parameter,
+	// choose one of the listed addresses and rewrite host to match.
+	// Otherwise, execute the old code.
+	//
+	// Note that private networks are handled in Daemon::New_addr(),
+	// which, if it finds a match, will completely rewrite the sinful,
+	// discarding everything except the match as the primary.  If the
+	// network names match, but there's no private address, it instead
+	// removes the CCB information.  In that case, the sinful string
+	// will still have an addrs attribute.  Arguably, it should also
+	// remove addrs, since there's by definition only one private
+	// (and networks are protocol-separated); in practice, that probably
+	// won't matter much.
+	//
+	// The Sinful s must not go out of scope until /after/ the call to
+	// special_connect(), since the new value of host may point into it.
+	//
+	Sinful s( host );
+	if(! (s.valid() && s.hasAddrs()) ) { return false; }
+
+	condor_sockaddr candidate;
+	std::vector< condor_sockaddr > * v = s.getAddrs();
+
+	// In practice, all C++03 implementations are stable with respect
+	// to insertion order; the  C++11 standard requires that they are.
+	// We have a unit test for the former.
+	std::multimap< int, condor_sockaddr > sortedByDesire;
+
+	// If we don't multiply by -1 and instead use rbegin()/rend(),
+	// then addresses of the same desirability will be checked in
+	// the reverse order.
+	dprintf( D_HOSTNAME, "Found address %lu candidates:\n", v->size() );
+	for( unsigned i = 0; i < v->size(); ++i ) {
+		condor_sockaddr c = (*v)[i];
+		int d = -1 * c.desirability();
+		sortedByDesire.insert(std::make_pair( d, c ));
+		dprintf( D_HOSTNAME, "\t%d\t%s\n", d, c.to_ip_and_port_string().c_str() );
+	}
+
+	bool foundAddress = false;
+	std::multimap< int, condor_sockaddr >::const_iterator iter;
+	for( iter = sortedByDesire.begin(); iter != sortedByDesire.end(); ++iter ) {
+		candidate = (* iter).second;
+
+		// Assume that we "have" any protocol that's enabled.  It turns
+		// out that anything else (including considering the desirability
+		// of our interfaces) gets really complicated.  Instead, we
+		// document that ENABLE_IPV6 should be false unless you have a
+		// public IPv6 address (or one which everyone in your pool can
+		// otherwise reach).
+		dprintf( D_HOSTNAME, "Considering address candidate %s.\n", candidate.to_ip_and_port_string().c_str() );
+		if(( candidate.is_ipv4() && param_boolean( "ENABLE_IPV4", true ) ) ||
+			( candidate.is_ipv6() && param_boolean( "ENABLE_IPV6", false ) )) {
+			dprintf( D_HOSTNAME, "Found compatible candidate %s.\n", candidate.to_ip_and_port_string().c_str() );
+			foundAddress = true;
+			break;
+		}
+	}
+	delete v;
+
+	if(! foundAddress) {
+		dprintf( D_ALWAYS, "Sock::do_connect() unable to locate address of a compatible protocol in Sinful string '%s'.\n", host );
+		return FALSE;
+	}
+
+	// Change the "primary" address.
+	s.setHost( candidate.to_ip_string().c_str() );
+	s.setPort( candidate.get_port() );
+	addr = s.getSinful();
+
+	set_connect_addr( addr.c_str() );
+	_who = candidate;
+	addr_changed();
+
+	return true;
+}
+
 int Sock::do_connect(
 	char const	*host,
 	int		port,
@@ -986,23 +1197,25 @@ int Sock::do_connect(
 {
 	if (!host || port < 0) return FALSE;
 
-	_who.clear();
-	if (!guess_address_string(host, port, _who)) {
-		return FALSE;
-	}
-
-	if (_condor_is_ipv6_mode() && _who.is_ipv4())
-		_who.convert_to_ipv6();
+	std::string addr;
+	if( chooseAddrFromAddrs( host, addr ) ) {
+		host = addr.c_str();
+	} else {
+		_who.clear();
+		if (!guess_address_string(host, port, _who)) {
+			return FALSE;
+		}
 
 		// current code handles sinful string and just hostname differently.
 		// however, why don't we just use sinful string at all?
-	if (host[0] == '<') {
-		set_connect_addr(host);
-	}
-	else { // otherwise, just use ip string.
-		set_connect_addr(_who.to_ip_string().Value());
-	}
-    addr_changed();
+		if (host[0] == '<') {
+			set_connect_addr(host);
+		}
+		else { // otherwise, just use ip string.
+			set_connect_addr(_who.to_ip_string().Value());
+		}
+    	addr_changed();
+    }
 
 	// now that we have set _who (useful for getting informative
 	// peer_description), see if we should do a reverse connect
@@ -1019,7 +1232,7 @@ int Sock::do_connect(
 		/* assigned to the stream if needed		*/
 		/* TRUE means this is an outgoing connection */
 	if (_state == sock_virgin || _state == sock_assigned) {
-		bind(true);
+		bind( _who.get_protocol(), true, 0, false );
 	}
 
 	if (_state != sock_bound) return FALSE;
@@ -1446,7 +1659,7 @@ Sock::cancel_connect()
 	_state = sock_virgin;
 		
 		// now create a new socket
-	if (assign() == FALSE) {
+	if (assignInvalidSocket() == FALSE) {
 		dprintf(D_ALWAYS,
 			"assign() failed after a failed connect!\n");
 		connect_state.connect_refused = true; // better give up
@@ -1455,7 +1668,7 @@ Sock::cancel_connect()
 
 	// finally, bind the socket
 	/* TRUE means this is an outgoing connection */
-	if( !bind(true) ) {
+	if( ! bind( _who.get_protocol(), true, 0, false ) ) {
 		connect_state.connect_refused = true; // better give up
 	}
 
@@ -1515,7 +1728,7 @@ bool Sock::test_connection()
 #else
 		setConnectFailureErrno(errno,"getsockopt");
 #endif
-        dprintf(D_ALWAYS, "Sock::test_connection - getsockopt failed\n");
+        dprintf(D_NETWORK, "Sock::test_connection - getsockopt failed\n");
         return false;
     }
     // return result
@@ -1536,13 +1749,19 @@ int Sock::close()
 
 	if (_state == sock_virgin) return FALSE;
 
-	if (type() == Stream::reli_sock && IsDebugLevel(D_NETWORK)) {
-		dprintf( D_NETWORK, "CLOSE %s fd=%d\n", 
+	if (IsDebugLevel(D_NETWORK) && _sock != INVALID_SOCKET) {
+		dprintf( D_NETWORK, "CLOSE %s %s fd=%d\n",
+						type() == Stream::reli_sock ? "TCP" : "UDP",
 						sock_to_string(_sock), _sock );
 	}
 
 	if ( _sock != INVALID_SOCKET ) {
-		if (::closesocket(_sock) < 0) return FALSE;
+		if (::closesocket(_sock) < 0) {
+			dprintf( D_NETWORK, "CLOSE FAILED %s %s fd=%d\n",
+						type() == Stream::reli_sock ? "TCP" : "UDP",
+						sock_to_string(_sock), _sock );
+			return FALSE;
+		}
 	}
 
 	_sock = INVALID_SOCKET;
@@ -1553,7 +1772,15 @@ int Sock::close()
 	connect_state.host = NULL;
 	_who.clear();
     addr_changed();
-	
+
+	// we need to reset the crypto keys
+	set_MD_mode(MD_OFF);
+	set_crypto_key(false, NULL);
+
+	// we also need to reset the FQU
+	setFullyQualifiedUser(NULL);
+	setTriedAuthentication(false);
+
 	return TRUE;
 }
 
@@ -1605,6 +1832,12 @@ Sock::bytes_available_to_read()
 	return ret_val;
 }
 
+	// NOTE NOTE: this returns true in the same cases the select() syscall
+	// would return true.  This includes BOTH when a message is ready (such
+	// as a complete CEDAR message - if there's just an incomplete message,
+	// calling this will consume any bytes available on the socket) AND
+	// when the reli sock has been closed.  Take note that at least the CCB
+	// depends on this returning true when the reli sock is closed.
 bool
 Sock::readReady() {
 	Selector selector;
@@ -1619,11 +1852,20 @@ Sock::readReady() {
 		return true;
 	}
 
-	selector.add_fd( _sock, Selector::IO_READ );
-	selector.set_timeout( 0 );
-	selector.execute();
+	if (type() == Stream::safe_sock)
+	{
+		selector.add_fd( _sock, Selector::IO_READ );
+		selector.set_timeout( 0 );
+		selector.execute();
 
-	return selector.has_ready();
+		return selector.has_ready();
+	}
+	else if (type() == Stream::reli_sock)
+	{
+		ReliSock *relisock = static_cast<ReliSock*>(this);
+		return relisock->is_closed();
+	}
+	return false;
 }
 
 int
@@ -1665,8 +1907,8 @@ Sock::timeout_no_timeout_multiplier(int sec)
 		int fcntl_flags;
 		if ( (fcntl_flags=fcntl(_sock, F_GETFL)) < 0 )
 			return -1;
-		fcntl_flags &= ~O_NONBLOCK;	// reset blocking mode
-		if ( fcntl(_sock,F_SETFL,fcntl_flags) == -1 )
+			// reset blocking mode
+		if ( ((fcntl_flags & O_NONBLOCK) == O_NONBLOCK) && fcntl(_sock,F_SETFL,fcntl_flags & ~O_NONBLOCK) == -1 )
 			return -1;
 #endif
 	} else {
@@ -1681,8 +1923,8 @@ Sock::timeout_no_timeout_multiplier(int sec)
 			int fcntl_flags;
 			if ( (fcntl_flags=fcntl(_sock, F_GETFL)) < 0 )
 				return -1;
-			fcntl_flags |= O_NONBLOCK;	// set nonblocking mode
-			if ( fcntl(_sock,F_SETFL,fcntl_flags) == -1 )
+				// set nonblocking mode
+			if ( ((fcntl_flags & O_NONBLOCK) == 0) && fcntl(_sock,F_SETFL,fcntl_flags | O_NONBLOCK) == -1 )
 				return -1;
 #endif
 		}
@@ -1948,7 +2190,7 @@ char * Sock::serialize(char *buf)
 	// here we want to restore our state from the incoming buffer
 	i = sscanf(buf,"%u*%d*%d*%d*%lu*%lu*%n",&passed_sock,(int*)&_state,&_timeout,&tried_authentication,(unsigned long *)&fqulen,(unsigned long *)&verstring_len,&pos);
 	if (i!=6) {
-		EXCEPT("Failed to parse serialized socket information (%d,%d): '%s'\n",i,pos,buf);
+		EXCEPT("Failed to parse serialized socket information (%d,%d): '%s'",i,pos,buf);
 	}
 	buf += pos;
 
@@ -1962,7 +2204,7 @@ char * Sock::serialize(char *buf)
 	free(fqubuf);
 	buf += fqulen;
 	if( *buf != '*' ) {
-		EXCEPT("Failed to parse serialized socket fqu (%lu): '%s'\n",(unsigned long)fqulen,buf);
+		EXCEPT("Failed to parse serialized socket fqu (%lu): '%s'",(unsigned long)fqulen,buf);
 	}
 	buf++;
 
@@ -1983,7 +2225,7 @@ char * Sock::serialize(char *buf)
 	free( verstring );
 	buf += verstring_len;
 	if( *buf != '*' ) {
-		EXCEPT("Failed to parse serialized peer version string (%lu): '%s'\n",(unsigned long)verstring_len,buf);
+		EXCEPT("Failed to parse serialized peer version string (%lu): '%s'",(unsigned long)verstring_len,buf);
 	}
 	buf++;
 
@@ -2197,6 +2439,14 @@ Sock::my_addr()
 	return addr;
 }
 
+condor_sockaddr
+Sock::my_addr_wildcard_okay() 
+{
+	condor_sockaddr addr;
+	condor_getsockname(_sock, addr);
+	return addr;
+}
+
 const char *
 Sock::my_ip_str()
 {
@@ -2512,12 +2762,12 @@ bool Sock :: is_hdr_encrypt(){
 	return FALSE;
 }
 
-int Sock :: authenticate(KeyInfo *&, const char * /* methods */, CondorError* /* errstack */, int /*timeout*/, char ** /*method_used*/)
+int Sock :: authenticate(KeyInfo *&, const char * /* methods */, CondorError* /* errstack */, int /*timeout*/, bool /*non_blocking*/, char ** /*method_used*/)
 {
 	return -1;
 }
 
-int Sock :: authenticate(const char * /* methods */, CondorError* /* errstack */, int /*timeout*/)
+int Sock :: authenticate(const char * /* methods */, CondorError* /* errstack */, int /*timeout*/, bool /*non_blocking*/)
 {
 	/*
 	errstack->push("AUTHENTICATE", AUTHENTICATE_ERR_NOT_BUILT,
@@ -2526,6 +2776,11 @@ int Sock :: authenticate(const char * /* methods */, CondorError* /* errstack */
 	return -1;
 }
 
+int Sock :: authenticate_continue(CondorError* /* errstack */, bool /*non_blocking*/, char ** /*method_used*/)
+{
+	return -1;
+}
+
 bool Sock :: is_encrypt()
 {
     return FALSE;
