@@ -168,7 +168,7 @@ JobRouter::GetInstanceLock() {
 
 	lock->setBlocking(FALSE);
 	if(!lock->obtain(WRITE_LOCK)) {
-		EXCEPT("Failed to get lock on %s.\n",lock_fullname.c_str());
+		EXCEPT("Failed to get lock on %s.",lock_fullname.c_str());
 	}
 }
 
@@ -242,6 +242,8 @@ JobRouter::config() {
 
 	RoutingTable *new_routes = new RoutingTable(200,hashFuncStdString,rejectDuplicateKeys);
 
+	bool merge_defaults = param_boolean("MERGE_JOB_ROUTER_DEFAULT_ADS", false);
+
 	classad::ClassAd router_defaults_ad;
 	std::string router_defaults;
 	if (param(router_defaults, PARAM_JOB_ROUTER_DEFAULTS) && ! router_defaults.empty()) {
@@ -249,12 +251,47 @@ JobRouter::config() {
 		if (router_defaults[0] != '[') {
 			router_defaults.insert(0, "[ ");
 			router_defaults.append(" ]");
+			merge_defaults = false;
 		}
+		int length = (int)router_defaults.size();
 		classad::ClassAdParser parser;
-		if ( ! parser.ParseClassAd(router_defaults, router_defaults_ad)) {
-			dprintf(D_ALWAYS|D_ERROR,"JobRouter CONFIGURATION ERROR: Disabling job routing, because failed to parse %s classad:\n%s\n",
-				PARAM_JOB_ROUTER_DEFAULTS, router_defaults.c_str());
+		int offset = 0;
+		if ( ! parser.ParseClassAd(router_defaults, router_defaults_ad, offset)) {
+			dprintf(D_ALWAYS|D_ERROR,"JobRouter CONFIGURATION ERROR: Disabling job routing, failed to parse at offset %d in %s classad:\n%s\n",
+				offset, PARAM_JOB_ROUTER_DEFAULTS, router_defaults.c_str());
 			m_enable_job_routing = false;
+		} else if (merge_defaults && (offset < length)) {
+			// whoh! we appear to have received multiple classads as a hacky way to append to the defaults ad
+			// so go ahead and parse the remaining ads and merge them into the defaults ad.
+			do {
+				// skip trailing whitespace and ] and look for an open [
+				bool parse_err = false;
+				for ( ; offset < length; ++offset) {
+					int ch = router_defaults[offset];
+					if (ch == '[') break;
+					if ( ! isspace(router_defaults[offset]) && ch != ']') {
+						parse_err = true;
+						break;
+					}
+					// TODO: skip comments?
+				}
+
+				if (offset < length && ! parse_err) {
+					classad::ClassAd other_ad;
+					if ( ! parser.ParseClassAd(router_defaults, other_ad, offset)) {
+						parse_err = true;
+					} else {
+						router_defaults_ad.Update(other_ad);
+					}
+				}
+
+				if (parse_err) {
+					m_enable_job_routing = false;
+					dprintf(D_ALWAYS|D_ERROR,"JobRouter CONFIGURATION ERROR: Disabling job routing, failed to parse at offset %d in %s ad : \n%s\n",
+							offset, PARAM_JOB_ROUTER_DEFAULTS, router_defaults.substr(offset).c_str());
+					break;
+				}
+			} while (offset < length);
 		}
 	}
 	if(!m_enable_job_routing) {
@@ -582,9 +619,15 @@ JobRouter::EvalSrcJobPeriodicExpr(RoutedJob* job)
 	bool ret_val = false;
 
 	converted_ad = job->src_ad;
+#ifdef USE_NON_MUTATING_USERPOLICY
+	user_policy.Init();
+
+	action = user_policy.AnalyzePolicy(converted_ad, PERIODIC_ONLY);
+#else
 	user_policy.Init(&converted_ad);
 
 	action = user_policy.AnalyzePolicy(PERIODIC_ONLY);
+#endif
 
 	user_policy.FiringReason(reason,reason_code,reason_subcode);
 	if ( reason == "" ) {
@@ -750,7 +793,8 @@ JobRouter::ParseRoutingEntries( std::string const &routing_string, char const *p
 				break;
 			}
 
-			dprintf(D_ALWAYS,"JobRouter CONFIGURATION ERROR: Ignoring the malformed route entry in %s, starting here: %s\n",param_name,routing_string.c_str() + this_offset);
+			dprintf(D_ALWAYS,"JobRouter CONFIGURATION ERROR: Ignoring the malformed route entry in %s, at offset %d starting here:\n%s\n",
+				param_name, this_offset, routing_string.substr(this_offset, 79).c_str());
 
 			// skip any junk and try parsing the next route in the list
 			while((int)routing_string.size() > offset && routing_string[offset] != '[') offset++;
@@ -766,7 +810,8 @@ JobRouter::ParseRoutingEntries( std::string const &routing_string, char const *p
 
 			int override = route.OverrideRoutingEntry();
 			if( override < 0 ) {
-				dprintf(D_ALWAYS,"JobRouter CONFIGURATION WARNING while parsing %s: two route entries have the same name '%s' so the second one will override the first one; if you have not already explicitly given these routes a name with name=\"blah\", you may want to give them different names.  If you just want to suppress this warning, then define OverrideRoutingEntry=True/False in the second routing entry.\n",param_name,route.Name());
+				dprintf(D_ALWAYS,"JobRouter CONFIGURATION WARNING: while parsing %s two route entries have the same name '%s' so the second one will override the first one; if you have not already explicitly given these routes a name with name=\"blah\", you may want to give them different names.  If you just want to suppress this warning, then define OverrideRoutingEntry=True/False in the second routing entry.\n",
+					param_name,route.Name());
 				override = 1;
 			}
 			if( override > 0 ) {  // OverrideRoutingEntry=true
@@ -999,7 +1044,7 @@ JobRouter::AdoptOrphans() {
 
 	constraint_tree = parser.ParseExpression(dest_jobs.c_str());
 	if(!constraint_tree) {
-		EXCEPT("JobRouter: Failed to parse orphan dest job constraint: '%s'\n",dest_jobs.c_str());
+		EXCEPT("JobRouter: Failed to parse orphan dest job constraint: '%s'",dest_jobs.c_str());
 	}
 
     query.Bind(ad_collection2);
@@ -1107,7 +1152,7 @@ JobRouter::AdoptOrphans() {
 
 	constraint_tree = parser.ParseExpression(src_jobs.c_str());
 	if(!constraint_tree) {
-		EXCEPT("JobRouter: Failed to parse orphan constraint: '%s'\n",src_jobs.c_str());
+		EXCEPT("JobRouter: Failed to parse orphan constraint: '%s'",src_jobs.c_str());
 	}
 
     query.Bind(ad_collection);
@@ -1155,20 +1200,34 @@ JobRouter::GetCandidateJobs() {
 	HashTable<std::string,std::string> constraint_list(200,hashFuncStdString,rejectDuplicateKeys);
 	std::string umbrella_constraint;
 
+	std::string dbuf("JobRouter: Checking for candidate jobs. routing table is:\n"
+		"Route Name             Submitted/Max        Idle/Max     Throttle");
+	if ( ! m_operate_as_tool) {
+		dbuf += " Recent: Started Succeeded Failed\n";
+	} else {
+		dbuf += "\n";
+	}
 	m_routes->startIterations();
 	while(m_routes->iterate(route)) {
-		dprintf(D_ALWAYS,
-		      "JobRouter (route=%s): %d submitted (max %d), %d idle (max %d), throttle: %s, recent stats: %d started, %d succeeded, %d failed.\n",
+		formatstr_cat(dbuf, "%-24s %7d/%7d %7d/%7d %8s",
 		      route->Name(),
 		      route->CurrentRoutedJobs(),
 		      route->MaxJobs(),
 		      route->CurrentIdleJobs(),
 		      route->MaxIdleJobs(),
-		      route->ThrottleDesc().c_str(),
+		      route->ThrottleDesc().c_str());
+		if ( ! m_operate_as_tool) {
+			formatstr_cat(dbuf, "         %7d %9d %6d\n",
 			  route->RecentRoutedJobs(),
 		      route->RecentSuccesses(),
-			  route->RecentFailures());
+			  route->RecentFailures()
+			);
+		} else {
+			dbuf += "\n";
+		}
 	}
+	dprintf(D_ALWAYS, "%s", dbuf.c_str());
+
 
 	if(!AcceptingMoreJobs()) return; //router is full
 
@@ -1223,7 +1282,7 @@ JobRouter::GetCandidateJobs() {
 	umbrella_constraint += m_job_router_name;
 	umbrella_constraint += "\")";
 
-	if (m_operate_as_tool || !can_switch_ids()) {
+	if (!can_switch_ids()) {
 			// We are not running as root.  Ensure that we only try to
 			// manage jobs submitted by the same user we are running as.
 
@@ -1250,11 +1309,11 @@ JobRouter::GetCandidateJobs() {
 		free(domain);
 	}
 
-	dprintf(D_FULLDEBUG,"JobRouter: umbrella constraint: %s\n",umbrella_constraint.c_str());
+	dprintf(D_FULLDEBUG,"JobRouter: Umbrella constraint: %s\n",umbrella_constraint.c_str());
 
 	constraint_tree = parser.ParseExpression(umbrella_constraint);
 	if(!constraint_tree) {
-		EXCEPT("JobRouter: Failed to parse umbrella constraint: '%s'\n",umbrella_constraint.c_str());
+		EXCEPT("JobRouter: Failed to parse umbrella constraint: %s",umbrella_constraint.c_str());
 	}
 
     query.Bind(ad_collection);
@@ -1281,6 +1340,8 @@ JobRouter::GetCandidateJobs() {
 		ad = ad_collection->GetClassAd(key);
 		ASSERT(ad);
 
+		if (m_operate_as_tool) { dprintf(D_FULLDEBUG, "JobRouter: Checking Job src=%s against all routes\n", key.c_str()); }
+
 		bool all_routes_full;
 		route = ChooseRoute(ad,&all_routes_full);
 		if(!route) {
@@ -1288,7 +1349,7 @@ JobRouter::GetCandidateJobs() {
 				dprintf(D_FULLDEBUG,"JobRouter: all routes are full (%d managed jobs).  Skipping further searches for candidate jobs.\n",NumManagedJobs());
 				break;
 			}
-			dprintf(D_FULLDEBUG,"JobRouter (src=%s): no route found\n",key.c_str());
+			dprintf(D_FULLDEBUG,"JobRouter: no route found for src=%s\n",key.c_str());
 			continue;
 		}
 
@@ -1311,7 +1372,7 @@ JobRouter::GetCandidateJobs() {
 		dprintf(D_FULLDEBUG,"JobRouter DEBUG (%s): combined = %s\n",job->JobDesc().c_str(),ClassAdToString(&job->src_ad).c_str());
 		*/
 
-		dprintf(D_FULLDEBUG,"JobRouter (%s): found candidate job\n",job->JobDesc().c_str());
+		dprintf(D_FULLDEBUG,"JobRouter: Found candidate job %s\n",job->JobDesc().c_str());
 		AddJob(job);
 		++cJobsAdded;
 
@@ -1340,6 +1401,7 @@ JobRouter::ChooseRoute(classad::ClassAd *job_ad,bool *all_routes_full) {
 
 		if(mad.EvaluateAttrBool("RightMatchesLeft", match) && match) {
 			matches.push_back(route);
+			if (m_operate_as_tool) { dprintf(D_FULLDEBUG, "JobRouter: \tRoute Matches: %s\n", route->Name()); }
 		}
 
 		mad.RemoveLeftAd();
@@ -1683,7 +1745,7 @@ static bool ClassAdHasDirtyAttributes(classad::ClassAd *ad) {
 }
 
 void
-JobRouter::UpdateRoutedJobStatus(RoutedJob *job, classad::ClassAd update) {
+JobRouter::UpdateRoutedJobStatus(RoutedJob *job, const classad::ClassAd &update) {
 	classad::ClassAd *new_ad = NULL;
 	classad::ClassAdCollection *ad_collection2 = GetSchedd2ClassAds();
 
