@@ -1,5 +1,5 @@
 #! /usr/bin/env perl
-use strict;
+#use strict;
 use warnings;
 use Cwd;
 use IPC::Open3;
@@ -28,7 +28,7 @@ my $btdebug = 0;
 
 use base 'Exporter';
 
-our @EXPORT = qw(runcmd FAIL PASS ANY SIGNALED SIGNAL async_read verbose_system Which TRUE FALSE is_cygwin_perl is_windows is_windows_native_perl is_cygwin_perl fullchomp CreateEmptyFile CreateDir CopyIt TarCreate TarExtract MoveIt GetDirList DirLs List WhereIsInstallDir quoteMyString MyHead GeneralServer GeneralClient DagmanReadFlowLog);
+our @EXPORT = qw(runcmd FAIL PASS ANY SIGNALED SIGNAL async_read verbose_system Which TRUE FALSE is_cygwin_perl is_windows is_windows_native_perl is_cygwin_perl fullchomp CreateEmptyFile CreateDir CopyIt TarCreate TarExtract MoveIt GetDirList DirLs List WhereIsInstallDir quoteMyString MyHead GeneralServer GeneralClient DagmanReadFlowLog DryExtract GatherDryData LoadResults runCommandCarefully);
 
 sub TRUE{1};
 sub FALSE{0};
@@ -107,6 +107,7 @@ sub async_reader {
 			sub {
 				$Q->enqueue($_) while <$fh>;
 				$Q->enqueue(undef);
+				$Q->end();
 			}	
 		)->detach;
 
@@ -173,6 +174,7 @@ sub runcmd {
 		emit_string
 		use_system
 		sh_wrap
+		arguments
 		);
 
 	foreach my $key (keys %{$options}) {
@@ -219,7 +221,11 @@ sub runcmd {
 		$rc = system("$args");
 		$t1 = [Time::HiRes::gettimeofday];
 	} else {
-		$childpid = IPC::Open3::open3(\*IN, \*OUT, \*ERR, $args);
+		if( defined( ${$options}{arguments} ) ) {
+			$childpid = IPC::Open3::open3(\*IN, \*OUT, \*ERR, @{${$options}{arguments}} );
+		} else {
+			$childpid = IPC::Open3::open3(\*IN, \*OUT, \*ERR, $args);
+		}
 
 		my $bulkout = "";
 		my $bulkerror = "";
@@ -378,6 +384,19 @@ sub runcmd {
 	return \%returnthings;
 }
 
+sub runCommandCarefully {
+	my $options = shift( @_ );
+	my @argv = @_;
+
+	my %altOptions;
+	if( ! defined( $options ) ) {
+		$options = \%altOptions;
+	}
+	${$options}{arguments} = \@argv;
+
+	return runcmd( $argv[0], $options );
+}
+
 sub ProcessReturn {
 	my ($status) = @_;
 	my $rc = -1;
@@ -440,6 +459,10 @@ sub SetDefaults {
 		${$options}{sh_wrap} = TRUE;
 	}
 
+	if(!(exists ${$options}{arguments})) {
+		${$options}{arguments} = undef;
+	}
+
 }
 
 sub PrintStdOut {
@@ -707,6 +730,7 @@ sub CreateDir
 			return($ret);
 		}
 		$ret = system("$fullcmd");
+		#system("chmod 777 $cmdline");
 		#print "Tried to create dir got ret value:$ret path:$cmdline/$fullcmd\n";
 	}
 	return($ret);
@@ -1256,4 +1280,52 @@ sub DagmanReadFlowLog {
 	return(0);
 }
 
+sub DryExtract {
+    my $dryinarrayref = shift;
+    my $dryoutarrayref = shift;
+    my $extractstring = shift;
+    foreach my $dryline (@{$dryinarrayref}) {
+        chomp($dryline);
+        if($dryline =~ /\s*$extractstring=/) {
+#print "DryExtract:$dryline\n";
+            push @{$dryoutarrayref}, $dryline;
+		}
+	}
+}
+
+sub GatherDryData {
+    my $submitfile = shift;
+	my $targetfile = shift;
+	my @storage = ();
+    my $cmdtorun = "condor_submit -dry-run $targetfile $submitfile";
+	my $res = system("$cmdtorun");
+	if($res != 0) {
+		die "Return from system call non-zero\n";
+	} else {
+		open(TF,"<$targetfile") or die "Failed to open dry data file:$targetfile:$!\n";
+		while (<TF>) {
+			fullchomp($_);
+			push @storage, $_;
+		}
+		close(TF);
+	}
+	return(\@storage);
+}
+
+sub LoadResults {
+	my $arraytoload = shift;
+	my $filetoloadfrom = shift;
+
+	if(-f $filetoloadfrom) {
+		open(TF,"<$filetoloadfrom");
+		while (<TF>) {
+			fullchomp($_);
+			push @{$arraytoload}, $_;
+		}
+		close(TF);
+	} else {
+		die "LoadResults called with a file which does not exist:$filetoloadfrom:$!\n";
+	}
+}
+
 1;
