@@ -50,6 +50,8 @@
 #include "internet_obsolete.h"
 #include "ipv6_hostname.h"
 
+#include "condor_sockfunc.h"
+
 extern "C" {
 	void log_checkpoint (struct rusage *, struct rusage *);
 	void log_image_size (int);
@@ -66,11 +68,14 @@ extern "C" {
 	void reaper();
 }
 
+#include "std_univ_io.h"
+
 extern int JobStatus;
 extern int MyPid;
 extern int ImageSize;
 extern struct rusage JobRusage;
-extern ReliSock *syscall_sock;
+// extern ReliSock *syscall_sock;
+extern StdUnivSock *syscall_sock;
 extern PROC *Proc;
 extern char CkptName[];
 extern char ICkptName[];
@@ -247,7 +252,9 @@ pseudo_reallyexit( int *status, struct rusage *use_p )
 int
 pseudo_free_fs_blocks( const char *path )
 {
-	return sysapi_disk_space( path );
+	long long ck = sysapi_disk_space( path );
+	PRAGMA_REMIND("FIXME: disk_space truncation to INT_MAX here")
+	return (int)MIN(INT_MAX, ck);
 }
 
 int
@@ -392,12 +399,12 @@ pseudo_send_a_file( const char *path, mode_t mode )
 	(void)umask(omask);
 
 	syscall_sock->encode();
-	assert( syscall_sock->code(fd) );
-	assert( syscall_sock->end_of_message() );
+	CONDOR_ASSERT( syscall_sock->code(fd) );
+	CONDOR_ASSERT( syscall_sock->end_of_message() );
 
 		/* Get the file length */
 	syscall_sock->decode();
-	assert( syscall_sock->code(file_len) );
+	CONDOR_ASSERT( syscall_sock->code(file_len) );
 	dprintf(D_SYSCALLS, "\tFile size is %d\n", file_len );
 		
 		/* Transfer all the data */
@@ -421,8 +428,8 @@ pseudo_send_a_file( const char *path, mode_t mode )
 	(void)close( fd );
 
 		/* Get a repeat of the file length as a check */
-	assert( syscall_sock->code(checksum) );
-	assert( syscall_sock->end_of_message() );
+	CONDOR_ASSERT( syscall_sock->code(checksum) );
+	CONDOR_ASSERT( syscall_sock->end_of_message() );
 	dprintf(D_SYSCALLS, "\tchecksum is %d\n", checksum );
 	assert( checksum == file_len );
 
@@ -450,7 +457,7 @@ pseudo_get_file( const char *name )
 
 		/* Send the status from safe_open_wrapper_follow() so client knows we are going ahead */
 	syscall_sock->encode();
-	assert( syscall_sock->code(fd) );
+	CONDOR_ASSERT( syscall_sock->code(fd) );
 
 		/* Send the size of the file */
 	file_size = lseek( fd, 0, 2 );
@@ -459,7 +466,7 @@ pseudo_get_file( const char *name )
 	if( file_size < 0 ) {
 		return -1;
 	}
-	assert( syscall_sock->code(file_size) );
+	CONDOR_ASSERT( syscall_sock->code(file_size) );
 
 		/* Transfer the data */
 	for( bytes_to_go = file_size; bytes_to_go; bytes_to_go -= len ) {
@@ -468,13 +475,13 @@ pseudo_get_file( const char *name )
 			dprintf( D_ALWAYS, "Can't read from \"%s\"\n", name );
 			read_status = -1;
 		}
-		assert( syscall_sock->code_bytes(buf,len) );
+		CONDOR_ASSERT( syscall_sock->code_bytes(buf,len) );
 	}
 	(void)close( fd );
 
 		/* Send the size of the file again as a check */
-	assert( syscall_sock->code(file_size) );
-	assert( syscall_sock->end_of_message() );
+	CONDOR_ASSERT( syscall_sock->code(file_size) );
+	CONDOR_ASSERT( syscall_sock->end_of_message() );
 	dprintf(D_SYSCALLS, "\tSent file size %d\n", file_size );
 
 		/* Send final status */
@@ -676,6 +683,26 @@ pseudo_rename(char *from, char *to)
 	return -1;					// should never get here
 }
 
+// create_tcp_port() EXCEPT()d on all errors.
+void createListenPort( int & fd, condor_sockaddr & saFileServer ) {
+	condor_sockaddr saSysCall = syscall_sock->my_addr();
+
+	fd = socket( saSysCall.get_aftype(), SOCK_STREAM, 0 );
+	assert( fd != -1 );
+
+	// _condor_local_bind() binds to the "any" address, which must
+	// include the address in saSysCall.  We could limit to that address,
+	// but that seems unnecessary.
+	int rv = _condor_local_bind( FALSE, fd );
+	assert( rv == TRUE );
+
+	rv = listen( fd, 1 );
+	assert( rv == 0 );
+
+	rv = condor_getsockname( fd, saFileServer );
+	assert( rv == 0 );
+}
+
 /*
   Provide a process which will serve up the requested file as a
   stream.  The ip_addr and port number passed back are in host
@@ -756,12 +783,28 @@ pseudo_get_file_stream(
 	dprintf( D_FULLDEBUG, "\tlen = %lu\n", (unsigned long)*len );
 	BytesSent += *len;
 
-    //get_host_addr( ip_addr );
-	//display_ip_addr( *ip_addr );
+	//
+	// Obviously, our client just communicated with us over the syscall
+	// socket, so we know that's a workable protocol and address.  All
+	// that createListenPort() needs to do is find an allowed port.
+	//
+	condor_sockaddr saFileServer;
+	createListenPort( connect_sock, saFileServer );
+	dprintf( D_ALWAYS, "Created TCP listen socket %s\n", saFileServer.to_sinful().c_str() );
+	* port = saFileServer.get_port();
+	if( saFileServer.is_ipv4() ) {
+		// saFileServer is always bound to INADDR_ANY, which is useless.
+		saFileServer = get_local_ipaddr( CP_IPV4 );
+		* ip_addr = ntohl( saFileServer.to_sin().sin_addr.s_addr );
+	} else {
+		// All 8.3.3 and earlier clients should have failed in partSendJob(),
+		// where do_connect() called _condor_local_bind() and bound to IPv4
+		// when trying to make an IPv6 connection.  Nonetheless, the all-0
+		// address is a better flag than the all-1 address, because somebody
+		// may have forgotten to disallow omnicast.
+		* ip_addr = 0;
+	}
 
-    create_tcp_port( ip_addr, port, &connect_sock );
-    dprintf( D_NETWORK, "\taddr = %s\n", ipport_to_string(htonl(*ip_addr), htons(*port)));
-		
 	switch( child_pid = fork() ) {
 	case -1:	/* error */
 		dprintf( D_ALWAYS, "fork() failed, errno = %d\n", errno );
@@ -837,12 +880,6 @@ pseudo_put_file_stream(
 	dprintf( D_ALWAYS, "\tlen = %lu\n", len );
 	dprintf( D_ALWAYS, "\towner = %s\n", p->owner );
 
-    // ip_addr will be updated down below because I changed create_tcp_port so that
-    // ip address the socket is bound to is determined by that function. Therefore,
-    // we don't need to initialize ip_addr here
-    //get_host_addr( ip_addr );
-	//display_ip_addr( *ip_addr );
-
 		/* open the file */
 	if (CkptFile && UseCkptServer) {
 		SetCkptServerHost(CkptServerHost);
@@ -925,12 +962,28 @@ pseudo_put_file_stream(
 
 	(void)umask(omask);
 
-	//get_host_addr( ip_addr );
-	//display_ip_addr( *ip_addr );
+	//
+	// Obviously, our client just communicated with us over the syscall
+	// socket, so we know that's a workable protocol and address.  All
+	// that createListenPort() needs to do is find an allowed port.
+	//
+	condor_sockaddr saFileServer;
+	createListenPort( connect_sock, saFileServer );
+	dprintf( D_ALWAYS, "Created TCP listen socket %s\n", saFileServer.to_sinful().c_str() );
+	* port = saFileServer.get_port();
+	if( saFileServer.is_ipv4() ) {
+		// saFileServer is always bound to INADDR_ANY, which is useless.
+		saFileServer = get_local_ipaddr( CP_IPV4 );
+		* ip_addr = ntohl( saFileServer.to_sin().sin_addr.s_addr );
+	} else {
+		// All 8.3.3 and earlier clients should have failed in partSendJob(),
+		// where do_connect() called _condor_local_bind() and bound to IPv4
+		// when trying to make an IPv6 connection.  Nonetheless, the all-0
+		// address is a better flag than the all-1 address, because somebody
+		// may have forgotten to disallow omnicast.
+		* ip_addr = 0;
+	}
 
-	create_tcp_port(ip_addr, port, &connect_sock);
-    dprintf(D_NETWORK, "\taddr = %s\n", ipport_to_string(htonl(*ip_addr), htons(*port)));
-	
 	switch( child_pid = fork() ) {
 	  case -1:	/* error */
 		dprintf( D_ALWAYS, "fork() failed, errno = %d\n", errno );
@@ -952,7 +1005,7 @@ pseudo_put_file_stream(
 
 			/* Send status assuring peer that we got everything */
 		answer = htonl( bytes_read );
-		write( data_sock, &answer, sizeof(answer) );
+		write( data_sock, &answer, sizeof(answer));
 		dprintf( D_ALWAYS,
 			"\tSTREAM FILE RECEIVED OK (%d bytes)\n", bytes_read );
 
@@ -1027,64 +1080,14 @@ file_size( int fd )
 	return answer;
 }
 
-/*
-  Create a tcp socket and begin listening for a connection by some
-  other process.  Using "ip", "port", and "fd" as output parameters, give back
-  the address to which the other process should connect, and the
-  file descriptor with which this process can refer to the socket.  Return
-  0 if everything works OK, and -1 otherwise.  N.B. The port number is
-  returned in host byte order.
-*/
-int
-create_tcp_port( unsigned int *ip, u_short *port, int *fd )
-{
-    struct sockaddr_in  sin;
-    SOCKET_LENGTH_TYPE  addr_len;
-   
-    addr_len = sizeof(sin);
-
-        /* create a tcp socket */
-    if( (*fd=socket(AF_INET,SOCK_STREAM,0)) < 0 ) {
-        EXCEPT( "create_tcp_port(): socket() failed: %d(%s)", 
-			errno, strerror(errno) );
-    }
-    dprintf( D_FULLDEBUG, "\tconnect_sock = %d\n", *fd );
-
-        /* bind it to an address */
-
-        /* FALSE means this is an incoming connection */
-    if( ! _condor_local_bind(FALSE, *fd) ) {
-        EXCEPT( "create_tcp_port(): bind() failed: %d(%s)",
-			errno, strerror(errno) );
-    }
-
-        /* determine which local port number we were assigned to */
-    struct sockaddr_in *tmp = getSockAddr(*fd);
-    if (tmp == NULL) {
-        EXCEPT("create_tcp_port(): getSockAddr() failed");
-    }
-    *ip = ntohl(tmp->sin_addr.s_addr);
-    *port = ntohs(tmp->sin_port);
-
-        /* Get ready to accept a connection */
-    if( listen(*fd,1) < 0 ) {
-        EXCEPT( "create_tcp_port(): listen() failed: %d(%s)", 
-			errno, strerror(errno) );
-    }
-    dprintf( D_FULLDEBUG, "\tListening...\n" );
-
-    return 0;
-}
-
-
 /*
   Get the IP address of this host.  The address is in host byte order.
 */
 void
 get_host_addr( unsigned int *ip_addr )
 {
-	// [TODO:IPV6]
-	condor_sockaddr myaddr = get_local_ipaddr();
+	// TODO: This is arbitrarily chosing IPv4. Frankly, no one should use this function, it's awful.
+	condor_sockaddr myaddr = get_local_ipaddr(CP_IPV4);
 	sockaddr_in saddr = myaddr.to_sin();
 	*ip_addr = ntohl(saddr.sin_addr.s_addr);
 	display_ip_addr( *ip_addr );
@@ -1851,8 +1854,7 @@ JobPreCkptServerScheddNameChange()
 	job_version[0] = '\0';
 	if (JobAd && JobAd->LookupString(ATTR_VERSION, job_version, sizeof(job_version))) {
 		CondorVersionInfo ver(job_version, "JOB");
-		if (ver.built_since_version(6,2,0) &&
-			ver.built_since_date(11,16,2000)) {
+		if (ver.built_since_version(6,2,0)) {
 			return false;
 		}
 	}
