@@ -28,7 +28,6 @@
 #include "condor_common.h"
 #include "startd.h"
 #include "classad_merge.h"
-#include "dynuser.h"
 #include "condor_auth_x509.h"
 #include "setenv.h"
 #include "my_popen.h"
@@ -42,10 +41,6 @@
 #include "glexec_starter.linux.h"
 #endif
 
-#ifdef WIN32
-extern dynuser *myDynuser;
-#endif
-
 Starter::Starter()
 {
 	s_ad = NULL;
@@ -576,6 +571,12 @@ Starter::executeDir()
 	return s_execute_dir.Length() ? s_execute_dir.Value() : NULL;
 }
 
+char const *
+Starter::encryptedExecuteDir()
+{
+	return s_encrypted_execute_dir.Length() ? s_encrypted_execute_dir.Value() : NULL;
+}
+
 int 
 Starter::spawn( time_t now, Stream* s )
 {
@@ -671,7 +672,14 @@ Starter::exited(int status)
 
 		// Now, delete any files lying around.
 	ASSERT( executeDir() );
-	cleanup_execute_dir( s_pid, executeDir() );
+	if ( encryptedExecuteDir() ) {
+		// Remove $EXECUTE/encryptedX/dir_pid/*
+		cleanup_execute_dir( s_pid, encryptedExecuteDir(), true );
+		s_encrypted_execute_dir = "";
+	} else {
+		// Remove $EXECUTE/dir_pid/*
+		cleanup_execute_dir( s_pid, executeDir() );
+	}
 
 #if defined(LINUX)
 	if( param_boolean( "GLEXEC_STARTER", false ) ) {
@@ -712,7 +720,7 @@ Starter::execJobPipeStarter( void )
 
 	int* std_fds_p = NULL;
 	int std_fds[3];
-	int pipe_fds[2];
+	int pipe_fds[2] = {-1,-1};
 	if( s_claim->hasJobAd() ) {
 		if( ! daemonCore->Create_Pipe(pipe_fds) ) {
 			dprintf( D_ALWAYS, "ERROR: Can't create pipe to pass job ClassAd "
@@ -908,6 +916,44 @@ Starter::execDCStarter( ArgList const &args, Env const *env,
 	ASSERT( executeDir() );
 	new_env.SetEnv( "_CONDOR_EXECUTE", executeDir() );
 
+		// Handle encrypted execute directory
+	FilesystemRemap  fs_remap_obj;	// put on stack so destroyed when leave this method
+	FilesystemRemap* fs_remap = NULL;
+	// If admin desires encrypted exec dir in config, do it
+	bool encrypt_execdir = param_boolean_crufty("ENCRYPT_EXECUTE_DIRECTORY",false);
+	// Or if user wants encrypted exec in job ad, do it
+	if (!encrypt_execdir && s_claim->ad()) {
+		s_claim->ad()->LookupBool(ATTR_ENCRYPT_EXECUTE_DIRECTORY,encrypt_execdir);
+	}
+	if ( encrypt_execdir ) {
+#ifdef LINUX
+		// On linux, setup a directory $EXECUTE/encryptedX subdirectory
+		// to serve as an ecryptfs mount point; pass this directory
+		// down to the condor_starter as if it were $EXECUTE so
+		// that the starter creates its dir_<pid> directory on the
+		// ecryptfs filesystem setup by doing an AddEncryptedMapping.
+		static int unsigned long privdirnum = 0;
+		TemporaryPrivSentry sentry(PRIV_CONDOR);
+		s_encrypted_execute_dir.formatstr("%s%cencrypted%lu",executeDir(),
+				DIR_DELIM_CHAR,privdirnum++);
+		if( mkdir(encryptedExecuteDir(), 0755) < 0 ) {
+			dprintf( D_FAILURE|D_ALWAYS,
+			         "Failed to create encrypted dir %s: %s\n",
+			         encryptedExecuteDir(),
+			         strerror(errno) );
+			return 0;
+		}
+		dprintf( D_ALWAYS,
+		         "Created encrypted dir %s\n", encryptedExecuteDir() );
+		fs_remap = &fs_remap_obj;
+		if ( fs_remap->AddEncryptedMapping(encryptedExecuteDir()) ) {
+			// FilesystemRemap object dprintfs out an error message for us
+			return 0;
+		}
+		new_env.SetEnv( "_CONDOR_EXECUTE", encryptedExecuteDir() );
+#endif
+	}
+
 	env = &new_env;
 
 
@@ -927,7 +973,23 @@ Starter::execDCStarter( ArgList const &args, Env const *env,
 		}
 	}
 
-	if (param_boolean("ASSIGN_CPU_AFFINITY", false)) {
+	bool affinityBool = false;
+	if ( ! s_claim || ! s_claim->ad()) {
+		affinityBool = param_boolean("ASSIGN_CPU_AFFINITY", false);
+	} else {
+		auto_free_ptr assign_cpu_affinity(param("ASSIGN_CPU_AFFINITY"));
+		if ( ! assign_cpu_affinity.empty()) {
+			classad::Value value;
+			if (s_claim->ad()->EvaluateExpr(assign_cpu_affinity.ptr(), value)) {
+				if ( ! value.IsBooleanValueEquiv(affinityBool)) {
+					// was an expression, but not a bool, so report it and continue
+					EXCEPT("ASSIGN_CPU_AFFINITY does not evaluate to a boolean, it is : %s", ClassAdValueToString(value));
+				}
+			}
+		}
+	}
+
+	if (affinityBool) {
 		new_env.SetEnv("_CONDOR_STARTD_ASSIGNED_AFFINITY", affinityString.c_str());
 		new_env.SetEnv("_CONDOR_ENFORCE_CPU_AFFINITY", "true");
 		dprintf(D_ALWAYS, "Setting affinity env to %s\n", affinityString.c_str());
@@ -1010,7 +1072,8 @@ Starter::execDCStarter( ArgList const &args, Env const *env,
 
 	s_pid = daemonCore->
 		Create_Process( final_path, *final_args, PRIV_ROOT, reaper_id,
-		                TRUE, env, NULL, &fi, inherit_list, std_fds );
+		                TRUE, TRUE, env, NULL, &fi, inherit_list, std_fds,
+						NULL, 0, NULL, 0, NULL, NULL, NULL, NULL, fs_remap);
 	if( s_pid == FALSE ) {
 		dprintf( D_ALWAYS, "ERROR: exec_starter failed!\n");
 		s_pid = 0;
@@ -1125,6 +1188,7 @@ Starter::execOldStarter( void )
 	                                         PRIV_ROOT,    // start as root
 	                                         main_reaper,  // reaper
 	                                         FALSE,        // no command port
+	                                         FALSE,        // no command port
 	                                         &env,
 	                                         NULL,         // inherit out cwd
 	                                         &family_info, // new family
