@@ -290,23 +290,35 @@ command_pckpt_all( Service*, int, Stream* )
 int
 command_x_event( Service*, int, Stream* s ) 
 {
-	dprintf( D_FULLDEBUG, "command_x_event() called.\n" );
+	// Simple attempt to avoid D_ALWAYS warnings from registering twice.
+	static Stream * lastStashed = NULL;
 
 		// Only trust events over the network if the network message
 		// originated from our local machine.
 	if ( !s ||							// trust calls from within the startd
 		 (s && s->peer_is_local())		// trust only sockets from local machine
-	   ) 
+	   )
 	{
 		sysapi_last_xevent();
 	} else {
-		dprintf( D_ALWAYS, 
+		dprintf( D_ALWAYS,
 			"ERROR command_x_event received from %s is not local - discarded\n",
 			s->peer_ip_str() );
 	}
 
 	if( s ) {
 		s->end_of_message();
+
+		if( lastStashed != s ) {
+			int rc = daemonCore->Register_Command_Socket( s, "kbdd socket" );
+			if( rc < 0 ) {
+				dprintf( D_ALWAYS, "Failed to register kbdd socket for updates: error %d.\n", rc );
+				return FALSE;
+			}
+			lastStashed = s;
+		}
+
+		return KEEP_STREAM;
 	}
 	return TRUE;
 }
@@ -1859,21 +1871,41 @@ activate_claim( Resource* rip, Stream* stream )
 	} 
 #ifndef WIN32
 	else {
+		//
+		// This should be exclusively for the standard universe.
+		//
+		condor_sockaddr streamSA;
+		if(! streamSA.from_ip_string( stream->my_ip_str() )) {
+			EXCEPT( "Unable to extract socket address from stream.\n" );
+		}
+
+		// We don't officially support IPv6 in standard universe...
+		if( streamSA.is_ipv6() ) {
+			dprintf( D_ALWAYS, "WARNING -- request to run standard-universe job via IPv6.  There may be problems.\n" );
+		}
 
-			// Set up the two starter ports and send them to the shadow
 		stRec.version_num = VERSION_FOR_FLOCK;
-		sock_1 = create_port(&rsock_1);
-		sock_2 = create_port(&rsock_2);
+
+		// The shadow expects to be able to connect to the given port
+		// numbers at the address (and protocol) is used to contact the
+		// startd in the first place.
+		rsock_1.bind( streamSA.get_protocol(), false, 0, false );
+		rsock_1.listen();
+		sock_1 = rsock_1.get_file_desc();
 		stRec.ports.port1 = rsock_1.get_port();
+
+		rsock_2.bind( streamSA.get_protocol(), false, 0, false );
+		rsock_2.listen();
+		sock_2 = rsock_2.get_file_desc();
 		stRec.ports.port2 = rsock_2.get_port();
 
-		stRec.server_name = strdup( my_full_hostname() );
-	
-			// Send our local IP address, too.
-		
+		stRec.server_name = strdup( get_local_fqdn().Value() );
+
+		// Send our local IP address, too.
 		// stRec.ip_addr actually is never used.
 		// Just make sure that it does not have 0 value.
-		condor_sockaddr local_addr = get_local_ipaddr();
+		// // TODO: Arbitrarily picking IPv4 
+		condor_sockaddr local_addr = get_local_ipaddr(CP_IPV4);
 		struct in_addr local_in_addr = local_addr.to_sin().sin_addr;
 		memcpy( &stRec.ip_addr, &local_in_addr, sizeof(struct in_addr) );
 
@@ -1920,12 +1952,14 @@ activate_claim( Resource* rip, Stream* stream )
 				if( fd_2 == -2 ) {
 					rip->dprintf( D_ALWAYS, "accept timed out\n" );
 					delete tmp_starter;
+					close(fd_1);
 					ABORT;
 				} else {
 					rip->dprintf( D_ALWAYS, 
 								  "tcp_accept_timeout returns %d, errno=%d\n",
 								  fd_2, errno );
 					delete tmp_starter;
+					close(fd_1);
 					ABORT;
 				}
 			}
@@ -2304,6 +2338,33 @@ cleanup:
 }
 
 
+int
+caUpdateMachineAd( Stream * s, char * c, ClassAd * ad ) {
+	//
+	// Update the machine ad (for the lifetime of this startd).
+	//
+	resmgr->updateExtrasClassAd( ad );
+
+	// Force an update to the collector right away.
+	resmgr->update_all();
+
+	if( false ) {
+		std::string errorMessage = "Unable to update machine ad.";
+		sendErrorReply( s, c, CA_FAILURE, errorMessage.c_str() );
+		return FALSE;
+	}
+
+	ClassAd reply;
+	reply.Assign( ATTR_RESULT, getCAResultString( CA_SUCCESS ) );
+	if( ! sendCAReply( s, c, & reply ) ) {
+		dprintf( D_ALWAYS | D_FAILURE, "Failed to send update machine ad reply.\n" );
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
 int
 command_classad_handler( Service*, int dc_cmd, Stream* s )
 {
@@ -2331,6 +2392,12 @@ command_classad_handler( Service*, int dc_cmd, Stream* s )
 	}
 
 	switch( cmd ) {
+	case CA_UPDATE_MACHINE_AD:
+		rval = caUpdateMachineAd( s, cmd_str, & ad );
+		free( cmd_str );
+		return rval;
+		break;
+
 	case CA_LOCATE_STARTER:
 			// special case, since it's not really about claims at
 			// all, but instead are worrying about trying to find a
