@@ -71,10 +71,10 @@ VMStarterInfo::updateUsageOfVM(void)
 	if( ProcAPI::getProcInfo(m_vm_pid, pi, proc_status) == 
 			PROCAPI_SUCCESS ) {
 		memcpy(&m_vm_alive_pinfo, pi, sizeof(struct procInfo));
-		if( (DebugFlags & D_FULLDEBUG) && (DebugFlags & D_LOAD) ) {
-			dprintf(D_FULLDEBUG,"Usage of process[%d] for a VM is updated\n", 
+		if( IsDebugVerbose(D_LOAD) ) {
+			dprintf(D_LOAD,"Usage of process[%d] for a VM is updated\n", 
 					m_vm_pid);
-			dprintf(D_FULLDEBUG,"sys_time=%lu, user_time=%lu, image_size=%lu\n", 
+			dprintf(D_LOAD,"sys_time=%lu, user_time=%lu, image_size=%lu\n", 
 					pi->sys_time, pi->user_time, get_image_size(*pi));
 		}
 		delete pi;
@@ -119,8 +119,8 @@ VMStarterInfo::getUsageOfVM(ProcFamilyUsage &usage)
 #endif
 	}
 
-	if( (DebugFlags & D_FULLDEBUG) && (DebugFlags & D_LOAD) ) {
-		dprintf( D_FULLDEBUG,
+	if( IsDebugVerbose(D_LOAD) ) {
+		dprintf( D_LOAD,
 				"VMStarterInfo::getUsageOfVM(): Percent CPU usage "
 				"for VM process with pid %u is: %f\n",
 				m_vm_pid,
@@ -289,7 +289,7 @@ void
 VMUniverseMgr::printVMGahpInfo( int debug_level )
 {
 	dprintf( debug_level, "........VMGAHP info........\n");
-	m_vmgahp_info.AttrList::dPrint(debug_level);
+	dPrintAd(debug_level, m_vmgahp_info);
 	dprintf( debug_level, "\n");
 }
 
@@ -328,7 +328,8 @@ VMUniverseMgr::publish( ClassAd* ad, amask_t  /*mask*/ )
 		}else if( strcasecmp(attr_name, ATTR_VM_NETWORKING) == MATCH ) {
 			ad->Assign(ATTR_VM_NETWORKING, m_vm_networking); 
 		}else {
-			ad->Insert(attr_name, expr->Copy());
+			ExprTree * pTree =  expr->Copy();
+			ad->Insert(attr_name, pTree, false);
 		}
 	}
 
@@ -417,7 +418,7 @@ VMUniverseMgr::testVMGahp(const char* gahppath, const char* vmtype)
 #if !defined(WIN32)
 	if( can_switch_ids() ) {
 		MyString tmp_str;
-		tmp_str.sprintf("%d", (int)get_condor_uid());
+		tmp_str.formatstr("%d", (int)get_condor_uid());
 		SetEnv("VMGAHP_USER_UID", tmp_str.Value());
 	}
 #endif
@@ -687,7 +688,7 @@ VMUniverseMgr::freeVM(pid_t s_pid)
 
 	MyString pid_dir;
 	Directory execute_dir(info->m_execute_dir.Value(), PRIV_ROOT);
-	pid_dir.sprintf("dir_%ld", (long)s_pid);
+	pid_dir.formatstr("dir_%ld", (long)s_pid);
 
 	if( execute_dir.Find_Named_Entry( pid_dir.Value() ) ) {
 		// starter didn't exit cleanly,
@@ -881,7 +882,7 @@ VMUniverseMgr::killVM(const char *matchstring)
 #if !defined(WIN32)
 	if( can_switch_ids() ) {
 		MyString tmp_str;
-		tmp_str.sprintf("%d", (int)get_condor_uid());
+		tmp_str.formatstr("%d", (int)get_condor_uid());
 		SetEnv("VMGAHP_USER_UID", tmp_str.Value());
 	}
 #endif
@@ -921,7 +922,7 @@ VMUniverseMgr::killVM(VMStarterInfo *info)
 	MyString matchstring;
 	MyString workingdir;
 
-	workingdir.sprintf("%s%cdir_%ld", info->m_execute_dir.Value(),
+	workingdir.formatstr("%s%cdir_%ld", info->m_execute_dir.Value(),
 	                   DIR_DELIM_CHAR, (long)info->m_pid);
 
 	if( (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_XEN ) == MATCH) || (strcasecmp(m_vm_type.Value(), CONDOR_VM_UNIVERSE_KVM) == 0)) {
