@@ -35,7 +35,7 @@
 //-------------------------------------------------------------
 
 // about self
-char* mySubSystem = "STARTD_FACTORY";		// used by Daemon Core
+const char* mySubSystem = "STARTD_FACTORY";		// used by Daemon Core
 
 //-------------------------------------------------------------
 
@@ -76,15 +76,12 @@ void main_shutdown_graceful()
 
 //-------------------------------------------------------------
 
-void
-main_pre_dc_init( int /* argc */, char* /* argv */ [] )
+int
+main( int argc, char **argv )
 {
-		// dprintf isn't safe yet...
+	dc_main_init = main_init;
+	dc_main_config = main_config;
+	dc_main_shutdown_fast = main_shutdown_fast;
+	dc_main_shutdown_graceful = main_shutdown_graceful;
+	return dc_main( argc, argv );
 }
-
-
-void
-main_pre_command_sock_init( )
-{
-}
-
