@@ -23,16 +23,22 @@
 #include "io_proxy.h"
 #include "io_proxy_handler.h"
 #include "ipv6_hostname.h"
+#include "condor_config.h"
 
 #define IO_PROXY_COOKIE_SIZE 32
 
 static char * cookie_create( int length );
 
 IOProxy::IOProxy()
+	:
+	server(NULL),
+	m_shadow(NULL),
+	cookie(0),
+	socket_registered(false),
+	m_want_io(false),
+	m_want_updates(false),
+	m_want_delayed(false)
 {
-	server = NULL;
-	cookie = 0;
-	socket_registered = false;
 }
 
 IOProxy::~IOProxy()
@@ -61,7 +67,10 @@ int IOProxy::connect_callback( Stream * /*stream*/ )
 
 	success = server->accept(*client);
 	if(success) {
-		if(get_local_ipaddr().compare_address(client->peer_addr())) {
+		// I'm not sure why we believe the socket could be bound to anything
+		// other than the address we want to hear from, but...
+		condor_sockaddr remote = client->peer_addr();
+		if( remote.is_loopback() ) {
 			dprintf(D_FULLDEBUG,"IOProxy: accepting connection from %s\n",client->peer_ip_str());
 			accept_client = true;
 		} else {
@@ -72,7 +81,7 @@ int IOProxy::connect_callback( Stream * /*stream*/ )
 	}
 	
 	if(accept_client) {
-		IOProxyHandler *handler = new IOProxyHandler();
+		IOProxyHandler *handler = new IOProxyHandler(m_shadow, m_want_io, m_want_updates, m_want_delayed);
 		if(!handler->init(client,cookie)) {
 			dprintf(D_ALWAYS,"IOProxy: couldn't register request callback!\n");
 			client->close();
@@ -92,8 +101,13 @@ Initialize this proxy and dump the contact information into the given file.
 Returns true on success, false otherwise.
 */
 
-bool IOProxy::init( const char *config_file )
+bool IOProxy::init( JICShadow *shadow, const char *config_file, bool want_io, bool want_updates, bool want_delayed )
 {
+	m_shadow = shadow;
+	m_want_io = want_io;
+	m_want_updates = want_updates;
+	m_want_delayed = want_delayed;
+
 	FILE *file=0;
 	int fd=-1;
 
@@ -103,12 +117,22 @@ bool IOProxy::init( const char *config_file )
 		return false;
 	}
 
-	/* passing FALSE to bind means this is an incomming connection.
-	 * however, here we are going to pass TRUE because only machines
-	 * on this host need to connect to the ioproxy (chirp) socket,
-	 * so there is no need to register it with a ccb broker. 
-	 **/
-	if(!server->bind(TRUE)) {
+	//
+	// Without other arguments, Sock::bind() now generates an IPv6 loopback
+	// address if we're in mixed-mode on machines without a real IPv6 address.
+	// That's fine (we'll rewrite the address on the way out if we have to),
+	// but my_ip_string() no longer matches, which causes chirp grief (because
+	// we pass our address to it here, via the chirp.config file).
+	//
+	// Instead, just go ahead and bind to IPv4 (unless it's disabled, in
+	// which case, use IPv6); and use the loopback address, which is more
+	// secure.  Regardless, use the actual address on which we're listening
+	// to fill in the .chirp.config file.
+	//
+
+	condor_protocol proto = CP_IPV4;
+	if( ! param_boolean( "ENABLE_IPV4", true ) ) { proto = CP_IPV6; }
+	if(!server->bind( proto, false, 0, true )) {
 		dprintf(D_ALWAYS,"IOProxy: couldn't bind: %s\n",strerror(errno));
 		return false;
 	}
@@ -140,7 +164,7 @@ bool IOProxy::init( const char *config_file )
 		goto failure;
 	}
 
-	fprintf(file,"%s %d %s\n",my_ip_string(),server->get_port(),cookie);
+	fprintf( file, "%s %d %s\n", server->my_ip_str(), server->get_port(), cookie );
 	fclose(file);
 
 	daemonCore->Register_Socket( server, "IOProxy listen socket", 
@@ -151,7 +175,6 @@ bool IOProxy::init( const char *config_file )
 
 	failure:
 	if(cookie) free(cookie);
-	if(file) fclose(file);
 	IGNORE_RETURN unlink(config_file);
 	server->close();
 	return false;
