@@ -149,6 +149,8 @@ public:
 		 */
 	int reconnect( ReliSock* s, ClassAd* ad );
 
+		/// Disconnect from the shadow - jic virtual method handler
+	void disconnect();
 
 		// // // // // // // // // // // //
 		// Notfication to the shadow
@@ -238,6 +240,15 @@ public:
 		/* Update Job ClassAd with checkpoint info and log it */
 	void updateCkptInfo(void);
 
+		/* Record an attribute to update */
+	bool recordDelayedUpdate( const std::string &name, const classad::ExprTree &expr );
+
+		/* Return an attribute from the combination of the delayed ad and the starter */
+	std::auto_ptr<classad::ExprTree> getDelayedUpdate( const std::string &name );
+
+	virtual bool wroteChirpConfig() { return m_wrote_chirp_config; }
+	virtual const std::string chirpConfigFilename() { return m_chirp_config_filename; }
+
 private:
 
 		// // // // // // // // // // // //
@@ -421,7 +432,7 @@ private:
 	void setX509ProxyExpirationTimer();
 
 		// The proxy is about to expire, do something!
-	int proxyExpiring();
+	void proxyExpiring();
 
 		// // // // // // // //
 		// Private Data Members
@@ -438,6 +449,8 @@ private:
 		/// hostname (or whatever the startd gave us) of our shadow 
 	char* m_shadow_name;
 
+	classad::ClassAd m_delayed_updates;
+	std::vector<std::string> m_delayed_update_attrs;
 	IOProxy io_proxy;
 
 	FileTransfer *filetrans;
@@ -460,6 +473,9 @@ private:
 	char* fs_domain;
 	bool trust_uid_domain;
 
+	std::string m_chirp_config_filename;
+	bool m_wrote_chirp_config;
+
 		/** A flag to keep track of the case where we were trying to
 			cleanup our job but we discovered that we were
 			disconnected from the shadow.  This way, if there's a
@@ -468,6 +484,24 @@ private:
 		*/
 	bool job_cleanup_disconnected;
 
+		/** A flag to keep track of if the syscall_sock is registered
+			with daemonCore for read.  We do this so we notice asap
+			when this socket is closed, ie due to failed TCP keep alives.
+		*/
+	bool syscall_sock_registered;
+		/** handler for read callbacks on syscall sock, to notice if it closes */
+	int syscall_sock_handler(Stream *s);
+		/** epoch time when the syscall_sock got closed (shadow disappeared) */
+	time_t syscall_sock_lost_time;
+		/** timer id of timer to invoke job_lease_expired() when syscall_sock closed */
+	int syscall_sock_lost_tid;
+		/** invoked when job lease expired - exits w/ well known status */
+	void job_lease_expired();
+		/** must be invoked whenever our syscall_sock is reconnected */
+	void syscall_sock_reconnect();
+		/** must be invoked whenever we notice our syscall_sock is borked */
+	void syscall_sock_disconnect();
+
 	Stream *m_job_startd_update_sock;
 
 		/** A list of output files that have been dynamically added
