@@ -43,14 +43,16 @@ int sysapi_phys_memory(void);
 int sysapi_phys_memory_raw_no_param(void);
 
 /* How to get the free disk blocks from a full pathname, answer in KB */
-int sysapi_disk_space_raw(const char *filename);
-int sysapi_disk_space(const char *filename);
+long long sysapi_disk_space_raw(const char *filename);
+long long sysapi_disk_space(const char *filename);
+
+/* get ncpus without making any calls to config system, this uses cached values */
+void sysapi_ncpus_raw(int * num_cpus, int * num_hyperthreads);
+/* For backward compatibility, this is an alias for the above call */
+void sysapi_ncpus(int * num_cpus, int * num_hyperthreads);
+/* call call cpu/hyperthread detection code.  this should only be called once per process. */
+void sysapi_detect_cpu_cores(int * num_cpus, int * num_hyperthreads);
 
-/* return the number of cpus there on a machine */
-int sysapi_ncpus_raw(void);
-int sysapi_ncpus(void);
-/* get raw ncpus without making any calls to config system */
-void sysapi_ncpus_raw_no_param(int *num_cpus,int *num_hyperthread_cpus);
 
 #if 0 // removed from condor_utils to reduce shadow memory use
 /* calculate the number of mips the machine is. Even though this is a user
@@ -205,10 +207,13 @@ private:
 	bool m_up;
 };
 
-bool sysapi_get_network_device_info(std::vector<NetworkDeviceInfo> &devices);
+bool sysapi_get_network_device_info(std::vector<NetworkDeviceInfo> &devices, bool want_ipv4, bool want_ipv6);
 
 void sysapi_clear_network_device_info_cache();
 
+/* determine if a linux version is version X or newer */
+bool sysapi_is_linux_version_atleast(const char *version_to_check);
+
 #endif // __cplusplus
 
 #endif
