@@ -20,6 +20,15 @@
 package SimpleJob;
 
 use CondorTest;
+
+# need a memory so one can callbacks one per test
+# not every call to RunCheck  bt 4/29/15
+my %savedcallbacks = ();
+my %args = ();
+
+# track children so we can reap them later
+my @children = ();
+
 my $timeout = 0;
 my $defaulttimeout = 300;
 $timeout = $defaulttimeout;
@@ -50,9 +59,93 @@ $ExitSuccess = sub {
 	CondorTest::debug("Job completed\n",1);
 };
 
+my %defaultcallbacks = (
+	"on_success" => $ExitSuccess,
+	"on_submit" => $submitted,
+	"on_abort" => 0,
+	"on_failure" => 0,
+	"on_exitedabnormal" => 0,
+	"return_submit_file_name" => 0,
+);
+
+sub ShowKeys {
+	foreach my $key (sort keys %savedcallbacks) {
+		print "key: $key\n";
+	}
+}
+
+sub CallbackReset {
+	%savedcallbacks = ();
+}
+
+# we only want to set things into memory if they have been
+# explicitly set and explicitly setting it again will override
+# first setting.
+sub CallBackMemoryUsage {
+	my $action = shift;
+	#print "Current action in CallBackMemoryUsage is:$action\n";
+	#ShowKeys();
+	if(exists $args{$action}) {
+		#print "Saved and defaults for $action being requested\n";
+	}
+	if(exists $savedcallbacks{$action}) {
+		#print "we have a saved callback for $action\n";
+	}
+	if((exists $savedcallbacks{$action}) & (!(exists $args{$action}))) {
+		# this one has been set before and is not being reset
+		#print "CallBackMemoryUsage: saved callback for $action\n";
+		return($savedcallbacks{$action});
+	} elsif((exists $savedcallbacks{$action}) & (exists $args{$action})) {
+		# set before but overriding
+		#print "CallBackMemoryUsage: saved callback for $action being changed\n";
+		$savedcallbacks{$action} = $args{$action};
+		return($args{$action});
+	} elsif((!(exists $savedcallbacks{$action})) & (exists $args{$action})) {
+		# being explictely set for first time
+		#print "CallBackMemoryUsage:$action being set for first time\n";
+		#ShowKeys();
+		$savedcallbacks{$action} = $args{$action};
+		#ShowKeys();
+		return($args{$action});
+	}
+	# now we must look at default vaules which get returned but
+	# NOT saved
+	if((exists $defaultcallbacks{$action}) & ($defaultcallbacks{$action} != 0)) {
+		#print "CallBackMemoryUsage: reserved call back for $action being set, not saved\n";
+		# a non dummy callback
+		return($defaultcallbacks{$action});
+	} elsif((exists $defaultcallbacks{$action}) & ($defaultcallbacks{$action} == 0)) {
+		#print "CallBackMemoryUsage: $action has no defaullt callback\n";
+		# this one gets no default callback
+		return(undef);
+	} else {
+		# this one gets the function $dummy
+		#print "CallBackMemoryUsage: $action gets standard default callback\n";
+		return($dummy);
+	}
+}
+
+sub FigureTimedCallback {
+	if((exists $savedcallbacks{alt_timed}) & (!(exists $args{alt_timed}))) {
+		# this one has been set before and is not being reset
+		#return($savedcallbacks{alt_timed});
+	} elsif((exists $savedcallbacks{alt_timed}) & (exists $args{alt_timed})) {
+		# set before but overriding
+		#$savedcallbacks{alt_timed} = $args{alt_timed};
+		return($args{alt_timed});
+	} elsif((!(exists $savedcallbacks{alt_timed})) & (exists $args{alt_timed})) {
+		# being explictely set for first time
+		$savedcallbacks{alt_timed} = $args{alt_timed};
+		return($args{alt_timed});
+	} elsif((!(exists $savedcallbacks{alt_timed})) & (!(exists $args{alt_timed}))) {
+		# using default time callback
+		return($timed_callback);
+	}
+}
+
 sub RunCheck
 {
-    my %args = @_;
+    %args = @_;
 	my $availableslots = 0;
     my $result = 0;
 
@@ -68,9 +161,41 @@ sub RunCheck
 #
 ################################################################################
 
+	#print "Checking duration being passsed to RunCheck <$args{duration}>\n";
+    my $execute_fn = CallBackMemoryUsage("on_execute");
+    my $hold_fn = CallBackMemoryUsage("on_hold");
+    my $shadow_fn = CallBackMemoryUsage("on_shadow");
+    my $suspended_fn = CallBackMemoryUsage("on_suspended");
+    my $released_fn = CallBackMemoryUsage("on_released");
+    my $unsuspended_fn = CallBackMemoryUsage("on_unsuspended");
+    my $disconnected_fn = CallBackMemoryUsage("on_disconnected");
+    my $reconnected_fn = CallBackMemoryUsage("on_reconnected");
+    my $reconnectfailed_fn = CallBackMemoryUsage("on_reconnectfailed");
+    my $imageupdated_fn = CallBackMemoryUsage("on_imageupdated");
+    my $evicted_ewc_fn = CallBackMemoryUsage("on_evictedwithcheckpoint");
+    my $evicted_ewoc_fn = CallBackMemoryUsage("on_evictedwithoutcheckpoint");
+    my $evicted_wreq_fn = CallBackMemoryUsage("on_evictedwithrequeue");
+	my $wanterror_fn =CallBackMemoryUsage("on_wanterror");
+    my $submit_fn = CallBackMemoryUsage("on_submit");
+    my $ulog_fn = CallBackMemoryUsage("on_ulog");
+    my $abort_fn = CallBackMemoryUsage("on_abort");
+	# no default on abort handling handled in above function call
+	my $donewithsuccess_fn = CallBackMemoryUsage("on_success");
+	my $exitedabnormal_fn = CallBackMemoryUsage("on_exitedabnormal");
+	my $exitedfailure_fn = CallBackMemoryUsage("on_failure");
+	my $namecallback_fn = CallBackMemoryUsage("return_submit_file_name");
+
+	# question when do I clear this timeout?
+	my $timedcallback_fn = FigureTimedCallback();
+
+	# we preset call backs before fork since if child does this and requests saved
+	# callbacks, the next child won't see it
+	 
 	if($args{no_wait}) {
 		my $pid = fork();
 		if($pid != 0) {
+			push @children, $pid;
+			#ShowChildren();
 			return(0);
 		}
 	}
@@ -82,6 +207,7 @@ sub RunCheck
 		$availableslots = ExamineSlots($args{check_slots});
 	
 		if($availableslots < $queuesz) {
+			printf "RunCheck returning before submission. available slots:$availableslots vs requested:$queuesz\n";
     		CondorTest::RegisterResult( $result, %args );
     		return $result;
 		}
@@ -116,7 +242,7 @@ sub RunCheck
 	if(exists $args{duration}) {
 		$duration = $args{duration};
 	} else {
-		print "duration not set, defaulting to 1\n";
+		#print "duration not set, defaulting to 1\n";
 	}
 
 	if(exists $args{timeout}){
@@ -125,98 +251,90 @@ sub RunCheck
 		$timeout = $args{timeout};
 	}
 	 
-	#print "Checking duration being passsed to RunCheck <$args{duration}>\n";
-    my $execute_fn = $args{on_execute} || $dummy;
-    my $hold_fn = $args{on_hold} || $dummy;
-    my $shadow = $args{on_shadow} || $dummy;
-    my $suspended_fn = $args{on_suspended} || $dummy;
-    my $released_fn = $args{on_released} || $dummy;
-    my $unsuspended_fn = $args{on_unsuspended} || $dummy;
-    my $disconnected_fn = $args{on_disconnected} || $dummy;
-    my $reconnected_fn = $args{on_reconnected} || $dummy;
-    my $reconnectfailed_fn = $args{on_reconnectfailed} || $dummy;
-    my $imageupdated_fn = $args{on_imageupdated} || $dummy;
-    my $evicted_ewc_fn = $args{on_evictedwithcheckpoint} || $dummy;
-    my $evicted_ewoc_fn = $args{on_evictedwithoutcheckpoint} || $dummy;
-    my $evicted_wreq_fn = $args{on_evictedwithrequeue} || $dummy;
-	my $wanterror_fn =$args{on_wanterror} || $dummy;
-    my $submit_fn = $args{on_submit} || $submitted;
-    my $ulog_fn = $args{on_ulog} || $dummy;
-    my $abort_fn = $args{on_abort} || $aborted;
-	my $donewithsuccess_fn = $args{on_success} || $ExitSuccess;
-
-
-	if(defined $args{alt_timed}) {
-		CondorTest::RegisterTimed( $testname, $args{alt_timed}, $timeout);
-	} else {
-		CondorTest::RegisterTimed( $testname, $timed_callback, $timeout);
+	if(defined $abort_fn) {
+    	CondorTest::RegisterAbort( $testname, $abort_fn );
+	}
+	if(defined $donewithsuccess_fn) {
+    	CondorTest::RegisterExitedSuccess( $testname, $donewithsuccess_fn );
 	}
 
-    CondorTest::RegisterAbort( $testname, $abort_fn );
-    CondorTest::RegisterExitedSuccess( $testname, $donewithsuccess_fn );
-    CondorTest::RegisterExecute($testname, $execute_fn);
-    CondorTest::RegisterULog($testname, $ulog_fn);
-    CondorTest::RegisterSubmit( $testname, $submit_fn );
+	if(defined $execute_fn) {
+    	CondorTest::RegisterExecute($testname, $execute_fn);
+	}
+	if(defined $ulog_fn) {
+    	CondorTest::RegisterULog($testname, $ulog_fn);
+	}
+	if(defined $submit_fn) {
+    	CondorTest::RegisterSubmit( $testname, $submit_fn );
+	}
 
 	#If we register thees to dummy, then we don't get
 	#the error function registered which says this is bad
 
-	if( exists $args{on_shadow} ) {
-    	CondorTest::RegisterShadow( $testname, $shadow );
+	if(defined $shadow_fn) {
+    	CondorTest::RegisterShadow( $testname, $shadow_fn );
 	}
-	if( exists $args{on_wanterror} ) {
+	if(defined $wanterror_fn) {
     	CondorTest::RegisterWantError( $testname, $wanterror_fn );
 	}
-	if( exists $args{on_imageupdated} ) {
+	if(defined $imageupdated_fn) {
     	CondorTest::RegisterImageUpdated( $testname, $imageupdated_fn );
 	}
-	if( exists $args{on_evictedwithcheckpoint} ) {
+	if(defined $evicted_ewc_fn) {
     	CondorTest::RegisterEvictedWithCheckpoint( $testname, $evicted_ewc_fn );
 	}
-	if( exists $args{on_evictedwithoutcheckpoint} ) {
+	if(defined $evicted_ewoc_fn) {
     	CondorTest::RegisterEvictedWithoutCheckpoint( $testname, $evicted_ewoc_fn );
 	}
-	if( exists $args{on_evictedwithrequeue} ) {
+	if(defined $evicted_wreq_fn) {
     	CondorTest::RegisterEvictedWithRequeue( $testname, $evicted_wreq_fn );
 	}
-	if( exists $args{on_hold} ) {
+	if(defined $hold_fn) {
     	CondorTest::RegisterHold( $testname, $hold_fn );
 	}
-	if( exists $args{on_released} ) {
+	if(defined $released_fn) {
     	CondorTest::RegisterRelease( $testname, $released_fn );
 	}
-	if( exists $args{on_exitedabnormal} ) {
-    	CondorTest::RegisterExitedAbnormal( $testname, $args{on_exitedabnormal} );
+	if(defined $exitedabnormal_fn) {
+    	CondorTest::RegisterExitedAbnormal( $testname, $exitedabnormal_fn );
 	}
-	if( exists $args{on_evicted} ) {
+	if(defined $evicted_fn) {
     	CondorTest::RegisterEvicted( $testname, $evicted_fn );
 	}
-	if( exists $args{on_disconnected} ) {
+	if(defined $disconnected_fn) {
     	CondorTest::RegisterDisconnected( $testname, $disconnected_fn );
 	}
-	if( exists $args{on_reconnectfailed} ) {
+	if(defined $reconnectfailed_fn) {
     	CondorTest::RegisterReconnectFailed( $testname, $reconnectfailed_fn );
 	}
-	if( exists $args{on_reconnected} ) {
+	if(defined $reconnected_fn) {
     	CondorTest::RegisterReconnected( $testname, $reconnected_fn );
 	}
-	if( exists $args{on_suspended} ) {
+	if(defined $suspended_fn) {
     	CondorTest::RegisterSuspended( $testname, $suspended_fn );
 	}
-	if( exists $args{on_unsuspended} ) {
+	if(defined $unsuspended_fn) {
     	CondorTest::RegisterUnsuspended( $testname, $unsuspended_fn );
 	}
-	if(exists $args{on_failure}) {
-		CondorTest::RegisterExitedFailure( $testname, $args{on_failure} );
+	if(defined $exitedfailure_fn) {
+		CondorTest::RegisterExitedFailure( $testname, $exitedfailure_fn );
+	}
+	
+	if(defined $timedcallback_fn) {
+		#print "Timed callbach set in simplejob\n";
+		# here starts our new timer just for this module
+		# look at timer use in other module here in Check
+		# like the log checking ones
+		CondorTest::RegisterSJTimed( $testname, $timedcallback_fn, $timeout  );
 	}
 
 	my $program = $args{runthis} || "x_sleep.pl";
     my $submit_fname = CondorTest::TempFileName("$testname.submit");
 
-	my $namecallback;
-	if(exists $args{return_submit_file_name}) {
-		$namecallback = $args{return_submit_file_name};
-		&$namecallback($submit_fname);
+	# was used in concurrency tests is used job_max_running
+	if(defined $namecallback_fn) {
+		# call it
+		&$namecallback_fn($submit_fname);
 	}
 
     open( SUBMIT, ">$submit_fname" ) || die "error writing to $submit_fname: $!\n";
@@ -277,18 +395,21 @@ sub RunCheck
 	}
     close( SUBMIT );
 
+	print "RunCheck finally calling RunTest\n";
 	if (defined $args{GetClusterId}) {
     	$result = CondorTest::RunTest($testname, $submit_fname, 0, $args{GetClusterId});
 	} else {
     	$result = CondorTest::RunTest($testname, $submit_fname, 0);
 	}
-    CondorTest::RegisterResult( $result, %args );
-	 if($args{no_wait}) {
-		exit(0);
-	} else {
-		return $result;
-	}
+	#print "Simple job back!\n";
 
+	Condor::RemoveSJTimed();
+
+    CondorTest::RegisterResult( $result, %args );
+	 	if(($args{no_wait}) & ($pid == 0)) {
+	 	#print "Child exiting\n";
+		exit(0);
+	}
 }
 
 sub ExamineSlots
@@ -329,4 +450,40 @@ sub ExamineSlots
 	return($slotcount);
 }
 
+sub ShowChildren {
+	my $kids = @children;
+	print "ShowChildren: $kids kids\n";
+	foreach my $child (@children) {
+	}
+}
+
+sub HarvestChildren {
+	my $kids = @children;
+	#print "HarvestChildren: $kids kids\n";
+	foreach my $child (@children) {
+		#print "waiting for child:$child\n";
+		while(($childpid = waitpid($child,0)) != -1) {
+			$retvalue = $?;
+			if($childpid == 0) {
+				#print "Waitinng on child:$child waitpid says:$childpid\n";
+			} else {
+				#print "Child:$childpid has returned\n";
+				if ($retvalue & 0x7f) {
+					# died with signal and maybe coredump.
+					# Ignore the fact a coredump happened for now.
+					TestDebug( "Monitor done and status bad: \n",4);
+					$accumulatedreturn += 1;
+				} else {
+					# Child returns valid exit code
+					my $rc = $retvalue >> 8;
+					#print "ProcessReturn: Exited normally $rc\n";
+					$accumulatedreturn += $rc;
+				}
+			}
+		}
+	}
+	#print "HarvestChildren: all done\n";
+	return(0);
+}
+
 1;
