@@ -26,8 +26,6 @@
 // The transferd object which does the work requested by the parent
 TransferD g_td;
 
-DECL_SUBSYSTEM( "TRANSFERD", SUBSYSTEM_TYPE_DAEMON );
-
 void main_init(int argc, char *argv[])
 {
 	// parse the command line attributes
@@ -74,15 +72,14 @@ void main_shutdown_graceful()
 
 //-------------------------------------------------------------
 
-void
-main_pre_dc_init( int argc, char* argv[] )
+int
+main( int argc, char **argv )
 {
-		// dprintf isn't safe yet...
+	set_mySubSystem( "TRANSFERD", SUBSYSTEM_TYPE_DAEMON );
+
+	dc_main_init = main_init;
+	dc_main_config = main_config;
+	dc_main_shutdown_fast = main_shutdown_fast;
+	dc_main_shutdown_graceful = main_shutdown_graceful;
+	return dc_main( argc, argv );
 }
-
-
-void
-main_pre_command_sock_init( )
-{
-}
-
