@@ -20,24 +20,27 @@
 /*
 	A header file for the function driver code for unit_tests.
  */
+#include <list>
 
 typedef bool (*test_func_ptr)(void);
 
 class FunctionDriver {
 public:
 		// ctor/detor
-	FunctionDriver(int size=10);
+	FunctionDriver(int num_tests = INT_MAX);
 	~FunctionDriver();
+
+	void init(int num_tests);
 	
 		// register a function with the driver
 	void register_function(test_func_ptr);
 
 		// perform all the functions.  Returns true if every function
 		// called returns true, false otherwise.
-	bool do_all_functions();
+	bool do_all_functions(bool increment_tests_run = true);
 	
 private:
-	test_func_ptr *pointers;
-	int max_functions;
-	int used_functions;
+	std::list<test_func_ptr> pointers;
+	int num_funcs_or_objs;
 };
+
