@@ -2759,549 +2759,12 @@ classad::ExprTree *RemoveExplicitTargetRefs( classad::ExprTree *tree )
 	}
 }	
 
-#if 0
-static ClassAd job_attrs_ad;
-static ClassAd machine_attrs_ad;
-static ClassAd schedd_attrs_ad;
-static StringList job_attrs_strlist;
-static StringList machine_attrs_strlist;
-
-static bool target_attrs_init = false;
-
-static const char *machine_attrs_list[] = {
-	ATTR_NAME,
-	ATTR_MACHINE,
-	ATTR_AVAIL_TIME,
-	ATTR_LAST_AVAIL_INTERVAL,
-	ATTR_AVAIL_SINCE,
-	ATTR_AVAIL_TIME_ESTIMATE,
-	ATTR_IS_VALID_CHECKPOINT_PLATFORM,
-	ATTR_WITHIN_RESOURCE_LIMITS,
-	ATTR_RUNNING_COD_JOB,
-	ATTR_ARCH,
-	ATTR_OPSYS,
-	ATTR_HAS_IO_PROXY,
-	ATTR_CHECKPOINT_PLATFORM,
-	ATTR_TOTAL_VIRTUAL_MEMORY,
-	ATTR_TOTAL_CPUS,
-	ATTR_TOTAL_MEMORY,
-	ATTR_KFLOPS,
-	ATTR_MIPS,
-	ATTR_LOCAL_CREDD,
-	ATTR_LAST_BENCHMARK,
-	ATTR_TOTAL_LOAD_AVG,
-	ATTR_TOTAL_CONDOR_LOAD_AVG,
-	ATTR_CLOCK_MIN,
-	ATTR_CLOCK_DAY,
-	ATTR_RUN_BENCHMARKS,
-	ATTR_VIRTUAL_MEMORY,
-	ATTR_TOTAL_DISK,
-	ATTR_DISK,
-	ATTR_CONDOR_LOAD_AVG,
-	ATTR_LOAD_AVG,
-	ATTR_KEYBOARD_IDLE,
-	ATTR_CONSOLE_IDLE,
-	ATTR_MEMORY,
-	ATTR_CPUS,
-	ATTR_MAX_JOB_RETIREMENT_TIME,
-	ATTR_FETCH_WORK_DELAY,
-	ATTR_UNHIBERNATE,
-	ATTR_MACHINE_LAST_MATCH_TIME,
-	ATTR_SLOT_WEIGHT,
-	ATTR_IS_OWNER,
-	ATTR_CPU_BUSY,
-	ATTR_TOTAL_SLOTS,
-	ATTR_TOTAL_VIRTUAL_MACHINES,
-	ATTR_STATE,
-	ATTR_LAST_HEARD_FROM,
-	ATTR_ENTERED_CURRENT_STATE,
-	ATTR_ACTIVITY,
-	ATTR_ENTERED_CURRENT_ACTIVITY,
-	ATTR_TOTAL_TIME_OWNER_IDLE,
-	ATTR_TOTAL_TIME_UNCLAIMED_IDLE,
-	ATTR_TOTAL_TIME_UNCLAIMED_BENCHMARKING,
-	ATTR_TOTAL_TIME_MATCHED_IDLE,
-	ATTR_TOTAL_TIME_CLAIMED_IDLE,
-	ATTR_TOTAL_TIME_CLAIMED_BUSY,
-	ATTR_TOTAL_TIME_CLAIMED_SUSPENDED,
-	ATTR_TOTAL_TIME_CLAIMED_RETIRING,
-	ATTR_TOTAL_TIME_PREEMPTING_VACATING,
-	ATTR_TOTAL_TIME_PREEMPTING_KILLING,
-	ATTR_TOTAL_TIME_BACKFILL_IDLE,
-	ATTR_TOTAL_TIME_BACKFILL_BUSY,
-	ATTR_TOTAL_TIME_BACKFILL_KILLING,
-	ATTR_CPU_IS_BUSY,
-	ATTR_CPU_BUSY_TIME,
-	ATTR_VIRTUAL_MACHINE_ID,
-	ATTR_SLOT_PARTITIONABLE,
-	ATTR_SLOT_DYNAMIC,
-	ATTR_LAST_FETCH_WORK_SPAWNED,
-	ATTR_LAST_FETCH_WORK_COMPLETED,
-	ATTR_NEXT_FETCH_WORK_DELAY,
-	ATTR_CURRENT_RANK,
-	ATTR_REMOTE_USER,
-	ATTR_REMOTE_OWNER,
-	ATTR_CLIENT_MACHINE,
-	ATTR_NUM_COD_CLAIMS,
-	ATTR_HIBERNATION_SUPPORTED_STATES,
-	ATTR_STARTER_ABILITY_LIST,
-	ATTR_HAS_VM,
-	ATTR_VM_GUEST_MAC,
-	ATTR_VM_GUEST_IP,
-	ATTR_VM_GUEST_MEM,
-	ATTR_VM_AVAIL_NUM,
-	ATTR_VM_MEMORY,
-	ATTR_VM_NETWORKING,
-	ATTR_VM_ALL_GUEST_IPS,
-	ATTR_VM_ALL_GUEST_MACS,
-	ATTR_VM_TYPE,
-	ATTR_VM_NETWORKING_TYPES,
-	ATTR_HAS_RECONNECT,
-	ATTR_HAS_FILE_TRANSFER,
-	ATTR_HAS_FILE_TRANSFER_PLUGIN_METHODS,
-	ATTR_HAS_PER_FILE_ENCRYPTION,
-	ATTR_HAS_MPI,
-	ATTR_HAS_TDP,
-	ATTR_HAS_JOB_DEFERRAL,
-	ATTR_HAS_JIC_LOCAL_CONFIG,
-	ATTR_HAS_JIC_LOCAL_STDIN,
-	ATTR_HAS_JAVA,
-	ATTR_HAS_WIN_RUN_AS_OWNER,
-	ATTR_JAVA_VENDOR,
-	ATTR_JAVA_VERSION,
-	"JavaSpecificationVersion",
-	ATTR_JAVA_MFLOPS,
-	ATTR_MATCHED_CONCURRENCY_LIMITS,
-	ATTR_REMOTE_USER_PRIO,
-	ATTR_REMOTE_USER_RESOURCES_IN_USE,
-	ATTR_REMOTE_GROUP_RESOURCES_IN_USE,
-	ATTR_REMOTE_GROUP_QUOTA,
-	NULL		// list must end with NULL
-};
-
-static const char *job_attrs_list[]  = {
-	ATTR_CLUSTER_ID,
-	ATTR_PROC_ID,
-	ATTR_Q_DATE,
-	ATTR_COMPLETION_DATE,
-	ATTR_OWNER,
-	ATTR_NT_DOMAIN,
-	ATTR_JOB_REMOTE_WALL_CLOCK,
-	ATTR_JOB_LOCAL_USER_CPU,
-	ATTR_JOB_LOCAL_SYS_CPU,
-	ATTR_JOB_REMOTE_USER_CPU,
-	ATTR_JOB_REMOTE_SYS_CPU,
-	ATTR_NUM_CKPTS,
-	ATTR_NUM_JOB_STARTS,
-	ATTR_NUM_RESTARTS,
-	ATTR_NUM_SYSTEM_HOLDS,
-	ATTR_JOB_COMMITTED_TIME,
-	ATTR_COMMITTED_SLOT_TIME,
-	ATTR_CUMULATIVE_SLOT_TIME,
-	ATTR_TOTAL_SUSPENSIONS,
-	ATTR_LAST_SUSPENSION_TIME,
-	ATTR_CUMULATIVE_SUSPENSION_TIME,
-	ATTR_COMMITTED_SUSPENSION_TIME,
-	ATTR_JOB_UNIVERSE,
-	ATTR_JOB_CMD,
-	ATTR_TRANSFER_EXECUTABLE,
-	ATTR_WANT_REMOTE_SYSCALLS,
-	ATTR_WANT_CHECKPOINT,
-	ATTR_JOB_CMD_MD5,
-	ATTR_JOB_CMD_HASH,
-	ATTR_GRID_RESOURCE,
-	ATTR_JOB_VM_TYPE,
-	ATTR_JOB_VM_CHECKPOINT,
-	ATTR_JOB_VM_NETWORKING,
-	ATTR_JOB_VM_NETWORKING_TYPE,
-	ATTR_JOB_VM_MEMORY,
-	ATTR_JOB_VM_VCPUS,
-	ATTR_JOB_VM_MACADDR,
-	ATTR_JOB_VM_HARDWARE_VT,
-	ATTR_WHEN_TO_TRANSFER_OUTPUT,
-	ATTR_SHOULD_TRANSFER_FILES,
-	ATTR_MIN_HOSTS,
-	ATTR_MAX_HOSTS,
-	ATTR_MACHINE_COUNT,
-	ATTR_REQUEST_CPUS,
-	ATTR_NEXT_JOB_START_DELAY,
-	ATTR_IMAGE_SIZE,
-	ATTR_EXECUTABLE_SIZE,
-	ATTR_DISK_USAGE,
-	ATTR_REQUEST_MEMORY,
-	ATTR_REQUEST_DISK,
-	ATTR_FILE_REMAPS,
-	ATTR_BUFFER_FILES,
-	ATTR_BUFFER_SIZE,
-	ATTR_BUFFER_BLOCK_SIZE,
-	ATTR_TRANSFER_INPUT_FILES,
-	ATTR_TRANSFER_OUTPUT_FILES,
-	ATTR_JAR_FILES,
-	ATTR_JOB_INPUT,
-	ATTR_JOB_OUTPUT,
-	ATTR_JOB_ERROR,
-	ATTR_TRANSFER_OUTPUT_REMAPS,
-	ATTR_ENCRYPT_INPUT_FILES,
-	ATTR_ENCRYPT_OUTPUT_FILES,
-	ATTR_DONT_ENCRYPT_INPUT_FILES,
-	ATTR_DONT_ENCRYPT_OUTPUT_FILES,
-	ATTR_FETCH_FILES,
-	ATTR_COMPRESS_FILES,
-	ATTR_APPEND_FILES,
-	ATTR_LOCAL_FILES,
-	ATTR_JOB_JAVA_VM_ARGS1,
-	ATTR_JOB_JAVA_VM_ARGS2,
-	ATTR_PARALLEL_SCRIPT_SHADOW,
-	ATTR_PARALLEL_SCRIPT_STARTER,
-	ATTR_TRANSFER_INPUT,
-	ATTR_STREAM_INPUT,
-	ATTR_TRANSFER_OUTPUT,
-	ATTR_STREAM_OUTPUT,
-	ATTR_TRANSFER_ERROR,
-	ATTR_STREAM_ERROR,
-	ATTR_JOB_STATUS,
-	ATTR_LAST_JOB_STATUS,
-	ATTR_HOLD_REASON,
-	ATTR_HOLD_REASON_CODE,
-	ATTR_HOLD_REASON_SUBCODE,	
-	ATTR_ENTERED_CURRENT_STATUS,
-	ATTR_JOB_PRIO,
-	ATTR_NICE_USER,
-	ATTR_PERIODIC_HOLD_CHECK,
-	ATTR_PERIODIC_RELEASE_CHECK,
-	ATTR_PERIODIC_REMOVE_CHECK,
-	ATTR_ON_EXIT_HOLD_CHECK,
-	ATTR_JOB_LEAVE_IN_QUEUE,
-	ATTR_ON_EXIT_REMOVE_CHECK,
-	ATTR_JOB_NOOP,
-	ATTR_JOB_NOOP_EXIT_SIGNAL,
-	ATTR_JOB_NOOP_EXIT_CODE,
-	ATTR_JOB_NOTIFICATION,
-	ATTR_NOTIFY_USER,
-	ATTR_EMAIL_ATTRIBUTES,
-	ATTR_LAST_MATCH_LIST_LENGTH,
-	ATTR_DAG_NODE_NAME,
-	ATTR_DAGMAN_JOB_ID,
-	ATTR_JOB_REMOTE_IWD,
-	ATTR_JOB_ARGUMENTS1,
-	ATTR_JOB_ARGUMENTS2,
-	ATTR_DEFERRAL_TIME,
-	ATTR_DEFERRAL_WINDOW,
-	ATTR_DEFERRAL_PREP_TIME,
-	ATTR_SCHEDD_INTERVAL,
-	ATTR_JOB_ENVIRONMENT1,
-	ATTR_JOB_ENVIRONMENT2,
-	ATTR_JOB_ENVIRONMENT1_DELIM,
-	ATTR_JOB_ROOT_DIR,
-	ATTR_TOOL_DAEMON_CMD,
-	ATTR_TOOL_DAEMON_INPUT,
-	ATTR_TOOL_DAEMON_OUTPUT,
-	ATTR_TOOL_DAEMON_ERROR,
-	ATTR_TOOL_DAEMON_ARGS1,
-	ATTR_TOOL_DAEMON_ARGS2,
-	ATTR_SUSPEND_JOB_AT_EXEC,
-	ATTR_JOB_RUNAS_OWNER,
-	ATTR_JOB_LOAD_PROFILE,
-	ATTR_JOB_IWD,
-	ATTR_ULOG_FILE,
-	ATTR_ULOG_USE_XML,
-	ATTR_CORE_SIZE,
-	ATTR_JOB_LEASE_DURATION,
-	ATTR_GRID_JOB_ID,
-	ATTR_JOB_MATCHED,
-	ATTR_CURRENT_HOSTS,
-	ATTR_GLOBUS_RESUBMIT_CHECK,
-	ATTR_USE_GRID_SHELL,
-	ATTR_REMATCH_CHECK,
-	ATTR_GLOBUS_RSL,
-	ATTR_NORDUGRID_RSL,
-	ATTR_KEYSTORE_ALIAS,
-	ATTR_KEYSTORE_PASSPHRASE_FILE,
-	ATTR_X509_USER_PROXY_SUBJECT,
-	ATTR_X509_USER_PROXY,
-	ATTR_X509_USER_PROXY_VONAME,
-	ATTR_X509_USER_PROXY_FIRST_FQAN,
-	ATTR_X509_USER_PROXY_FQAN,
-	ATTR_MYPROXY_HOST_NAME,
-	ATTR_MYPROXY_SERVER_DN,
-	ATTR_MYPROXY_PASSWORD,
-	ATTR_MYPROXY_CRED_NAME,
-	ATTR_MYPROXY_REFRESH_THRESHOLD,
-	ATTR_MYPROXY_NEW_PROXY_LIFETIME,
-	ATTR_MYPROXY_PASSWORD_EXISTS,
-	ATTR_KILL_SIG,
-	ATTR_REMOVE_KILL_SIG,
-	ATTR_HOLD_KILL_SIG,
-	ATTR_CONCURRENCY_LIMITS,
-	ATTR_REMOVE_REASON,
-	ATTR_USER,
-	ATTR_SHADOW_BIRTHDATE,
-	ATTR_JOB_FINISHED_HOOK_DONE,
-	ATTR_STAGE_OUT_FINISH,
-	ATTR_STAGE_IN_START,
-	ATTR_STAGE_OUT_START,
-	ATTR_JOB_STATUS_ON_RELEASE,
-	ATTR_LAST_REJ_MATCH_REASON,
-	ATTR_LAST_REJ_MATCH_TIME,
-	ATTR_LAST_MATCH_TIME,
-	ATTR_NUM_MATCHES,
-	ATTR_LAST_JOB_LEASE_RENEWAL,
-	ATTR_JOB_START_DATE,
-	ATTR_JOB_LAST_START_DATE,
-	ATTR_JOB_CURRENT_START_DATE,
-	ATTR_JOB_RUN_COUNT,
-	ATTR_NUM_SHADOW_STARTS,
-	ATTR_NUM_RESTARTS,
-	ATTR_REMOTE_HOST,
-	ATTR_REMOTE_SLOT_ID,
-	ATTR_REMOTE_POOL,
-	ATTR_STARTD_PRINCIPAL,
-	ATTR_JOB_WALL_CLOCK_CKPT,
-	ATTR_LAST_REMOTE_HOST,
-	ATTR_LAST_PUBLIC_CLAIM_ID,
-	ATTR_ORIG_MAX_HOSTS,
-	ATTR_NUM_SHADOW_EXCEPTIONS,
-	ATTR_JOB_EXIT_STATUS,
-	ATTR_ON_EXIT_CODE,
-	ATTR_ON_EXIT_BY_SIGNAL,
-	ATTR_ON_EXIT_SIGNAL,
-	ATTR_RELEASE_REASON,
-	ATTR_GRID_JOB_ID,
-	ATTR_WANT_MATCH_DIAGNOSTICS,
-	ATTR_AUTO_CLUSTER_ID,
-	ATTR_SUBMITTER_USER_PRIO,
-	ATTR_SUBMITTOR_PRIO,
-	ATTR_SUBMITTER_USER_RESOURCES_IN_USE,
-	ATTR_SUBMITTER_GROUP_RESOURCES_IN_USE,
-	ATTR_SUBMITTER_GROUP_QUOTA,
-	NULL		// list must end with NULL
-};
-
-static const char *schedd_attrs_list[]  = {
-	ATTR_SCHEDD_IP_ADDR,
-	"MyShadowSockname",
-	"QueueCleanInterval",
-	"JobStartDelay",
-	"JobStartCount",
-	"JobsThisBurst",
-	ATTR_MAX_JOBS_RUNNING,
-	"MaxJobsSubmitted",
-	"JobsStarted",
-	"SwapSpace",
-	"ShadowSizeEstimate",
-	"SwapSpaceExhausted",
-	"ReservedSwap",
-	"JobsIdle",
-	"JobsRunning",
-	"BadCluster",
-	"BadProc",
-	"N_Owners",
-	"NegotiationRequestTime",
-	"ExitWhenDone",
-	"StartJobTimer",
-	"CondorAdministrator",
-	"AccountantName",
-	"MaxFlockLevel",
-	"FlockLevel",
-	"MaxExceptions",
-	ATTR_ARCH,
-	ATTR_OPSYS,
-	ATTR_MEMORY,
-	ATTR_DISK,
-	ATTR_TOTAL_LOCAL_IDLE_JOBS,
-	ATTR_TOTAL_LOCAL_RUNNING_JOBS,
-	ATTR_START_LOCAL_UNIVERSE,
-	ATTR_TOTAL_SCHEDULER_IDLE_JOBS,
-	ATTR_TOTAL_SCHEDULER_RUNNING_JOBS,
-	ATTR_START_SCHEDULER_UNIVERSE,
-	NULL		// list must end with NULL
-};
-
-static void InitTargetAttrLists()
-{
-	const char **attr;
-	char *tmp;
-	MyString buff;
-	StringList tmp_strlist;
-
-	///////////////////////////////////
-	// Set up Machine attributes list
-	///////////////////////////////////
-	for ( attr = machine_attrs_list; *attr; attr++ ) {
-		machine_attrs_ad.AssignExpr( *attr, "True" );
-	}
-
-	tmp = param( "STARTD_EXPRS" );
-	if ( tmp ) {
-		tmp_strlist.initializeFromString( tmp );
-		free( tmp );
-		tmp_strlist.rewind();
-		while ( (tmp = tmp_strlist.next()) ) {
-			machine_attrs_ad.AssignExpr( tmp, "True" );
-		}
-		tmp_strlist.clearAll();
-	}
-
-	tmp = param( "STARTD_ATTRS" );
-	if ( tmp ) {
-		tmp_strlist.initializeFromString( tmp );
-		free( tmp );
-		tmp_strlist.rewind();
-		while ( (tmp = tmp_strlist.next()) ) {
-			machine_attrs_ad.AssignExpr( tmp, "True" );
-		}
-		tmp_strlist.clearAll();
-	}
-
-	tmp = param( "STARTD_RESOURCE_PREFIX" );
-	if ( tmp ) {
-		buff.formatstr( "%s*", tmp );
-		machine_attrs_strlist.append( buff.Value() );
-		free( tmp );
-	} else {
-		machine_attrs_strlist.append( "slot*" );
-	}
-
-	tmp = param( "TARGET_MACHINE_ATTRS" );
-	if ( tmp ) {
-		machine_attrs_strlist.initializeFromString( tmp );
-		free( tmp );
-	}
-
-	///////////////////////////////////
-	// Set up Job attributes list
-	///////////////////////////////////
-	for ( attr = job_attrs_list; *attr; attr++ ) {
-		job_attrs_ad.AssignExpr( *attr, "True" );
-	}
-
-	tmp = param( "SUBMIT_EXPRS" );
-	if ( tmp ) {
-		tmp_strlist.initializeFromString( tmp );
-		free( tmp );
-		tmp_strlist.rewind();
-		while ( (tmp = tmp_strlist.next()) ) {
-			job_attrs_ad.AssignExpr( tmp, "True" );
-		}
-		tmp_strlist.clearAll();
-	}
-
-	buff.formatstr( "%s*", ATTR_LAST_MATCH_LIST_PREFIX );
-	job_attrs_strlist.append( buff.Value() );
-
-	buff.formatstr( "%s*", ATTR_NEGOTIATOR_MATCH_EXPR );
-	job_attrs_strlist.append( buff.Value() );
-
-	tmp = param( "TARGET_JOB_ATTRS" );
-	if ( tmp ) {
-		job_attrs_strlist.initializeFromString( tmp );
-		free( tmp );
-	}
-
-	///////////////////////////////////
-	// Set up Schedd attributes list
-	///////////////////////////////////
-	for ( attr = schedd_attrs_list; *attr; attr++ ) {
-		schedd_attrs_ad.AssignExpr( *attr, "True" );
-	}
-
-	target_attrs_init = true;
-}
-#endif
 
 classad::ExprTree *AddTargetRefs( classad::ExprTree *tree, TargetAdType /*target_type*/ )
 {
 	// Disable AddTargetRefs for now
 	return tree->Copy();
 
-#if 0
-	if ( !target_attrs_init ) {
-		InitTargetAttrLists();
-	}
-
-	if( tree == NULL ) {
-		return NULL;
-	}
-	if ( target_type != TargetMachineAttrs && target_type != TargetJobAttrs &&
-		 target_type != TargetScheddAttrs ) {
-		return NULL;
-	}
-	classad::ExprTree::NodeKind nKind = tree->GetKind( );
-	switch( nKind ) {
-	case classad::ExprTree::ATTRREF_NODE: {
-		classad::ExprTree *expr = NULL;
-		string attr = "";
-		bool abs = false;
-		( ( classad::AttributeReference * )tree )->GetComponents(expr,attr,abs);
-		if( abs || expr != NULL ) {
-			return tree->Copy( );
-		}
-		else {
-			bool add_target = false;
-			if ( target_type == TargetMachineAttrs ) {
-				if ( machine_attrs_ad.Lookup( attr.c_str() ) ||
-					 machine_attrs_strlist.contains_anycase_withwildcard( attr.c_str() ) ) {
-					add_target = true;
-				}
-			} else if ( target_type == TargetJobAttrs ) {
-				if ( job_attrs_ad.Lookup( attr.c_str() ) ||
-					 job_attrs_strlist.contains_anycase_withwildcard( attr.c_str() ) ) {
-					add_target = true;
-				}
-			} else {
-				if ( schedd_attrs_ad.Lookup( attr.c_str() ) ) {
-					add_target = true;
-				}
-			}
-			if( add_target ) {
-					// attribute is in our list, so insert "target"
-				classad::AttributeReference *target = NULL;
-				target = classad::AttributeReference::MakeAttributeReference(NULL,
-																	"target");
-				return classad::AttributeReference::MakeAttributeReference(target,attr);
-			}
-			else {
-				return tree->Copy( );
-			}
-		}
-	}
-	case classad::ExprTree::OP_NODE: {
-		classad::Operation::OpKind oKind;
-		classad::ExprTree * expr1 = NULL;
-		classad::ExprTree * expr2 = NULL;
-		classad::ExprTree * expr3 = NULL;
-		classad::ExprTree * newExpr1 = NULL;
-		classad::ExprTree * newExpr2 = NULL;
-		classad::ExprTree * newExpr3 = NULL;
-		( ( classad::Operation * )tree )->GetComponents( oKind, expr1, expr2, expr3 );
-		if( expr1 != NULL ) {
-			newExpr1 = AddTargetRefs( expr1, target_type );
-		}
-		if( expr2 != NULL ) {
-			newExpr2 = AddTargetRefs( expr2, target_type );
-		}
-		if( expr3 != NULL ) {
-			newExpr3 = AddTargetRefs( expr3, target_type );
-		}
-		return classad::Operation::MakeOperation( oKind, newExpr1, newExpr2, newExpr3 );
-	}
-	case classad::ExprTree::FN_CALL_NODE: {
-		std::string fn_name;
-		classad::ArgumentList old_fn_args;
-		classad::ArgumentList new_fn_args;
-		( ( classad::FunctionCall * )tree )->GetComponents( fn_name, old_fn_args );
-		for ( classad::ArgumentList::iterator i = old_fn_args.begin(); i != old_fn_args.end(); i++ ) {
-			new_fn_args.push_back( AddTargetRefs( *i, target_type ) );
-		}
-		return classad::FunctionCall::MakeFunctionCall( fn_name, new_fn_args );
-	}
-	default: {
- 			// old ClassAds have no nested ClassAds or lists
-			// literals have no attrrefs in them
-		return tree->Copy( );
-	}
-	}
-#endif
 }
 
 const char *ConvertEscapingOldToNew( const char *str )
