@@ -50,28 +50,30 @@ class ClassAdFileParseHelper;
 
 bool ClassAdAttributeIsPrivate( char const *name );
 
+typedef std::set<std::string, classad::CaseIgnLTStr> AttrNameSet;
+
 	/** Print the ClassAd as an old ClassAd to the FILE
 		@param file The file handle to print to.
 		@return TRUE
 	*/
-int	fPrintAd(FILE *file, classad::ClassAd &ad, bool exclude_private = false, StringList *attr_white_list = NULL);
+int	fPrintAd(FILE *file, const classad::ClassAd &ad, bool exclude_private = false, StringList *attr_white_list = NULL);
 
 	/** Print the ClassAd as an old ClasAd with dprintf
 		@param level The dprintf level.
 	*/
-void dPrintAd( int level, classad::ClassAd &ad );
+void dPrintAd( int level, const classad::ClassAd &ad, bool exclude_private = true );
 
 	/** Format the ClassAd as an old ClassAd into the MyString.
 		@param output The MyString to write into
 		@return TRUE
 	*/
-int sPrintAd( MyString &output, classad::ClassAd &ad, bool exclude_private = false, StringList *attr_white_list = NULL );
+int sPrintAd( MyString &output, const classad::ClassAd &ad, bool exclude_private = false, StringList *attr_white_list = NULL );
 
 	/** Format the ClassAd as an old ClassAd into the std::string.
 		@param output The std::string to write into
 		@return TRUE
 	*/
-int sPrintAd( std::string &output, classad::ClassAd &ad, bool exclude_private = false, StringList *attr_white_list = NULL );
+int sPrintAd( std::string &output, const classad::ClassAd &ad, bool exclude_private = false, StringList *attr_white_list = NULL );
 
 class ClassAd : public classad::ClassAd
 {
@@ -108,11 +110,6 @@ class ClassAd : public classad::ClassAd
 		 */
 	int Insert(const char *str);
 
-		/** Insert an attribute/value into the ClassAd 
-		 *  @param expr A string of the form "Attribute = Value"
-		 */
-	int InsertOrUpdate(const char *expr) { return Insert(expr); }
-
 		/* Insert an attribute/value into the Classad
 		 */
 	int AssignExpr(char const *name,char const *value);
@@ -142,6 +139,10 @@ class ClassAd : public classad::ClassAd
 
 	int Assign(char const *name,unsigned long value)
 	{ return InsertAttr( name, (long long)value) ? TRUE : FALSE; }
+#ifdef WIN32
+    int Assign(char const *name,unsigned long long value)
+	{ return InsertAttr( name, (long long)value) ? TRUE : FALSE; }
+#endif
 
 	int Assign(char const *name,float value)
 	{ return InsertAttr( name, (double)value) ? TRUE : FALSE; }
@@ -332,36 +333,11 @@ class ClassAd : public classad::ClassAd
 
 	bool initFromString(char const *str,MyString *err_msg=NULL);
 
-		/** Read the old ClassAd from the stream, and fill in this ClassAd.
-		 * @param s the stream
-		 */
-	int initFromStream(Stream& s);
-
-		/** Read the old ClassAd as an old AttrList from the stream,
-		 * and fill in this ClassAd.
-		 * @param s the stream
-		 */
-	int initAttrListFromStream(Stream& s);
-
     void ResetExpr();
 
 	void ResetName();
 	const char *NextNameOriginal();
 
-	/* Create a new ExprTree based on the given one. The new tree will
-	 * be modified suched that any boolean value is converted to a
-	 * literal '0' or '1'.
-	 */
-	bool AddExplicitConditionals( classad::ExprTree *expr, classad::ExprTree *&newExpr );
-
-	// AddExplicitTargets creates a new ClassAd (the caller owns it)
-	// that is similar to the original ClassAd, except that if it refers
-	// to attributes that are not in the current classad and they are not
-	// scoped, then they are renamed "target.attribute"
-	void AddExplicitTargetRefs(  );
-	
-	void RemoveExplicitTargetRefs(  );
-	
 	void AddTargetRefs( TargetAdType target_type, bool do_version_check = true );
 
 	bool NextExpr( const char *&name, ExprTree *&value );
@@ -373,7 +349,7 @@ class ClassAd : public classad::ClassAd
 
 	// Set or clear the dirty flag for each expression.
 	void SetDirtyFlag(const char *name, bool dirty);
-	void GetDirtyFlag(const char *name, bool *exists, bool *dirty);
+	void GetDirtyFlag(const char *name, bool *exists, bool *dirty) const;
 
 	// Copy value of source_attr in source_ad to target_attr
 	// in this ad.  If source_ad is NULL, it defaults to this ad.
@@ -386,12 +362,6 @@ class ClassAd : public classad::ClassAd
 	// CopyAttribute(target_attr,target_attr,source_ad).
 	void CopyAttribute(char const *target_attr, classad::ClassAd *source_ad );
 
-    /** Basically just calls an Unparser so we can escape strings
-     *  @param val The string we're escaping stuff in. 
-     *  @return The escaped string.
-     */
-    static char const *EscapeStringValue(char const *val, MyString &buf);
-
     /** Takes the ad this is chained to, copies over all the 
      *  attributes from the parent ad that aren't in this classad
      *  (so attributes in both the parent ad and this ad retain the 
@@ -401,12 +371,12 @@ class ClassAd : public classad::ClassAd
     void ChainCollapse();
 
     void GetReferences(const char* attr,
-                StringList &internal_refs,
-                StringList &external_refs);
+                StringList *internal_refs,
+                StringList *external_refs) const;
 
     bool GetExprReferences(const char* expr,
-                StringList &internal_refs,
-                StringList &external_refs);
+                StringList *internal_refs,
+                StringList *external_refs) const;
 
 	static void Reconfig();
 	static bool m_initConfig;
@@ -414,7 +384,6 @@ class ClassAd : public classad::ClassAd
 
  private:
 	void evalFromEnvironment( const char *name, classad::Value val );
-	classad::ExprTree *AddExplicitConditionals( classad::ExprTree * );
 
 	enum ItrStateEnum {
 		ItrUninitialized,
@@ -432,8 +401,8 @@ class ClassAd : public classad::ClassAd
     bool m_dirtyItrInit;
 
 	void _GetReferences(classad::ExprTree *tree,
-						StringList &internal_refs,
-						StringList &external_refs);
+						StringList *internal_refs,
+						StringList *external_refs) const;
 
 	// poison Assign of ExprTree* type for public users
 	// otherwise the compiler will resolve against the bool overload 
@@ -498,7 +467,7 @@ bool IsValidAttrName(const char *name);
  * @param An optional white-list of attributes to be printed.
  * @return TRUE as long as the file existed.
  */
-int fPrintAdAsXML(FILE *fp, classad::ClassAd &ad,
+int fPrintAdAsXML(FILE *fp, const classad::ClassAd &ad,
 				  StringList *attr_white_list = NULL);
 
 /* Prints the classad as XML to a string. fPrintAdAsXML calls this.
@@ -508,9 +477,9 @@ int fPrintAdAsXML(FILE *fp, classad::ClassAd &ad,
  * @param An optional white-list of attributes to be printed.
  * @return TRUE
  */
-int sPrintAdAsXML(MyString &output, classad::ClassAd &ad,
+int sPrintAdAsXML(MyString &output, const classad::ClassAd &ad,
 				  StringList *attr_white_list = NULL);
-int sPrintAdAsXML(std::string &output, classad::ClassAd &ad,
+int sPrintAdAsXML(std::string &output, const classad::ClassAd &ad,
 				  StringList *attr_white_list = NULL);
 
 /** Given an attribute name, return a buffer containing the name
@@ -524,6 +493,12 @@ int sPrintAdAsXML(std::string &output, classad::ClassAd &ad,
  */
 char* sPrintExpr(const classad::ClassAd &ad, const char* name);
 
+/** Basically just calls an Unparser so we can escape strings
+ *  @param val The string we're escaping stuff in. 
+ *  @return The escaped string.
+ */
+char const *EscapeAdStringValue(char const *val, std::string &buf);
+
 	/** Set the MyType attribute */
 void SetMyTypeName(classad::ClassAd &ad, const char *);
 	/** Get the value of the MyType attribute */
@@ -534,18 +509,29 @@ void SetTargetTypeName(classad::ClassAd &ad, const char *);
 const char*	GetTargetTypeName(const classad::ClassAd& ad);
 
 
-void getTheMyRef( classad::ClassAd *ad );
-void releaseTheMyRef( classad::ClassAd *ad );
-
 classad::MatchClassAd *getTheMatchAd( classad::ClassAd *source,
 									  classad::ClassAd *target );
 void releaseTheMatchAd();
 
 
+// Modify all expressions in the given ad, such that if they refer
+// to attributes that are not in the current classad and they are not
+// scoped, then they are renamed "target.attribute"
+void AddExplicitTargetRefs( classad::ClassAd &ad );
+
+// Return a new ExprTree identical to the given one, except that for any
+// attributes referred to which are not scoped and don't appear in the
+// given set are renamed "target.attribute".
 classad::ExprTree *AddExplicitTargetRefs(classad::ExprTree *,
 						std::set < std::string, classad::CaseIgnLTStr > & );
 						
-classad::ExprTree *AddExplicitTargetRefs(classad::ExprTree *, classad::ClassAd*);
+// Modify all expressions in the given ad, removing the "target" scope
+// from all attributes references.
+void RemoveExplicitTargetRefs( classad::ClassAd &ad );
+
+// Return a new ExprTree identical to the given one, but removing any
+// "target" scope from all attribute references.
+// For example, Target.Foo will become Foo.
 classad::ExprTree *RemoveExplicitTargetRefs( classad::ExprTree * );
 
 
