@@ -28,6 +28,7 @@
 #include "CondorError.h"
 #include "condor_classad.h"
 #include "quill_enums.h"
+#include "dc_schedd.h"
 
 #ifdef HAVE_EXT_POSTGRESQL
 #include "pgsqldatabase.h"
@@ -217,7 +218,7 @@ fetchQueue (ClassAdList &list, StringList &attrs, ClassAd *ad, CondorError* errs
 	char    		scheddString [32];
 	const char 		*constraint;
 
-	bool useFastPath = false;
+	int useFastPath = 0;
 
 	// make the query ad
 	if ((result = query.makeQuery (tree)) != Q_OK)
@@ -234,7 +235,7 @@ fetchQueue (ClassAdList &list, StringList &attrs, ClassAd *ad, CondorError* errs
 			errstack->push("TEST", 0, "FOO");
 			return Q_SCHEDD_COMMUNICATION_ERROR;
 		}
-		useFastPath = true;
+		useFastPath = 2;
 	}
 	else
 	{
@@ -248,7 +249,7 @@ fetchQueue (ClassAdList &list, StringList &attrs, ClassAd *ad, CondorError* errs
 	}
 
 	// get the ads and filter them
-	getAndFilterAds (constraint, attrs, list, useFastPath);
+	getAndFilterAds (constraint, attrs, -1, list, useFastPath);
 
 	DisconnectQ (qmgr);
 	return Q_OK;
@@ -279,14 +280,17 @@ fetchQueueFromHost (ClassAdList &list, StringList &attrs, const char *host, char
 	if( !(qmgr = ConnectQ( host, connect_timeout, true, errstack)) )
 		return Q_SCHEDD_COMMUNICATION_ERROR;
 
-	bool useFastPath = false;
+	int useFastPath = 0;
 	if( schedd_version && *schedd_version ) {
 		CondorVersionInfo v(schedd_version);
-		useFastPath = v.built_since_version(6,9,3);
+		useFastPath = v.built_since_version(6,9,3) ? 1 : 0;
+		if (v.built_since_version(8, 1, 5)) {
+			useFastPath = 2;
+		}
 	}
 
 	// get the ads and filter them
-	result = getAndFilterAds (constraint, attrs, list, useFastPath);
+	result = getAndFilterAds (constraint, attrs, -1, list, useFastPath);
 
 	DisconnectQ (qmgr);
 	return result;
@@ -355,9 +359,11 @@ CondorQ::fetchQueueFromDB (ClassAdList &list,
 int
 CondorQ::fetchQueueFromHostAndProcess ( const char *host,
 										StringList &attrs,
+										int fetch_opts,
+										int match_limit,
 										condor_q_process_func process_func,
 										void * process_func_data,
-										bool useFastPath,
+										int useFastPath,
 										CondorError* errstack)
 {
 	Qmgr_connection *qmgr;
@@ -371,6 +377,17 @@ CondorQ::fetchQueueFromHostAndProcess ( const char *host,
 	constraint = strdup( ExprTreeToString( tree ) );
 	delete tree;
 
+	if (useFastPath == 2) {
+		int result = fetchQueueFromHostAndProcessV2(host, constraint, attrs, fetch_opts, match_limit, process_func, process_func_data, connect_timeout, errstack);
+		free( constraint);
+		return result;
+	}
+
+	if (fetch_opts != fetch_Default) {
+		free( constraint );
+		return Q_UNSUPPORTED_OPTION_ERROR;
+	}
+
 	/*
 	 connect to the Q manager.
 	 use a timeout of 20 seconds, and a read-only connection.
@@ -385,13 +402,92 @@ CondorQ::fetchQueueFromHostAndProcess ( const char *host,
 	}
 
 	// get the ads and filter them
-	result = getFilterAndProcessAds (constraint, attrs, process_func, process_func_data, useFastPath);
+	result = getFilterAndProcessAds (constraint, attrs, match_limit, process_func, process_func_data, useFastPath);
 
 	DisconnectQ (qmgr);
 	free( constraint );
 	return result;
 }
 
+int
+CondorQ::fetchQueueFromHostAndProcessV2(const char *host,
+					const char *constraint,
+					StringList &attrs,
+					int fetch_opts,
+					int match_limit,
+					condor_q_process_func process_func,
+					void * process_func_data,
+					int connect_timeout,
+					CondorError *errstack)
+{
+	classad::ClassAdParser parser;
+	classad::ExprTree *expr = NULL;
+	parser.ParseExpression(constraint, expr);
+	if (!expr) return Q_INVALID_REQUIREMENTS;
+
+	classad::ClassAd ad;
+	ad.Insert(ATTR_REQUIREMENTS, expr);
+
+	char *projection = attrs.print_to_delimed_string(",");
+	if (projection) {
+		ad.InsertAttr(ATTR_PROJECTION, projection);
+		free(projection);
+	}
+
+	if (fetch_opts == fetch_DefaultAutoCluster) {
+		ad.InsertAttr("QueryDefaultAutocluster", true);
+		ad.InsertAttr("MaxReturnedJobIds", 2); // TODO: make this settable by caller of this function.
+	} else if (fetch_opts == fetch_GroupBy) {
+		ad.InsertAttr("ProjectionIsGroupBy", true);
+		ad.InsertAttr("MaxReturnedJobIds", 2); // TODO: make this settable by caller of this function.
+	}
+
+	if (match_limit >= 0)
+	{
+		ad.InsertAttr(ATTR_LIMIT_RESULTS, match_limit);
+	}
+
+	DCSchedd schedd(host);
+	Sock* sock;
+	if (!(sock = schedd.startCommand(QUERY_JOB_ADS, Stream::reli_sock, connect_timeout, errstack))) return Q_SCHEDD_COMMUNICATION_ERROR;
+
+	classad_shared_ptr<Sock> sock_sentry(sock);
+
+	if (!putClassAd(sock, ad) || !sock->end_of_message()) return Q_SCHEDD_COMMUNICATION_ERROR;
+	dprintf(D_FULLDEBUG, "Sent classad to schedd\n");
+
+	int rval = 0;
+	do {
+		ClassAd * ad = new ClassAd();
+		if ( ! getClassAd(sock, *ad) || ! sock->end_of_message()) {
+			delete(ad);
+			rval = Q_SCHEDD_COMMUNICATION_ERROR;
+			break;
+		}
+		dprintf(D_FULLDEBUG, "Got classad from schedd.\n");
+		long long intVal;
+		if (ad->EvaluateAttrInt(ATTR_OWNER, intVal) && (intVal == 0))
+		{ // Last ad.
+			sock->close();
+			dprintf(D_FULLDEBUG, "Ad was last one from schedd.\n");
+			std::string errorMsg;
+			if (ad->EvaluateAttrInt(ATTR_ERROR_CODE, intVal) && intVal && ad->EvaluateAttrString(ATTR_ERROR_STRING, errorMsg))
+			{
+				delete(ad);
+				if (errstack) errstack->push("TOOL", intVal, errorMsg.c_str());
+				return Q_REMOTE_ERROR;
+			}
+			break;
+		}
+		if (process_func(process_func_data, ad)) {
+			delete(ad);
+		}
+		ad = NULL;
+	} while (true);
+
+	return rval;
+}
+
 int
 CondorQ::fetchQueueFromDBAndProcess ( const char *dbconn,
 									  char *&lastUpdate,
@@ -447,7 +543,7 @@ CondorQ::fetchQueueFromDBAndProcess ( const char *dbconn,
 	
 	while (ad != (ClassAd *) 0) {
 			// Process the data and insert it into the list
-		if ((*process_func) (ad, process_func_data) ) {
+		if ((*process_func) (process_func_data, ad) ) {
 			ad->Clear();
 			delete ad;
 		}
@@ -542,11 +638,12 @@ CondorQ::rawDBQuery(const char *dbconn, CondorQQueryType qType)
 int
 CondorQ::getFilterAndProcessAds( const char *constraint,
 								 StringList &attrs,
+								 int match_limit,
 								 condor_q_process_func process_func,
 								 void * process_func_data,
 								 bool useAll )
 {
-	ClassAd *ad;
+	int match_count = 0;
 
 	if (useAll) {
 			// The fast case with the new protocol
@@ -555,34 +652,41 @@ CondorQ::getFilterAndProcessAds( const char *constraint,
 		free(attrs_str);
 
 		while( true ) {
-			ad = new ClassAd;
+			ClassAd * ad = new ClassAd();
+			if (match_limit >= 0 && match_count >= match_limit)
+				break;
 			if( GetAllJobsByConstraint_Next( *ad ) != 0 ) {
-				delete ad;
 				break;
 			}
-			if ( ( *process_func )( process_func_data, ad ) ) {
+			++match_count;
+			if (process_func(process_func_data, ad)) {
 				delete(ad);
 			}
+			ad = NULL;
 		}
 	} else {
 
-	// slow case, using old protocol
-	if ((ad = GetNextJobByConstraint(constraint, 1)) != NULL) {
-		// Process the data and insert it into the list
-		if ( ( *process_func )( process_func_data, ad ) ) {
-			delete(ad);
-		}
-
-		while((ad = GetNextJobByConstraint(constraint, 0)) != NULL) {
+		// slow case, using old protocol
+		ClassAd * ad = GetNextJobByConstraint(constraint, 1);
+		if (ad) {
 			// Process the data and insert it into the list
-			if ( ( *process_func )( process_func_data, ad ) ) {
-				delete(ad);
+			if (process_func(process_func_data, ad)) {
+				delete ad;
+			}
+			ad = NULL;
+			++match_count;
+
+			while ((ad = GetNextJobByConstraint(constraint, 0)) != NULL) {
+				if (match_limit >= 0 && match_count >= match_limit)
+					break;
+				// Process the data and insert it into the list
+				if (process_func(process_func_data, ad)) {
+					delete ad;
+				}
 			}
 		}
 	}
 
-	}
-
 	// here GetNextJobByConstraint returned NULL.  check if it was
 	// because of the network or not.  if qmgmt had a problem with
 	// the net, then errno is set to ETIMEDOUT, and we should fail.
@@ -597,23 +701,29 @@ CondorQ::getFilterAndProcessAds( const char *constraint,
 int
 CondorQ::getAndFilterAds (const char *constraint,
 						  StringList &attrs,
+						  int match_limit,
 						  ClassAdList &list,
-						  bool useAllJobs)
+						  int useAllJobs)
 {
-	if (useAllJobs) {
-	char *attrs_str = attrs.print_to_delimed_string();
-	GetAllJobsByConstraint(constraint, attrs_str, list);
-	free(attrs_str);
+	if (useAllJobs == 1) {
+		char *attrs_str = attrs.print_to_delimed_string();
+		GetAllJobsByConstraint(constraint, attrs_str, list);
+		free(attrs_str);
 
 	} else {
-	ClassAd		*ad;
-	if ((ad = GetNextJobByConstraint(constraint, 1)) != NULL) {
-		list.Insert(ad);
-		while((ad = GetNextJobByConstraint(constraint, 0)) != NULL) {
+		ClassAd		*ad;
+		int match_count = 0;
+		if ((ad = GetNextJobByConstraint(constraint, 1)) != NULL) {
 			list.Insert(ad);
+			++match_count;
+			while((ad = GetNextJobByConstraint(constraint, 0)) != NULL) {
+				if (match_limit > 0 && match_count >= match_limit)
+					break;
+				list.Insert(ad);
+				++match_count;
+			}
 		}
 	}
-	}
 
 	// here GetNextJobByConstraint returned NULL.  check if it was
 	// because of the network or not.  if qmgmt had a problem with
