@@ -142,28 +142,6 @@ split_sin( const char *addr, char **host, char **port, char **params )
 	return 1;
 }
 
-//int
-//address_to_sin(char const *host, char const *port, struct sockaddr_in *sa_in)
-//{
-//	struct  hostent *hostptr;
-//	if( !host || !port ) {
-//		return 0;
-//	}
-//	if ( !is_ipaddr(host,NULL) &&
-//		((hostptr=condor_gethostbyname(host)) != NULL &&
-//		 hostptr->h_addrtype==AF_INET) )
-//	{
-//			sa_in->sin_addr = *(struct in_addr *)(hostptr->h_addr_list[0]);
-//	}
-//	else if( !condor_inet_aton(host, &sa_in->sin_addr) ) {
-//		return 0;
-//	}
-//
-//	sa_in->sin_port = htons((short)atoi(port));
-//	sa_in->sin_family = AF_INET;
-//
-//	return 1;
-//}
 
 /* Convert a string of the form "<xx.xx.xx.xx:pppp?params>" to a
   sockaddr_in TCP (Also allow strings of the form "<hostname:pppp?params>")
@@ -192,20 +170,6 @@ split_sin( const char *addr, char **host, char **port, char **params )
 //	return result;
 //}
 
-//char *
-//sin_to_ipstring(const struct sockaddr_in *sa_in,char *buf,size_t buflen)
-//{
-//	char const *ipstr = inet_ntoa(sa_in->sin_addr);
-//	if( !ipstr || strlen(ipstr)>=buflen ) {
-//		if( buflen > 0 ) {
-//			buf[0] = '\0';
-//		}
-//		return NULL;
-//	}
-//	strcpy(buf,ipstr);
-//	return buf;
-//}
-
 /* This function has a unit test. */
 //char *
 //sin_to_string(const struct sockaddr_in *sa_in)
@@ -248,29 +212,6 @@ sock_peer_to_string( SOCKET fd, char *buf, size_t buflen, char const *unknown )
 
 	addr.to_sinful(buf, buflen);
 	return buf;
-
-	/* This value is never used.
-	char const *sinful = sock_to_string( fd );
-	*/
-
-//	struct sockaddr_in who;
-//	SOCKET_LENGTH_TYPE addr_len;
-//
-//	addr_len = sizeof(who);
-//	memset(buf,0,buflen);
-//	if( getpeername(fd, (struct sockaddr *)&who, (socklen_t*)&addr_len) == 0) {
-//		if( who.sin_family == AF_INET ) {
-//			char const *sinful = sin_to_string( &who );
-//			if( sinful ) {
-//				strncpy(buf, sinful, buflen );
-//				if(buflen) {
-//					buf[buflen-1] = '\0'; /* ensure null termination */
-//				}
-//				return buf;
-//			}
-//		}
-//	}
-//	return unknown;
 }
 
 
@@ -540,7 +481,7 @@ NOTE: it looks like sin_addr may be modified even if the return
 int
 is_valid_sinful( const char *sinful )
 {
-	dprintf(D_HOSTNAME, "validate %s\n", sinful);
+	dprintf(D_HOSTNAME, "Checking if %s is a sinful address\n", sinful);
 	char addrbuf[INET6_ADDRSTRLEN];
 	const char* acc = sinful;
 	const char* tmp;
@@ -550,31 +491,31 @@ is_valid_sinful( const char *sinful )
 		return FALSE;
 
 	if (*acc != '<') {
-		dprintf(D_HOSTNAME, "is not begin with <\n");
+		dprintf(D_HOSTNAME, "%s is not a sinful address: does not begin with \"<\"\n", sinful);
 		return FALSE;
 	}
 	acc++;
 	if (*acc == '[') {
-		dprintf(D_HOSTNAME, "ipv6 address\n");
+		dprintf(D_HOSTNAME, "%s is an ipv6 address\n", sinful);
 		tmp = strchr(acc, ']');
 		if (!tmp) {
-			dprintf(D_HOSTNAME, "could not find ]\n");
+			dprintf(D_HOSTNAME, "%s is not a sinful address: could not find closing \"]\"\n", sinful);
 			return FALSE;
 		}
 		addr_begin = acc + 1;
 		addr_end = tmp;
 		// too long
 		if (addr_end - addr_begin > INET6_ADDRSTRLEN) {
-			dprintf(D_HOSTNAME, "addr too long %d\n", (int)(addr_end - addr_begin));
+			dprintf(D_HOSTNAME, "%s is not a sinful address: addr too long %d\n", sinful, (int)(addr_end - addr_begin));
 			return FALSE;
 		}
 
 		strncpy(addrbuf, addr_begin, addr_end - addr_begin);
 		addrbuf[addr_end - addr_begin] = '\0';
-		dprintf(D_HOSTNAME, "try to convert using inet_pton, %s\n", addrbuf);
+		dprintf(D_HOSTNAME, "tring to convert %s using inet_pton, %s\n", sinful, addrbuf);
 		in6_addr tmp_addr;
 		if (inet_pton(AF_INET6, addrbuf, &tmp_addr) <= 0) {
-			dprintf(D_HOSTNAME, "inet_pton failed\n");
+			dprintf(D_HOSTNAME, "%s is not a sinful address: inet_pton(AF_INET6, %s) failed\n", sinful, addrbuf);
 			return FALSE;
 		}
 		acc = tmp + 1;
@@ -589,16 +530,16 @@ is_valid_sinful( const char *sinful )
 		acc = acc + colon_pos;
 	}
 	if (*acc != ':') {
-		dprintf(D_HOSTNAME, "no colon found\n");
+		dprintf(D_HOSTNAME, "%s is not a sinful address: no colon found\n", sinful);
 		return FALSE;
 	}
 
 	tmp = strchr(acc, '>');
 	if (!tmp) {
-		dprintf(D_HOSTNAME, "no > found\n");
+		dprintf(D_HOSTNAME, "%s is not a sinful address: no closing \">\" found\n", sinful);
 		return FALSE;
 	}
-	dprintf(D_HOSTNAME, "success\n");
+	dprintf(D_HOSTNAME, "%s is a sinful address!\n", sinful);
 	return TRUE;
 }
 
@@ -710,10 +651,6 @@ typedef union {
 
 /* Bind the given fd to the correct local interface. */
 
-// _condor_local_bind(), get_port_range() and bindWithin()
-// should be ported to IPv6. However, it will be delayed until
-// internet.c becomes internet.cpp.
-
 // [IPV6] Ported
 int
 _condor_local_bind( int is_outgoing, int fd )
@@ -781,114 +718,54 @@ _condor_local_bind( int is_outgoing, int fd )
 	return TRUE;
 }
 
+int bindWithin( const int fd, const int lowPort, const int highPort ) {
+	int pid = (int)getpid();
+	int range = highPort - lowPort + 1;
+	int initialPort = lowPort + (pid * 173 % range);
 
-int bindWithin(const int fd, const int low_port, const int high_port)
-{
-	int start_trial, this_trial;
-	int pid, range;
+	condor_sockaddr initializedSA;
+	int rv = condor_getsockname( fd, initializedSA );
+	if( rv != 0 ) {
+		dprintf( D_ALWAYS, "_condor_local_bind::bindWithin() - getsockname() failed.\n" );
+		return FALSE;
+	}
+	initializedSA.set_addr_any();
 
-	// Use hash function with pid to get the starting point
-    pid = (int) getpid();
-    range = high_port - low_port + 1;
-    // this line must be changed to use the hash function of condor
-    start_trial = low_port + (pid * 173/*some prime number*/ % range);
-
-    this_trial = start_trial;
+	int trialPort = initialPort;
 	do {
-		struct sockaddr_in sa_in;
-		priv_state old_priv;
-		int bind_return_value;
+		condor_sockaddr trialSA = initializedSA;
+		trialSA.set_port( trialPort++ );
 
-		memset(&sa_in, 0, sizeof(sa_in));
-		sa_in.sin_family = AF_INET;
-		sa_in.sin_addr.s_addr = htonl(INADDR_ANY);
-		sa_in.sin_port = htons((u_short)this_trial++);
-
-// windows doesn't have privileged ports.
 #ifndef WIN32
-		if (this_trial <= 1024) {
+		priv_state oldPriv = PRIV_UNKNOWN;
+		if( trialPort <= 1024 ) {
 			// use root priv for the call to bind to allow privileged ports
-			old_priv = PRIV_UNKNOWN;
-			old_priv = set_root_priv();
+			oldPriv = set_root_priv();
 		}
 #endif
-		bind_return_value = bind(fd, (struct sockaddr *)&sa_in, sizeof(sa_in));
+
+		rv = bind( fd, trialSA.to_sockaddr(), trialSA.get_socklen() );
+
 #ifndef WIN32
-		if (this_trial <= 1024) {
-			set_priv (old_priv);
+		if( trialPort <= 1024 ) {
+			set_priv( oldPriv );
 		}
 #endif
-		if (bind_return_value == 0) { // success
-			dprintf(D_NETWORK, "_condor_local_bind - bound to %d...\n", this_trial-1);
+
+		if( rv == 0 ) {
+			dprintf( D_NETWORK, "_condor_local_bind::bindWithin(): bound to %d\n", trialPort - 1 );
 			return TRUE;
 		} else {
-            dprintf(D_NETWORK, "_condor_local_bind - failed to bind: %s\n", strerror(errno));
-        }
-		if ( this_trial > high_port )
-			this_trial = low_port;
-    } while(this_trial != start_trial);
+			dprintf( D_NETWORK, "_condor_local_bind::bindWithin(): failed to bind to %d (%s)\n", trialPort - 1, strerror(errno) );
+		}
 
-	dprintf(D_ALWAYS, "_condor_local_bind::bindWithin - failed to bind any port within (%d ~ %d)\n",
-	        low_port, high_port);
+		if( trialPort > highPort ) { trialPort = lowPort; }
+	} while( trialPort != initialPort );
 
+	dprintf( D_ALWAYS, "_condor_local_bind::bindWithin() - failed to bind any port within (%d ~ %d)\n", lowPort, highPort );
 	return FALSE;
 }
 
-/* Check if the ip is in private ip address space */
-/* ip: in host byte order */
-/* This function has a unit test. */
-int
-is_priv_net(uint32_t ip)
-{
-    return ((ip & 0xFF000000) == 0x0A000000 ||      // 10/8
-            (ip & 0xFFF00000) == 0xAC100000 ||      // 172.16/12
-            (ip & 0xFFFF0000) == 0xC0A80000);       // 192.168/16
-}
-
-int
-is_loopback_net(uint32_t ip)
-{
-    return ((ip & 0xFF000000) == 0x7F000000); // 127/8
-}
-
-int is_loopback_net_str(char const * /*ip*/)
-{
-//	struct in_addr sa;
-//	if( is_ipaddr_no_wildcard(ip,&sa) ) {
-//		return is_loopback_net(ntohl(sa.s_addr));
-//	}
-	return 0;
-}
-
-/* Check if two ip addresses, given in network byte, are in the same network */
-int
-in_same_net(uint32_t ipA, uint32_t ipB)
-{
-    unsigned char *byteA, *fA, *byteB;
-    int i, index_to;
-
-    fA = byteA = (unsigned char *)&ipA;
-    byteB = (unsigned char *)&ipB;
-
-    if (*fA < 128) { // A class
-        index_to = 1;
-    } else if(*fA < 192) { // B class
-        index_to = 2;
-    } else {    // C class
-        index_to = 3;
-    }
-
-    for (i = 0; i < index_to; i++) {
-        if (*byteA != *byteB) {
-            return 0;
-        }
-        byteA++;
-        byteB++;
-    }
-
-    return 1;
-}
-
 // ip: network-byte order
 // port: network-byte order
 char * ipport_to_string(const unsigned int ip, const unsigned short port)
@@ -908,26 +785,6 @@ char * ipport_to_string(const unsigned int ip, const unsigned short port)
     return buf;
 }
 
-char *
-prt_fds(int maxfd, fd_set *fds)
-{
-    static char buf[50];
-    int i, size;
-
-    sprintf(buf, "<");
-    for(i=0; i<maxfd; i++) {
-        if (fds && FD_ISSET(i, fds)) {
-            if ((size = strlen(buf)) > 40) {
-                strcat(buf, "...>");
-                return buf;
-            }
-        sprintf(&buf[strlen(buf)], "%d ", i);
-        }
-    }
-    strcat(buf, ">");
-    return buf;
-}
-
 /* This function has a unit test. */
 int
 getPortFromAddr( const char* addr )
@@ -1069,21 +926,18 @@ getSockAddr(int sockfd)
     // if getsockname returns INADDR_ANY, we rely upon get_local_addr() since returning
     // 0.0.0.0 is not a good idea.
     if (sa_in.sin_addr.s_addr == ntohl(INADDR_ANY)) {
-    	condor_sockaddr myaddr = get_local_ipaddr();
-    	sa_in.sin_addr = myaddr.to_sin().sin_addr;
+        // This is standad universe -only code at this point, so we know
+        // we want an IPv4 address.
+        condor_sockaddr myaddr = get_local_ipaddr( CP_IPV4 );
+        sa_in.sin_addr = myaddr.to_sin().sin_addr;
+        // This can happen, I think, if we're running in IPv6-only mode.
+        // It may make sense to check for ENABLE_IPV4 (if not an actual
+        // IPv4 interface) when the checkpoint server starts up, instead.
+        assert( sa_in.sin_addr.s_addr != ntohl(INADDR_ANY) );
     }
     return &sa_in;
 }
 
-//int
-//condor_inet_aton(const char *ipstr, struct in_addr *result)
-//{
-//	if( !is_ipaddr(ipstr,result) ) {
-//		return 0;
-//	}
-//	return 1;
-//}
-
 int generate_sinful(char* buf, int len, const char* ip, int port) {
 	if (strchr(ip, ':')) {
 		return snprintf(buf, len, "<[%s]:%d>", ip, port);
