@@ -111,6 +111,11 @@ int condor_fsync(int, const char*)
 	return 0;
 }
 
+int condor_fdatasync(int, const char*)
+{
+	return 0;
+}
+
 BEGIN_C_DECLS
 void
 config( int, bool )
@@ -267,7 +272,7 @@ int Stream::get_string_ptr(char const *&){ return not_impl(); }
 void Stream::prepare_crypto_for_secret(){not_impl();}
 void Stream::restore_crypto_after_secret(){not_impl();}
 bool Stream::prepare_crypto_for_secret_is_noop(){not_impl();return true;}
-void Stream::set_crypto_mode(bool /*enabled*/){not_impl();}
+bool Stream::set_crypto_mode(bool /*enabled*/){not_impl();return false;}
 bool Stream::get_encryption() const{not_impl();return false;}
 int Stream::put_secret( char const * /*s*/ ){not_impl();return 0;}
 int Stream::get_secret( char *& /*s*/ ){not_impl();return 0;}
@@ -353,7 +358,7 @@ void ConvertDefaultIPToSocketIP(char const *,std::string &,Stream&) {
 #include "Regex.h"
 
 Regex::Regex() {not_impl();}
-Regex::~Regex() {not_impl();}
+Regex::~Regex() {}
 bool Regex::compile(MyString const& , char const** , int* , int ) {not_impl();return false;}
 bool Regex::match(MyString const& , ExtArray<MyString>* ) {not_impl();return false;}
 
@@ -396,5 +401,10 @@ int pthread_sigmask(int how, const sigset_t *newmask, sigset_t *oldmask)
 }
 #endif
 
+int my_spawnl( const char*, ... )
+{ return not_impl(); }
 
 END_C_DECLS
+
+void statusString( int, MyString & )
+{ not_impl(); }
