@@ -206,9 +206,10 @@ int perm::userInLocalGroup( const char *account, const char *domain, const char
 	NET_API_STATUS status;
 	wchar_t group_name_unicode[MAX_GROUP_LENGTH+1];
 	_snwprintf(group_name_unicode, MAX_GROUP_LENGTH+1, L"%S", group_name);
+	group_name_unicode[MAX_GROUP_LENGTH] = 0;
 	
 	
-	DWORD resume_handle = 0;
+	DWORD_PTR resume_handle = 0;
 	
 	do {	 // loop until we have checked all the group members
 		
@@ -290,6 +291,8 @@ int perm::userInGlobalGroup( const char *account, const char *domain, const char
 	wchar_t group_name_unicode[MAX_GROUP_LENGTH+1];	// groups limited to 256 chars
 	_snwprintf(group_domain_unicode, MAX_DOMAIN_LENGTH+1, L"%S", group_domain);
 	_snwprintf(group_name_unicode, MAX_GROUP_LENGTH+1, L"%S", group_name);
+	group_domain_unicode[MAX_DOMAIN_LENGTH] = 0;
+	group_name_unicode[MAX_GROUP_LENGTH] = 0;
 	
 	GROUP_USERS_INFO_0 *group_members;
 	unsigned long entries_read, total_entries;
@@ -507,6 +510,7 @@ bool perm::init( const char *accountname, const char *domain )
 	must_freesid = false;
 	
 	psid = (PSID) &sidBuffer;
+	sidBufferSize = sizeof(sidBuffer);
 
 	dprintf(D_FULLDEBUG,"perm::init() starting up for account (%s) "
 			"domain (%s)\n", accountname, ( domain ? domain : "NULL"));
@@ -535,9 +539,11 @@ bool perm::init( const char *accountname, const char *domain )
 
 		// no domain specified, so just copy the accountname.
 		Domain_name = NULL;
-		strncpy(qualified_account, accountname, 1023);
+		strncpy(qualified_account, accountname, 1024);
+		qualified_account[1024-1] = 0;
 	}
 
+	domainBufferSize = COUNTOF(domainBuffer);
 	if ( !LookupAccountName( NULL,			// System
 		qualified_account,					// Account name
 		psid, &sidBufferSize,				// Sid
@@ -773,7 +779,9 @@ perm::set_acls( const char *filename )
 	}
 
 	// first get the file's old DACL so we can copy it into the new one.
-
+	// NOTE: oldDACL will be a pointer into the pSD structure - the code below
+	// will eventually call LocalFree(pSD), but should not call LocalFree(oldDACL) because
+	// the oldDACL is deallocated when the pSD is deallocated.
 	err = GetNamedSecurityInfo((char*)filename, SE_FILE_OBJECT,
 			DACL_SECURITY_INFORMATION, NULL, NULL, &oldDACL, NULL, &pSD);
 
@@ -793,7 +801,7 @@ perm::set_acls( const char *filename )
 	if ( ERROR_SUCCESS != err ) {
 		dprintf(D_ALWAYS, "perm::set_acls(%s): failed to get entries from ACL. "
 				"err=%d\n", filename, err);
-		LocalFree(oldDACL);
+		LocalFree(pSD);
 		return false;
 	}
 
@@ -810,7 +818,7 @@ perm::set_acls( const char *filename )
 					"in the ACL, so skipping the add\n");
 
 			LocalFree(entryList);
-			LocalFree(oldDACL);
+			LocalFree(pSD);
 			return true;
 		}
 	}
@@ -837,7 +845,7 @@ perm::set_acls( const char *filename )
 		dprintf(D_ALWAYS, "perm::set_acls(%s): failed to add new ACE "
 				"(err=%d)\n", filename, err);
 
-		LocalFree(oldDACL);
+		LocalFree(pSD);
 		return false;
 	}
 
@@ -878,7 +886,7 @@ perm::set_acls( const char *filename )
 	}
 
 	// clean up our memory.
-	LocalFree(oldDACL);
+	LocalFree(pSD);
 	LocalFree(newDACL);
 
 	if (err != ERROR_SUCCESS)
