@@ -304,7 +304,8 @@ MultiLogFiles::FileReader::Open( const MyString &filename )
 bool
 MultiLogFiles::FileReader::NextLogicalLine( MyString &line )
 {
-	char *tmpLine = getline( _fp );
+	int lines_read = 0;
+	char *tmpLine = getline_trim( _fp, lines_read );
 	if ( tmpLine != NULL ) {
 		line = tmpLine;
 		return true;
@@ -416,108 +417,6 @@ MultiLogFiles::readFileToString(const MyString &strFilename)
 	return strToReturn;
 }
 
-///////////////////////////////////////////////////////////////////////////////
-// Note: this method should get speeded up (see Gnats PR 846).
-MyString
-MultiLogFiles::loadLogFileNameFromSubFile(const MyString &strSubFilename,
-		const MyString &directory, bool &isXml, bool usingDefaultNode)
-{
-	dprintf( D_FULLDEBUG, "MultiLogFiles::loadLogFileNameFromSubFile(%s, %s)\n",
-				strSubFilename.Value(), directory.Value() );
-
-	TmpDir		td;
-	if ( directory != "" ) {
-		MyString	errMsg;
-		if ( !td.Cd2TmpDir(directory.Value(), errMsg) ) {
-			dprintf(D_ALWAYS, "Error from Cd2TmpDir: %s\n", errMsg.Value());
-			return "";
-		}
-	}
-
-	StringList	logicalLines;
-	if ( fileNameToLogicalLines( strSubFilename, logicalLines ) != "" ) {
-		return "";
-	}
-
-	MyString	logFileName("");
-	MyString	initialDir("");
-	MyString	isXmlLogStr("");
-
-		// Now look through the submit file logical lines to find the
-		// log file and initial directory (if specified) and combine
-		// them into a path to the log file that's either absolute or
-		// relative to the DAG submit directory.  Also look for log_xml.
-	const char *logicalLine;
-	while( (logicalLine = logicalLines.next()) != NULL ) {
-		MyString	submitLine(logicalLine);
-		MyString	tmpLogName = getParamFromSubmitLine(submitLine, "log");
-		if ( tmpLogName != "" ) {
-			logFileName = tmpLogName;
-		}
-
-			// If we are using the default node log, we don't care
-			// about these
-		if( !usingDefaultNode ) {
-			MyString	tmpInitialDir = getParamFromSubmitLine(submitLine,
-					"initialdir");
-			if ( tmpInitialDir != "" ) {
-				initialDir = tmpInitialDir;
-			}
-
-			MyString tmpLogXml = getParamFromSubmitLine(submitLine, "log_xml");
-			if ( tmpLogXml != "" ) {
-				isXmlLogStr = tmpLogXml;
-			}
-		}
-	}
-
-	if ( !usingDefaultNode ) {
-			//
-			// Check for macros in the log file name -- we currently don't
-			// handle those.
-			//
-			// If we are using the default node, we don't need to check this
-		if ( logFileName != "" ) {
-			if ( strstr(logFileName.Value(), "$(") ) {
-				dprintf(D_ALWAYS, "MultiLogFiles: macros ('$(...') not allowed "
-						"in log file name (%s) in DAG node submit files\n",
-						logFileName.Value());
-				logFileName = "";
-			}
-		}
-
-			// Do not need to prepend initialdir if we are using the 
-			// default node log
-		if ( logFileName != "" ) {
-				// Prepend initialdir to log file name if log file name is not
-				// an absolute path.
-			if ( initialDir != "" && !fullpath(logFileName.Value()) ) {
-				logFileName = initialDir + DIR_DELIM_STRING + logFileName;
-			}
-
-				// We do this in case the same log file is specified with a
-				// relative and an absolute path.  
-				// Note: we now do further checking that doesn't rely on
-				// comparing paths to the log files.  wenger 2004-05-27.
-			CondorError errstack;
-			if ( !makePathAbsolute( logFileName, errstack ) ) {
-				dprintf(D_ALWAYS, "%s\n", errstack.getFullText().c_str());
-				return "";
-			}
-		}
-		isXmlLogStr.lower_case();
-		isXml = (isXmlLogStr == "true");
-		if ( directory != "" ) {
-			MyString	errMsg;
-			if ( !td.Cd2MainDir(errMsg) ) {
-				dprintf(D_ALWAYS, "Error from Cd2MainDir: %s\n", errMsg.Value());
-				return "";
-			}
-		}
-	}
-	return logFileName;
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 // Note: this method should get speeded up (see Gnats PR 846).
 
@@ -624,7 +523,7 @@ MultiLogFiles::readFile(char const *filename,std::string& buf)
 	}
 
     while(1) {
-        size_t n = read(fd,chunk,sizeof(chunk)-1);
+        ssize_t n = read(fd,chunk,sizeof(chunk)-1);
         if(n>0) {
             chunk[n] = '\0';
             buf += chunk;
@@ -645,230 +544,6 @@ MultiLogFiles::readFile(char const *filename,std::string& buf)
     return rtnVal;
 }
 
-// Gets the log files from a Stork submit file.
-MyString
-MultiLogFiles::loadLogFileNamesFromStorkSubFile(
-		const MyString &strSubFilename,
-		const MyString &directory,
-		StringList &listLogFilenames)
-{
-	MyString rtnVal;
-	MyString path;
-	std::string adBuf;
-	classad::ClassAdParser parser;
-	classad::PrettyPrint unparser;
-	std::string unparsed;
-
-	dprintf( D_FULLDEBUG,
-			"MultiLogFiles::loadLogFileNamesFromStorkSubFile(%s, %s)\n",
-				strSubFilename.Value(), directory.Value() );
-
-	// Construct fully qualified path from directory and log file.
-	if ( directory.Length() > 0 ) {
-		path = directory + DIR_DELIM_STRING;
-	}
-	path += strSubFilename;
-
-	// Read submit file into std::string buffer, the native input buffer for
-	// the [new] ClassAds parser.
-	rtnVal = MultiLogFiles::readFile( path.Value(), adBuf);
-	if (rtnVal.Length() > 0 ) {
-		return rtnVal;
-	}
-
-	// read all classads out of the input file
-    int offset = 0;
-    classad::ClassAd ad;
-
-	// Loop through the Stork submit file, parsing out one submit job [ClassAd]
-	// at a time.
-	skip_whitespace(adBuf,offset);  // until the parser can do this itself
-    while (parser.ParseClassAd(adBuf, ad, offset) ) {
-		std::string logfile;
-
-		// ad now contains the next Stork job ClassAd.  Extract log file, if
-		// found.
-		if ( ! ad.EvaluateAttrString("log", logfile) ) {
-			// no log file specified
-			continue;
-		}
-
-		// reject empty log file names
-		if ( logfile.empty() ) {
-			unparser.Unparse( unparsed, &ad);
-			rtnVal.formatstr("Stork job specifies null log file:%s",
-					unparsed.c_str() );
-			return rtnVal;
-		}
-
-		// reject log file names with embedded macros
-		if ( logfile.find('$') != std::string::npos) {
-			unparser.Unparse( unparsed, &ad);
-			rtnVal.formatstr("macros not allowed in Stork log file names:%s",
-					unparsed.c_str() );
-			return rtnVal;
-		}
-
-		// All logfile must be fully qualified paths.  Prepend the current
-		// working directory if logfile not a fully qualified path.
-		if ( ! fullpath(logfile.c_str() ) ) {
-			MyString	currentDir;
-			if ( ! condor_getcwd(currentDir) ) {
-				rtnVal.formatstr("condor_getcwd() failed with errno %d (%s)",
-						errno, strerror(errno));
-				dprintf(D_ALWAYS, "ERROR: %s at %s:%d\n", rtnVal.Value(),
-						__FILE__, __LINE__);
-				return rtnVal;
-			}
-			std::string tmp  = currentDir.Value();
-			tmp += DIR_DELIM_STRING;
-			tmp += logfile;
-			logfile = tmp;
-		}
-
-		// Add the log file we just found to the log file list
-		// (if it's not already in the list -- we don't want
-		// duplicates).
-		listLogFilenames.rewind();
-		char *psLogFilename;
-		bool bAlreadyInList = false;
-		while ( (psLogFilename = listLogFilenames.next()) ) {
-			if (logfile == psLogFilename) {
-				bAlreadyInList = true;
-			}
-		}
-
-		if (!bAlreadyInList) {
-				// Note: append copies the string here.
-			listLogFilenames.append(logfile.c_str() );
-		}
-
-        skip_whitespace(adBuf,offset);	// until the parser can do this itself
-    }
-
-	return rtnVal;
-}
-
-///////////////////////////////////////////////////////////////////////////////
-
-int
-MultiLogFiles::getQueueCountFromSubmitFile(const MyString &strSubFilename,
-			const MyString &directory, MyString &errorMsg)
-{
-	dprintf( D_FULLDEBUG,
-				"MultiLogFiles::getQueueCountFromSubmitFile(%s, %s)\n",
-				strSubFilename.Value(), directory.Value() );
-
-	int queueCount = 0;
-	errorMsg = "";
-
-	MyString	fullpath("");
-	if ( directory != "" ) {
-		fullpath = directory + DIR_DELIM_STRING + strSubFilename;
-	} else {
-		fullpath = strSubFilename;
-	}
-
-	StringList	logicalLines;
-	if ( (errorMsg = fileNameToLogicalLines( strSubFilename,
-				logicalLines)) != "" ) {
-		return -1;
-	}
-
-		// Now look through the submit file logical lines to find any
-		// queue commands, and count up the total number of job procs
-		// to be queued.
-	const char *	paramName = "queue";
-	const char *logicalLine;
-	while( (logicalLine = logicalLines.next()) != NULL ) {
-		MyString	submitLine(logicalLine);
-		submitLine.Tokenize();
-		const char *DELIM = " ";
-		const char *rawToken = submitLine.GetNextToken( DELIM, true );
-		if ( rawToken ) {
-			MyString	token(rawToken);
-			token.trim();
-			if ( !strcasecmp(token.Value(), paramName) ) {
-				rawToken = submitLine.GetNextToken( DELIM, true );
-				if ( rawToken ) {
-					queueCount += atoi( rawToken );
-				} else {
-					queueCount++;
-				}
-			}
-		}
-	}
-
-	return queueCount;
-}
-
-///////////////////////////////////////////////////////////////////////////////
-
-MyString
-MultiLogFiles::getValuesFromFile(const MyString &fileName, 
-			const MyString &keyword, StringList &values, int skipTokens)
-{
-
-	MyString	errorMsg;
-	StringList	logicalLines;
-	if ( (errorMsg = fileNameToLogicalLines( fileName,
-				logicalLines )) != "" ) {
-		return errorMsg;
-	}
-
-	const char *	logicalLine;
-	while ( (logicalLine = logicalLines.next()) ) {
-
-		if ( strcmp(logicalLine, "") ) {
-
-				// Note: StringList constructor removes leading
-				// whitespace from lines.
-			StringList	tokens(logicalLine, " \t");
-			tokens.rewind();
-
-			if ( !strcasecmp(tokens.next(), keyword.Value()) ) {
-					// Skip over unwanted tokens.
-				for ( int skipped = 0; skipped < skipTokens; skipped++ ) {
-					if ( !tokens.next() ) {
-						MyString result = MyString( "Improperly-formatted DAG "
-									"file: value missing after keyword <" ) +
-									keyword + ">";
-			    		return result;
-					}
-				}
-
-					// Get the value.
-				const char *newValue = tokens.next();
-				if ( !newValue || !strcmp( newValue, "") ) {
-					MyString result = MyString( "Improperly-formatted DAG "
-								"file: value missing after keyword <" ) +
-								keyword + ">";
-			    	return result;
-				}
-
-					// Add the value we just found to the values list
-					// (if it's not already in the list -- we don't want
-					// duplicates).
-				values.rewind();
-				char *existingValue;
-				bool alreadyInList = false;
-				while ( (existingValue = values.next()) ) {
-					if (!strcmp( existingValue, newValue ) ) {
-						alreadyInList = true;
-					}
-				}
-
-				if (!alreadyInList) {
-						// Note: append copies the string here.
-					values.append(newValue);
-				}
-			}
-		}
-	}	
-
-	return "";
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 
 MyString
