@@ -45,9 +45,8 @@ CheckResult(int line, CheckEvents::check_event_result_t expectedResult,
 int main(int argc, char **argv)
 {
 		// Set up the dprintf stuff...
-	Termlog = true;
-	dprintf_config("test_check_events", get_param_functions());
-	DebugFlags = D_ALWAYS;
+	dprintf_set_tool_debug("test_check_events", 0);
+	set_debug_flags(NULL, D_ALWAYS);
 
 	bool			result = true;
 
@@ -695,7 +694,7 @@ CheckArgs(int argc, char **argv)
 				printf("%s", usage);
 				status = STATUS_ERROR;
 			} else {
-				set_debug_flags( argv[index] );
+				set_debug_flags( argv[index], 0 );
 			}
 
 		} else if ( !strcmp(argv[index], "-usage") ) {
