@@ -47,7 +47,7 @@ bool test_EvalStringMyString(compat_classad::ClassAd *c1, compat_classad::ClassA
 bool test_EvalStringStdString(compat_classad::ClassAd *c1, compat_classad::ClassAd *c2, int verbose);
 
 bool test_NextDirtyExpr(compat_classad::ClassAd *c1, int verbose);
-bool test_EscapeStringValue(compat_classad::ClassAd *c1, int verbose);
+bool test_EscapeAdStringValue(compat_classad::ClassAd *c1, int verbose);
 
 bool test_EvalTree(compat_classad::ClassAd *c1, compat_classad::ClassAd *c2,int verbose);
 
@@ -805,8 +805,8 @@ bool test_NextDirtyExpr(compat_classad::ClassAd *c1, int verbose)
 }
 //}}}
 
-//{{{ test_EscapeStringValue
-bool test_EscapeStringValue(compat_classad::ClassAd *c1, int verbose)
+//{{{ test_EscapeAdStringValue
+bool test_EscapeAdStringValue(compat_classad::ClassAd *c1, int verbose)
 {
     bool passed = true;
 
@@ -816,10 +816,10 @@ bool test_EscapeStringValue(compat_classad::ClassAd *c1, int verbose)
     const char *tmp; 
     string tmpString;
 
-    MyString msTmp;
+    string msTmp;
     c1->EvaluateAttrString("A", tmpString);
 
-    tmp = c1->EscapeStringValue(tmpString.c_str(), msTmp);
+    tmp = EscapeAdStringValue(tmpString.c_str(), msTmp);
 
     if(strcmp(tmp, ans1))
     {
@@ -827,10 +827,10 @@ bool test_EscapeStringValue(compat_classad::ClassAd *c1, int verbose)
     }
 
     if(verbose == 2)
-        printf("Expected %s and EscapeStringValue returned %s.\n", ans1, tmp); 
+        printf("Expected %s and EscapeAdStringValue returned %s.\n", ans1, tmp); 
 
     c1->EvaluateAttrString("F", tmpString);
-    tmp = c1->EscapeStringValue(tmpString.c_str(), msTmp);
+    tmp = EscapeAdStringValue(tmpString.c_str(), msTmp);
 
     if(strcmp(tmp, ans2))
     {
@@ -838,9 +838,9 @@ bool test_EscapeStringValue(compat_classad::ClassAd *c1, int verbose)
     }
 
     if(verbose == 2)
-        printf("Expected %s and EscapeStringValue returned %s.\n", ans2, tmp); 
+        printf("Expected %s and EscapeAdStringValue returned %s.\n", ans2, tmp); 
 
-    tmp = c1->EscapeStringValue(NULL, msTmp);
+    tmp = EscapeAdStringValue(NULL, msTmp);
     
     if(tmp != NULL)
     {
@@ -1480,8 +1480,8 @@ void setUpAndRun(int verbose)
     printf("-------------\n");
 
 
-    printf("Testing EscapeStringValue...\n");
-    passedTest[8] = test_EscapeStringValue(compC4, verbose);
+    printf("Testing EscapeAdStringValue...\n");
+    passedTest[8] = test_EscapeAdStringValue(compC4, verbose);
     printf("Escape String Value %s.\n", passedTest[9] ? "passed" : "failed");
     printf("-------------\n");
 
