@@ -201,8 +201,7 @@ void handle_sig(int sig)
 int
 main(int argc, const char **argv)
 {
-	DebugFlags = D_ALWAYS;
-	param_functions *p_funcs = NULL;
+	set_debug_flags(NULL, D_ALWAYS);
 
 	set_mySubSystem( "TEST_LOG_READER_STATE", SUBSYSTEM_TYPE_TOOL );
 
@@ -211,8 +210,7 @@ main(int argc, const char **argv)
 	config();
 
 		// Set up the dprintf stuff...
-	p_funcs = get_param_functions();
-	dprintf_config("TEST_LOG_READER_STATE", p_funcs);
+	dprintf_config("TEST_LOG_READER_STATE");
 
 	Options	opts;
 	if ( CheckArgs( argc, argv, opts ) < 0 ) {
@@ -851,7 +849,7 @@ Options::handleOpt( SimpleArg &arg, int &argno )
 
 	} else if ( arg.Match('d', "debug") ) {
 		if ( arg.hasOpt() ) {
-			set_debug_flags( const_cast<char *>(arg.getOpt()) );
+			set_debug_flags( const_cast<char *>(arg.getOpt()), 0 );
 			argno = arg.ConsumeOpt( );
 		} else {
 			fprintf(stderr, "Value needed for '%s'\n", arg.Arg() );
