@@ -368,9 +368,8 @@ main(int argc, const char **argv)
 	config();
 
 		// Set up the dprintf stuff...
-	Termlog = true;
-	dprintf_config("test_log_writer", get_param_functions());
-	DebugFlags = D_ALWAYS;
+	dprintf_set_tool_debug("test_log_writer", 0);
+	set_debug_flags(NULL, D_ALWAYS);
 
 	bool			error = false;
 	GlobalOptions	opts;
@@ -595,7 +594,7 @@ GlobalOptions::parseArgs( int argc, const char **argv )
 			if ( arg.hasOpt() ) {
 				const char	*flags;
 				arg.getOpt( flags );
-				set_debug_flags( const_cast<char *>(flags) );
+				set_debug_flags( const_cast<char *>(flags), 0 );
 			} else {
 				fprintf(stderr, "Value needed for '%s'\n", arg.Arg() );
 				printf("%s", usage);
