@@ -294,7 +294,10 @@ void uninit_file_owner_ids() {}
 uid_t getuid() { return get_my_uid(); }
 
 // Static/Global objects
-extern dynuser *myDynuser; 	// the "system wide" dynuser object
+
+// This is the global object to access Dynuser.
+dynuser		myDyn;
+dynuser		*myDynuser = &myDyn;
 
 static HANDLE CurrUserHandle = NULL;
 static char *UserLoginName = NULL; // either a "nobody" account or the submitting user
