@@ -163,8 +163,13 @@ ClassAd* user_job_policy(ClassAd *jad)
 			*/
 
 			UserPolicy userpolicy;
+#ifdef USE_NON_MUTATING_USERPOLICY
+			userpolicy.Init();
+			int analyze_result = userpolicy.AnalyzePolicy(*jad, PERIODIC_ONLY);
+#else
 			userpolicy.Init(jad);
 			int analyze_result = userpolicy.AnalyzePolicy(PERIODIC_ONLY);
+#endif
 
 			/* should I perform a periodic hold? */
 			if(analyze_result == HOLD_IN_QUEUE)
@@ -346,21 +351,100 @@ int JadKind(ClassAd *suspect)
 
 /* NEW INTERFACE */
 
+#ifdef USE_NON_MUTATING_USERPOLICY
+  #define POLICY_NONE                    SYS_POLICY_NONE
+  #define POLICY_SYSTEM_PERIODIC_HOLD    SYS_POLICY_PERIODIC_HOLD
+  #define POLICY_SYSTEM_PERIODIC_RELEASE SYS_POLICY_PERIODIC_RELEASE
+  #define POLICY_SYSTEM_PERIODIC_REMOVE  SYS_POLICY_PERIODIC_REMOVE
+#else
+  #define POLICY_NONE                    NULL
+  #define POLICY_SYSTEM_PERIODIC_HOLD    PARAM_SYSTEM_PERIODIC_HOLD
+  #define POLICY_SYSTEM_PERIODIC_RELEASE PARAM_SYSTEM_PERIODIC_RELEASE
+  #define POLICY_SYSTEM_PERIODIC_REMOVE  PARAM_SYSTEM_PERIODIC_REMOVE
+#endif
+
 
 UserPolicy::UserPolicy()
+#ifdef USE_NON_MUTATING_USERPOLICY
+	: m_sys_periodic_hold(NULL)
+	, m_sys_periodic_release(NULL)
+	, m_sys_periodic_remove(NULL)
+	, m_fire_subcode(0)
+#else
+	: m_ad(NULL)
+#endif
+	, m_fire_expr_val(-1)
+	, m_fire_source(FS_NotYet)
+	, m_fire_expr(NULL)
 {
-	m_ad = NULL;
-	m_fire_source = FS_NotYet;
-	m_fire_expr = NULL;
-	m_fire_expr_val = -1;
 }
 
 UserPolicy::~UserPolicy()
 {
+#ifdef USE_NON_MUTATING_USERPOLICY
+	ClearConfig();
+#else
 	m_ad = NULL;
+#endif
 	m_fire_expr = NULL;
 }
 
+#ifdef USE_NON_MUTATING_USERPOLICY
+
+void UserPolicy::Init()
+{
+	ResetTriggers();
+	Config();
+}
+
+void UserPolicy::ClearConfig()
+{
+	if (m_sys_periodic_hold) delete m_sys_periodic_hold; m_sys_periodic_hold = NULL;
+	if (m_sys_periodic_release) delete m_sys_periodic_release; m_sys_periodic_release = NULL;
+	if (m_sys_periodic_remove) delete m_sys_periodic_remove; m_sys_periodic_remove = NULL;
+}
+
+
+void UserPolicy::Config()
+{
+	ClearConfig();
+
+	auto_free_ptr expr_string(param(PARAM_SYSTEM_PERIODIC_HOLD));
+	if (expr_string) {
+		ParseClassAdRvalExpr(expr_string, m_sys_periodic_hold);
+		long long ival = 1;
+		if (m_sys_periodic_hold && ExprTreeIsLiteralNumber(m_sys_periodic_hold, ival) &&  ! ival) {
+			delete m_sys_periodic_hold; m_sys_periodic_hold = NULL;
+		}
+	}
+	expr_string.set(param(PARAM_SYSTEM_PERIODIC_RELEASE));
+	if (expr_string) {
+		ParseClassAdRvalExpr(expr_string, m_sys_periodic_release);
+		long long ival = 1;
+		if (m_sys_periodic_release && ExprTreeIsLiteralNumber(m_sys_periodic_release, ival) &&  ! ival) {
+			delete m_sys_periodic_release; m_sys_periodic_release = NULL;
+		}
+	}
+
+	expr_string.set(param(PARAM_SYSTEM_PERIODIC_REMOVE));
+	if (expr_string) {
+		ParseClassAdRvalExpr(expr_string, m_sys_periodic_remove);
+		long long ival = 1;
+		if (m_sys_periodic_remove && ExprTreeIsLiteralNumber(m_sys_periodic_remove, ival) &&  ! ival) {
+			delete m_sys_periodic_remove; m_sys_periodic_remove = NULL;
+		}
+	}
+}
+
+void UserPolicy::ResetTriggers()
+{
+	m_fire_expr_val = -1;
+	m_fire_source = FS_NotYet;
+	m_fire_expr = NULL;
+}
+
+#else
+
 void UserPolicy::Init(ClassAd *ad)
 {
 	ASSERT(ad);
@@ -410,24 +494,33 @@ void UserPolicy::SetDefaults()
 	}
 }
 
+#endif
+
+#ifdef USE_NON_MUTATING_USERPOLICY
+int
+UserPolicy::AnalyzePolicy(ClassAd & ad, int mode)
+{
+#else
 int
 UserPolicy::AnalyzePolicy( int mode )
 {
-	int on_exit_hold, on_exit_remove;
-	int timer_remove;
-	int state;
-
 	if (m_ad == NULL)
 	{
 		EXCEPT("UserPolicy Error: Must call Init() first!");
 	}
 
+	ClassAd & ad = *m_ad;
+#endif
+
+	int timer_remove;
+	int state;
+
 	if (mode != PERIODIC_ONLY && mode != PERIODIC_THEN_EXIT)
 	{
 		EXCEPT("UserPolicy Error: Unknown mode in AnalyzePolicy()");
 	}
 
-	if( !m_ad->LookupInteger(ATTR_JOB_STATUS,state) ) {
+	if( ! ad.LookupInteger(ATTR_JOB_STATUS,state) ) {
 		return UNDEFINED_EVAL;
 	}
 
@@ -448,12 +541,16 @@ UserPolicy::AnalyzePolicy( int mode )
 
 	/* Should I perform a remove based on the epoch time? */
 	m_fire_expr = ATTR_TIMER_REMOVE_CHECK;
-	if(!m_ad->LookupInteger(ATTR_TIMER_REMOVE_CHECK, timer_remove)) {
+	if ( ! ad.LookupInteger(ATTR_TIMER_REMOVE_CHECK, timer_remove)) {
 		//check if attribute exists, but is undefined
-		if(m_ad->Lookup(ATTR_TIMER_REMOVE_CHECK) != NULL)
+		ExprTree * expr = ad.Lookup(ATTR_TIMER_REMOVE_CHECK);
+		if (expr != NULL)
 		{
 			m_fire_expr_val = -1;
 			m_fire_source = FS_JobAttribute;
+		#ifdef USE_NON_MUTATING_USERPOLICY
+			ExprTreeToString(expr, m_fire_unparsed_expr);
+		#endif
 			return UNDEFINED_EVAL;
 		}
 		timer_remove = -1;
@@ -461,6 +558,9 @@ UserPolicy::AnalyzePolicy( int mode )
 	if( timer_remove >= 0 && timer_remove < time(NULL) ) {
 		m_fire_expr_val = 1;
 		m_fire_source = FS_JobAttribute;
+	#ifdef USE_NON_MUTATING_USERPOLICY
+		ExprTreeToString(ad.Lookup(ATTR_TIMER_REMOVE_CHECK), m_fire_unparsed_expr);
+	#endif
 		return REMOVE_FROM_QUEUE;
 	}
 
@@ -468,20 +568,20 @@ UserPolicy::AnalyzePolicy( int mode )
 
 	/* should I perform a periodic hold? */
 	if(state!=HELD) {
-		if(AnalyzeSinglePeriodicPolicy(ATTR_PERIODIC_HOLD_CHECK, PARAM_SYSTEM_PERIODIC_HOLD, HOLD_IN_QUEUE, retval)) {
+		if(AnalyzeSinglePeriodicPolicy(ad, ATTR_PERIODIC_HOLD_CHECK, POLICY_SYSTEM_PERIODIC_HOLD, HOLD_IN_QUEUE, retval)) {
 			return retval;
 		}
 	}
 
 	/* Should I perform a periodic release? */
 	if(state==HELD) {
-		if(AnalyzeSinglePeriodicPolicy(ATTR_PERIODIC_RELEASE_CHECK, PARAM_SYSTEM_PERIODIC_RELEASE, RELEASE_FROM_HOLD, retval)) {
+		if(AnalyzeSinglePeriodicPolicy(ad, ATTR_PERIODIC_RELEASE_CHECK, POLICY_SYSTEM_PERIODIC_RELEASE, RELEASE_FROM_HOLD, retval)) {
 			return retval;
 		}
 	}
 
 	/* Should I perform a periodic remove? */
-	if(AnalyzeSinglePeriodicPolicy(ATTR_PERIODIC_REMOVE_CHECK, PARAM_SYSTEM_PERIODIC_REMOVE, REMOVE_FROM_QUEUE, retval)) {
+	if(AnalyzeSinglePeriodicPolicy(ad, ATTR_PERIODIC_REMOVE_CHECK, POLICY_SYSTEM_PERIODIC_REMOVE, REMOVE_FROM_QUEUE, retval)) {
 		return retval;
 	}
 
@@ -495,7 +595,7 @@ UserPolicy::AnalyzePolicy( int mode )
 
 	/* This better be in the classad because it determines how the process
 		exited, either by signal, or by exit() */
-	if( ! m_ad->LookupExpr(ATTR_ON_EXIT_BY_SIGNAL) ) {
+	if( ! ad.LookupExpr(ATTR_ON_EXIT_BY_SIGNAL) ) {
 		EXCEPT( "UserPolicy Error: %s is not present in the classad",
 				ATTR_ON_EXIT_BY_SIGNAL );
 	}
@@ -504,15 +604,21 @@ UserPolicy::AnalyzePolicy( int mode )
 		are defined, if not, then except because
 		caller should have filled this in if calling
 		this function saying to check the exit policies. */
-	if( m_ad->LookupExpr(ATTR_ON_EXIT_CODE) == 0 && 
-		m_ad->LookupExpr(ATTR_ON_EXIT_SIGNAL) == 0 )
+	if( ad.LookupExpr(ATTR_ON_EXIT_CODE) == 0 && 
+		ad.LookupExpr(ATTR_ON_EXIT_SIGNAL) == 0 )
 	{
 		EXCEPT( "UserPolicy Error: No signal/exit codes in job ad!" );
 	}
 
 	/* Should I hold on exit? */
+#if 1
+	if (AnalyzeSinglePeriodicPolicy(ad, ATTR_ON_EXIT_HOLD_CHECK, POLICY_NONE, HOLD_IN_QUEUE, retval)) {
+		return retval;
+	}
+#else
+	int on_exit_hold, on_exit_remove;
 	m_fire_expr = ATTR_ON_EXIT_HOLD_CHECK;
-	if( ! m_ad->EvalBool(ATTR_ON_EXIT_HOLD_CHECK, m_ad, on_exit_hold) ) {
+	if( ! ad.EvalBool(ATTR_ON_EXIT_HOLD_CHECK, m_ad, on_exit_hold) ) {
 		m_fire_source = FS_JobAttribute;
 		return UNDEFINED_EVAL;
 	}
@@ -521,8 +627,29 @@ UserPolicy::AnalyzePolicy( int mode )
 		m_fire_source = FS_JobAttribute;
 		return HOLD_IN_QUEUE;
 	}
+#endif
 
 	/* Should I remove on exit? */
+#if 1
+	ExprTree * expr = ad.Lookup(ATTR_ON_EXIT_REMOVE_CHECK);
+	if ( ! expr) {
+		// If no expression, the default behavior is to remove on exit.
+		m_fire_expr_val = 1; 
+		m_fire_expr = ATTR_ON_EXIT_REMOVE_CHECK;
+		m_fire_source = FS_JobAttribute;
+	#ifdef USE_NON_MUTATING_USERPOLICY
+		m_fire_reason.clear();
+		m_fire_unparsed_expr = "true";
+	#endif
+		return REMOVE_FROM_QUEUE;
+	}
+	if (AnalyzeSinglePeriodicPolicy(ad, ATTR_ON_EXIT_REMOVE_CHECK, POLICY_NONE, REMOVE_FROM_QUEUE, retval)) {
+		return retval;
+	}
+#ifdef USE_NON_MUTATING_USERPOLICY
+	ExprTreeToString(expr, m_fire_unparsed_expr);
+#endif
+#else
 	m_fire_expr = ATTR_ON_EXIT_REMOVE_CHECK;
 	if( ! m_ad->EvalBool(ATTR_ON_EXIT_REMOVE_CHECK, m_ad, on_exit_remove) ) {
 		m_fire_source = FS_JobAttribute;
@@ -533,6 +660,7 @@ UserPolicy::AnalyzePolicy( int mode )
 		m_fire_source = FS_JobAttribute;
 		return REMOVE_FROM_QUEUE;
 	}
+#endif
 		// If we didn't want to remove it, OnExitRemove was false,
 		// which means we want the job to stay in the queue...
 	m_fire_expr_val = 0;
@@ -540,8 +668,197 @@ UserPolicy::AnalyzePolicy( int mode )
 	return STAYS_IN_QUEUE;
 }
 
+#ifdef USE_NON_MUTATING_USERPOLICY
 
-bool UserPolicy::AnalyzeSinglePeriodicPolicy(const char * attrname, const char * macroname, int on_true_return, int & retval)
+bool UserPolicy::AnalyzeSinglePeriodicPolicy(ClassAd & ad, ExprTree * expr, int on_true_return, int & retval)
+{
+	ASSERT(expr);
+
+	int result = 0;
+	long long ival = 0;
+
+	classad::Value val;
+	if (ad.EvaluateExpr(expr, val) && val.IsNumber(ival)) {
+		result = (ival != 0);
+	} else {
+		if (ExprTreeIsLiteral(expr, val) && val.IsUndefinedValue()) {
+			// if the expr is defined to be undefined, treat that a false result.
+			result = 0;
+		} else {
+			// if the expr evaluates to undefined, treat that as a triggering expression.
+			m_fire_expr_val = -1;
+			retval = UNDEFINED_EVAL;
+			return true;
+		}
+	}
+
+	if( result ) {
+		m_fire_expr_val = 1;
+		retval = on_true_return;
+		return true;
+	}
+
+	return false;
+}
+
+bool UserPolicy::AnalyzeSinglePeriodicPolicy(ClassAd & ad, const char * attrname, SysPolicyId sys_policy, int on_true_return, int & retval)
+{
+	ASSERT(attrname);
+
+	// Evaluate the specified expression in the job ad
+	m_fire_expr = attrname;
+	ExprTree * expr = ad.Lookup(attrname);
+	if (expr && AnalyzeSinglePeriodicPolicy(ad, expr, on_true_return, retval)) {
+		m_fire_source = FS_JobAttribute;
+		m_fire_reason.clear();
+		m_fire_subcode = 0;
+
+		// Save expression, subcode and reason for the FiringReason() method.
+		ExprTreeToString(expr, m_fire_unparsed_expr);
+		if (m_fire_expr_val != -1) {
+			std::string attr(attrname); attr += "SubCode";
+			ad.EvaluateAttrNumber(attr, m_fire_subcode);
+			attr = m_fire_expr; attr += "Reason";
+			ad.EvaluateAttrString(attr, m_fire_reason);
+		}
+		return true;
+	}
+
+	/* the old code.  looks like this always returns if there is no attrname expression, which is probably wrong...
+	int result = 0;
+	if(!ad.EvalBool(attrname, m_ad, result)) {
+        //check to see if the attribute actually exists, or if it's really
+        //  undefined.
+        //originally this if-statement wasn't here, so when the expression
+        //  tied to this attribute had an undefined value in it, e.g.
+        //      PeriodicRemove = DoesNotExist > 0
+        //  the function would set retval = UNDEFINED_EVAL and return
+        //
+        //now it can differentiate between the something in the 
+        //expression being undefined, and the attribute itself 
+        //being undefined.
+        if(m_ad->Lookup(attrname) != NULL)
+        {
+            m_fire_expr_val = -1;
+            m_fire_source = FS_JobAttribute;
+        }
+        retval = UNDEFINED_EVAL;
+		return true;
+	}
+	*/
+
+	const char * policy_name = NULL;
+	switch (sys_policy) {
+	case POLICY_SYSTEM_PERIODIC_HOLD:
+		expr = m_sys_periodic_hold;
+		policy_name = PARAM_SYSTEM_PERIODIC_HOLD;
+		break;
+	case POLICY_SYSTEM_PERIODIC_RELEASE:
+		expr = m_sys_periodic_release;
+		policy_name = PARAM_SYSTEM_PERIODIC_RELEASE;
+		break;
+	case POLICY_SYSTEM_PERIODIC_REMOVE:
+		expr = m_sys_periodic_remove;
+		policy_name = PARAM_SYSTEM_PERIODIC_REMOVE;
+		break;
+	default:
+		expr = NULL;
+		break;
+	}
+
+	if (expr) {
+#if 0 // don't do this, undefined system periodic expressions don't fire
+		int result_ok = AnalyzeSinglePeriodicPolicy(ad, expr, on_true_return, retval);
+		if (result_ok && m_fire_expr_val) {
+#else
+		long long ival = 0;
+		classad::Value val;
+		if (ad.EvaluateExpr(expr, val) && val.IsNumber(ival) && ival != 0) {
+			m_fire_expr_val = 1;
+#endif
+			m_fire_expr = policy_name;
+			m_fire_source = FS_SystemMacro;
+			m_fire_reason.clear();
+			m_fire_subcode = 0;
+
+			retval = on_true_return;
+
+			// fetch the unparsed value of the expression that fired.
+			ExprTreeToString(expr, m_fire_unparsed_expr);
+
+			// temp buffer for building _SUBCODE and _REASON param names.
+			char param_sub[sizeof("SYSTEM_PERIODIC_RELEASE_SUBCODE")+10];
+
+			std::string expr_string;
+			strcpy(param_sub, policy_name); strcat(param_sub, "_SUBCODE");
+			if (param(expr_string, param_sub, "") && ! expr_string.empty()) {
+				long long ival;
+				classad::Value val;
+				if (ad.EvaluateExpr(expr_string, val) && val.IsNumber(ival)) {
+					m_fire_subcode = (int)ival;
+				}
+			}
+
+			strcpy(param_sub, policy_name); strcat(param_sub, "_REASON");
+			if (param(expr_string, param_sub, "") && ! expr_string.empty()) {
+				classad::Value val;
+				if (ad.EvaluateExpr(expr_string, val) && val.IsStringValue(m_fire_reason)) {
+					// val.IsStringValue will have already set m_fire_reason
+				}
+			}
+
+			return true;
+		}
+	}
+
+	/*
+	// TODO: parse these macros into exprs outside of the loop.
+	if (macroname) {
+		param(m_fire_unparsed_expr, macroname, "");
+		if ( ! m_fire_unparsed_expr.empty()) {
+			// Just temporarily toss the expression into the job ad
+			ad.AssignExpr(ATTR_SCRATCH_EXPRESSION, m_fire_unparsed_expr.c_str());
+			expr = ad.Lookup(ATTR_SCRATCH_EXPRESSION);
+			int result_ok = expr && AnalyzeSinglePeriodicPolicy(ad, expr, on_true_return, retval);
+			ad.Delete(ATTR_SCRATCH_EXPRESSION);
+			if (result_ok && m_fire_expr_val) {
+				m_fire_expr = macroname;
+				m_fire_source = FS_SystemMacro;
+				m_fire_reason.clear();
+				m_fire_subcode = 0;
+
+				retval = on_true_return;
+
+				std::string param_name(macroname); param_name += "_SUBCODE";
+				std::string param_expr;
+				if (param(param_expr, param_name.c_str(), "") && ! param_expr.empty()) {
+					long long ival;
+					classad::Value val;
+					if (ad.EvaluateExpr(param_expr, val) && val.IsNumber(ival)) {
+						m_fire_subcode = (int)ival;
+					}
+				}
+				param_name = m_fire_expr; param_name += "_REASON";
+				if (param(param_expr, param_name.c_str(), "") && ! param_expr.empty()) {
+					classad::Value val;
+					if (ad.EvaluateExpr(param_expr, val) && val.IsStringValue(m_fire_reason)) {
+						//
+					}
+				}
+
+				return true;
+			}
+
+			m_fire_unparsed_expr.clear();
+		}
+	}
+	*/
+
+	return false;
+}
+#else
+
+bool UserPolicy::AnalyzeSinglePeriodicPolicy(ClassAd & /*ad*/, const char * attrname, const char * macroname, int on_true_return, int & retval)
 {
 	ASSERT(attrname);
 
@@ -599,6 +916,8 @@ bool UserPolicy::AnalyzeSinglePeriodicPolicy(const char * attrname, const char *
 	//macroname += attrname;
 }
 
+#endif
+
 const char* UserPolicy::FiringExpression(void)
 {
 	return m_fire_expr;
@@ -609,17 +928,22 @@ bool UserPolicy::FiringReason(MyString &reason,int &reason_code,int &reason_subc
 	reason_code = 0;
 	reason_subcode = 0;
 
-	if ( m_ad == NULL || m_fire_expr == NULL ) {
+	if ( m_fire_expr == NULL ) {
 		return false;
 	}
 
+	reason = "";
 
 	const char * expr_src;
+#ifdef USE_NON_MUTATING_USERPOLICY
+	std::string exprString;
+#else
 	MyString exprString;
 	std::string reason_expr_param;
 	std::string reason_expr_attr;
 	std::string subcode_expr_param;
 	std::string subcode_expr_attr;
+#endif
 	switch(m_fire_source) {
 		case FS_NotYet:
 			expr_src = "UNKNOWN (never set)";
@@ -628,6 +952,16 @@ bool UserPolicy::FiringReason(MyString &reason,int &reason_code,int &reason_subc
 		case FS_JobAttribute:
 		{
 			expr_src = "job attribute";
+#ifdef USE_NON_MUTATING_USERPOLICY
+			exprString = m_fire_unparsed_expr.c_str();
+			if (m_fire_expr_val == -1) {
+				reason_code = CONDOR_HOLD_CODE_JobPolicyUndefined;
+			} else {
+				reason_code = CONDOR_HOLD_CODE_JobPolicy;
+				reason_subcode = m_fire_subcode;
+				reason = m_fire_reason;
+			}
+#else
 			ExprTree *tree;
 			tree = m_ad->LookupExpr( m_fire_expr );
 
@@ -643,12 +977,24 @@ bool UserPolicy::FiringReason(MyString &reason,int &reason_code,int &reason_subc
 				formatstr(reason_expr_attr,"%sReason", m_fire_expr);
 				formatstr(subcode_expr_attr,"%sSubCode", m_fire_expr);
 			}
+#endif
 			break;
 		}
 
 		case FS_SystemMacro:
 		{
 			expr_src = "system macro";
+#ifdef USE_NON_MUTATING_USERPOLICY
+			exprString = m_fire_unparsed_expr.c_str();
+			if( m_fire_expr_val == -1 ) {
+				reason_code = CONDOR_HOLD_CODE_SystemPolicyUndefined;
+			}
+			else {
+				reason_code = CONDOR_HOLD_CODE_SystemPolicy;
+				reason_subcode = m_fire_subcode;
+				reason = m_fire_reason;
+			}
+#else
 			char * val = param(m_fire_expr);
 			exprString = val;
 			free(val);
@@ -660,6 +1006,7 @@ bool UserPolicy::FiringReason(MyString &reason,int &reason_code,int &reason_subc
 				formatstr(reason_expr_param,"%s_REASON", m_fire_expr);
 				formatstr(subcode_expr_param,"%s_SUBCODE", m_fire_expr);
 			}
+#endif
 			break;
 		}
 
@@ -668,8 +1015,10 @@ bool UserPolicy::FiringReason(MyString &reason,int &reason_code,int &reason_subc
 			break;
 	}
 
-	reason = "";
 
+#ifdef USE_NON_MUTATING_USERPOLICY
+	// don't need this. it's handled above.
+#else
 	MyString subcode_expr;
 	if( !subcode_expr_param.empty() &&
 		param(subcode_expr,subcode_expr_param.c_str(),NULL) &&
@@ -697,8 +1046,9 @@ bool UserPolicy::FiringReason(MyString &reason,int &reason_code,int &reason_subc
 	{
 		m_ad->EvalString(reason_expr_attr.c_str(), m_ad, reason);
 	}
+#endif
 
-	if( !reason.IsEmpty() ) {
+	if( !reason.empty() ) {
 		return true;
 	}
 
@@ -706,7 +1056,7 @@ bool UserPolicy::FiringReason(MyString &reason,int &reason_code,int &reason_subc
 	reason.formatstr( "The %s %s expression '%s' evaluated to ",
 					expr_src,
 					m_fire_expr,
-					exprString.Value());
+					exprString.c_str());
 
 	// Get a string for it's value
 	switch( m_fire_expr_val ) {
