@@ -236,7 +236,7 @@ VirshType::Shutdown()
 		    virErrorPtr err = virConnGetLastError(m_libvirt_connection);
 		    if (err && err->code != VIR_ERR_NO_DOMAIN)
 		      {
-			vmprintf(D_ALWAYS, "Error finding domain %s: %s\n", m_vm_name.Value(), (err ? err->message : "No reason found"));
+			vmprintf(D_ALWAYS, "Error finding domain %s: %s\n", m_vm_name.Value(), err->message);
 			return false;
 		      }
 		  }
@@ -523,15 +523,21 @@ VirshType::Suspend()
 	int result = virDomainSave(dom, tmpfilename.Value());
 	virDomainFree(dom);
 	set_priv(priv);
+
 	if( result != 0 ) {
-		// Read error output
-// 		char *temp = cmd_out.print_to_delimed_string("/");
-// 		m_result_msg = temp;
-// 		free( temp );
 		unlink(tmpfilename.Value());
 		return false;
 	}
 
+	priv = set_root_priv();
+	result = chown( tmpfilename.Value(), get_user_uid(), get_user_gid() );
+	set_priv( priv );
+
+	if( result != 0 ) {
+		dprintf( D_ALWAYS, "Error changing ownership of checkpoint: %d (%s), failing.\n", errno, strerror( errno ) );
+		return false;
+	}
+
 	m_suspendfile = tmpfilename;
 
 	setVMStatus(VM_SUSPENDED);
@@ -691,6 +697,7 @@ VirshType::Status()
 
 	virDomainInfo _info;
 	virDomainInfoPtr info = &_info;
+	memset( info, 0, sizeof( virDomainInfo ) );
 	if(virDomainGetInfo(dom, info) < 0)
 	  {
 	    virErrorPtr err = virConnGetLastError(m_libvirt_connection);
@@ -698,39 +705,49 @@ VirshType::Status()
 	    return false;
 	  }
 	if(info->state == VIR_DOMAIN_RUNNING || info->state == VIR_DOMAIN_BLOCKED)
-	  { 
-	    static unsigned long long LastCpuTime = 0; 
-	    static time_t LastStamp = time(0); 
-	    
+	  {
+	    static unsigned long long LastCpuTime = 0;
+	    static time_t LastStamp = time(0);
+
 	    time_t CurrentStamp = time(0);
-	    unsigned long long CurrentCpuTime= info->cpuTime;
-	    double percentUtilization=1.0;
-	    
+	    unsigned long long CurrentCpuTime = info->cpuTime;
+	    double percentUtilization = 1.0;
+
 	    setVMStatus(VM_RUNNING);
 	    // libvirt reports cputime in nanoseconds
 	    m_cpu_time = info->cpuTime / 1000000000.0;
 	    m_result_msg += "Running";
-	    
+
 	    m_result_msg += " ";
 	    m_result_msg += VMGAHP_STATUS_COMMAND_CPUUTILIZATION;
 	    m_result_msg += "=";
-	    
+
 	    if ( (CurrentStamp - LastStamp) > 0 )
 	    {
-	      // Old calc method because of libvirt version mismatches. 
+	      // Old calc method because of libvirt version mismatches.
 	      // courtesy of http://people.redhat.com/~rjones/virt-top/faq.html#calccpu 
 	      percentUtilization = (1.0 * (CurrentCpuTime-LastCpuTime) ) / ((CurrentStamp - LastStamp)*info->nrVirtCpu*1000000000.0);
 	      vmprintf(D_FULLDEBUG, "Computing utilization %f = (%llu) / (%d * %d * 1000000000.0)\n",percentUtilization, (CurrentCpuTime-LastCpuTime), (int) (CurrentStamp - LastStamp), info->nrVirtCpu );
 	    }
 
 	    m_result_msg += percentUtilization;
-	    
+
 	    m_result_msg += " " VMGAHP_STATUS_COMMAND_CPUTIME "=";
 	    m_result_msg += m_cpu_time;
 
-	    LastCpuTime = CurrentCpuTime; 
+	    LastCpuTime = CurrentCpuTime;
 	    LastStamp = CurrentStamp;
-	    
+
+	    // Memory usage is in kbytes.
+	    if( info->memory != 0 ) {
+	    	formatstr( m_result_msg, "%s %s=%lu", m_result_msg.c_str(), VMGAHP_STATUS_COMMAND_MEMORY, info->memory );
+	    }
+
+	    if( info->maxMem != 0 ) {
+	    	formatstr( m_result_msg, "%s %s=%lu", m_result_msg.c_str(), VMGAHP_STATUS_COMMAND_MAX_MEMORY, info->maxMem );
+	    }
+
+		vmprintf( D_FULLDEBUG, "Reporting status: %s\n", m_result_msg.c_str() );
 	    virDomainFree(dom);
 	    return true;
 	  }
@@ -1677,7 +1694,7 @@ KVMType::CreateConfigFile()
 	if( m_classAd.LookupString(VMPARAM_VM_DISK, kvm_disk) != 1 ) {
 		vmprintf(D_ALWAYS, "%s cannot be found in job classAd\n",
 				VMPARAM_VM_DISK);
-		m_result_msg = VMGAHP_ERR_JOBCLASSAD_XEN_NO_DISK_PARAM;
+		m_result_msg = VMGAHP_ERR_JOBCLASSAD_KVM_NO_DISK_PARAM;
 		return false;
 	}
 	kvm_disk.trim();
