@@ -84,6 +84,7 @@ globus_version_t local_version =
 
 extern globus_gass_transfer_listener_t gassServerListeners[];
 static int number_listening;
+static int number_open;
 
 #include "globus_gram_client.h"
 #include "globus_gss_assist.h"
@@ -131,14 +132,16 @@ globus_l_gass_server_ez_activate(void);
 static int
 globus_l_gass_server_ez_deactivate(void);
 
+static char module_name [] = "globus_gass_server_ez";
 globus_module_descriptor_t globus_i_gass_server_ez_module =
 {
-    "globus_gass_server_ez",
+    module_name,
     globus_l_gass_server_ez_activate,
     globus_l_gass_server_ez_deactivate,
     GLOBUS_NULL,
     GLOBUS_NULL,
-    &local_version
+    &local_version,
+    GLOBUS_NULL
 };
 
 
@@ -269,14 +272,15 @@ globus_gass_server_ez_init(globus_gass_transfer_listener_t * listener,
     server->callback=callback;
 
     globus_hashtable_insert(&globus_l_gass_server_ez_listeners,
-			    (void *)*listener,
+			    (void *)(intptr_t(*listener)),
 			    server);
 
-    rc=globus_gass_transfer_register_listen(*listener,
+    rc=globus_gass_transfer_register_listen((*listener),
 				globus_l_gass_server_ez_listen_callback,
 					(void *)reqattr);
 
     number_listening=1;
+	number_open = 1;
 /* insert error handling here*/
 
     error_exit:
@@ -446,8 +450,8 @@ globus_gass_server_ez_put_memory_done(void * arg,
 
 static void
 globus_l_gass_server_ez_close_callback(
-				void * user_arg,
-				globus_gass_transfer_listener_t listener)
+							   void * /*user_arg*/,
+							   globus_gass_transfer_listener_t /*listener*/)
 {
     /* should be cleaning up things related to the listener here
      * get rid of server struct stuff (hashtable) etc.
@@ -469,19 +473,20 @@ globus_l_gass_server_ez_listen_callback(
 				 user_arg,
 				 listener,
 				 globus_l_gass_server_ez_register_accept_callback,
-				 (void *)listener);
+				 (void *)(intptr_t(listener)));
 
-    try_to_listen();
-#if 0
-    if(rc != GLOBUS_SUCCESS)
-    {
-	/* to listen for additional requests*/
-	globus_gass_transfer_register_listen(
-	    listener,
-	    globus_l_gass_server_ez_listen_callback,
-	    user_arg);
-    }
-#endif
+	if ( number_open > 1 ) {
+		try_to_listen();
+	} else {
+		if(rc != GLOBUS_SUCCESS)
+		{
+			/* to listen for additional requests*/
+			globus_gass_transfer_register_listen(
+												 listener,
+												 globus_l_gass_server_ez_listen_callback,
+												 user_arg);
+		}
+	}
 }
 
 
@@ -501,6 +506,8 @@ globus_l_gass_server_ez_register_accept_callback(
     globus_byte_t * buf;
     int amt;
     int flags=0;
+    char fourohfour[] = "File Not Found";
+    char badrequest[] = "Bad Request";
 
     /* lookup our options */
     s=(globus_l_gass_server_ez_t *)globus_hashtable_lookup(
@@ -513,7 +520,7 @@ globus_l_gass_server_ez_register_accept_callback(
     if(rc != GLOBUS_SUCCESS ||
        parsed_url.url_path == GLOBUS_NULL || strlen(parsed_url.url_path) == 0U)
     {
-        globus_gass_transfer_deny(request, 404, "File Not Found");
+        globus_gass_transfer_deny(request, 404, fourohfour);
         globus_gass_transfer_request_destroy(request);
         if (rc == GLOBUS_SUCCESS)
             globus_url_destroy(&parsed_url);
@@ -619,7 +626,7 @@ globus_l_gass_server_ez_register_accept_callback(
                                                1024,
                                                1,
                                                globus_l_gass_server_ez_put_callback,
-                                               (void *) rc);
+                                               (void *) (intptr_t)rc);
 	    }
             break;
 
@@ -648,7 +655,7 @@ globus_l_gass_server_ez_register_accept_callback(
 	    }
 	    else
 	    {
-		globus_gass_transfer_deny(request, 404, "File Not Found");
+		globus_gass_transfer_deny(request, 404, fourohfour);
 		globus_gass_transfer_request_destroy(request);
 		goto reregister;
 	    }
@@ -658,6 +665,7 @@ globus_l_gass_server_ez_register_accept_callback(
             if(amt == -1)
             {
                 globus_free(buf);
+                close(rc);
                 goto deny;
             }
             globus_gass_transfer_authorize(request,
@@ -668,11 +676,11 @@ globus_l_gass_server_ez_register_accept_callback(
                                             amt,
                                             GLOBUS_FALSE,
                                             globus_l_gass_server_ez_get_callback,
-                                            (void *) rc);
+                                            (void *) (intptr_t)rc);
 	  break;
 	default:
 	deny:
-	  globus_gass_transfer_deny(request, 400, "Bad Request");
+	  globus_gass_transfer_deny(request, 400, badrequest);
 	  globus_gass_transfer_request_destroy(request);
 
 	}
@@ -681,13 +689,19 @@ globus_l_gass_server_ez_register_accept_callback(
     globus_url_destroy(&parsed_url);
 
   reregister_nourl:
-    try_to_listen();
-/*
-    globus_gass_transfer_register_listen(
-				(globus_gass_transfer_listener_t) listener,
+	if ( number_open > 1 ) {
+		try_to_listen();
+	} else {
+		/* In C++, casting from a 64-bit pointer to a 32-bit int is
+		 * illegal. But if you detour through a 64-bit integer type,
+		 * that's ok. The pointer started life as an int, so this
+		 * shouldn't cause any problems, though it is icky.
+		 */
+		globus_gass_transfer_register_listen(
+				(globus_gass_transfer_listener_t)(long) listener,
 				globus_l_gass_server_ez_listen_callback,
 				s->reqattr);
-*/
+	}
 
     if (path != GLOBUS_NULL) globus_free(path);
 
@@ -769,7 +783,39 @@ globus_l_gass_server_ez_put_callback(
 
     fd = (long) arg;
 
-    globus_libc_write(fd, bytes, len);
+#ifndef TARGET_ARCH_WIN32
+    ssize_t rc;
+#else
+        int rc;
+#endif
+    size_t written = 0;
+
+    while(written < len)
+    {
+        rc = globus_libc_write(fd,
+                               bytes  + written,
+                               len    - written);
+        if(rc < 0)
+        {
+            switch(errno)
+            {
+            case EAGAIN:
+            case EINTR:
+                break;
+            default:
+                goto cleanup;
+            }
+        }
+        else
+        {
+            written += rc;
+        }
+    }
+
+    // NOTE: Errors are completely ignored!
+
+cleanup:
+
     if(!last_data)
     {
         globus_gass_transfer_receive_bytes(request,
@@ -1006,8 +1052,9 @@ my_globus_gass_server_ez_init(globus_gass_transfer_listener_t * listener,
     server->reqattr=reqattr;
     server->callback=callback;
 
+	number_open++;
     globus_hashtable_insert(&globus_l_gass_server_ez_listeners,
-			    (void *)*listener,
+			    (void *)(intptr_t)*listener,
 			    server);
 
 /* insert error handling here*/
@@ -1027,7 +1074,7 @@ void try_to_listen(void)
     if (number_listening>0) return;
 
     n=-1;
-    for(i=0;i<20;i++) {
+    for(i=0;i<number_open;i++) {
       globus_gass_transfer_listener_struct_t *l = (globus_gass_transfer_listener_struct_t *) globus_handle_table_lookup(&globus_i_gass_transfer_listener_handles, gassServerListeners[i]);
       if (l->status == GLOBUS_GASS_TRANSFER_LISTENER_STARTING) {
         n=i;
@@ -1038,7 +1085,7 @@ void try_to_listen(void)
     if (n==-1) return;
 
     s = (globus_l_gass_server_ez_t *)globus_hashtable_lookup(
-                                                     &globus_l_gass_server_ez_listeners,(void *)gassServerListeners[n]);
+                                                     &globus_l_gass_server_ez_listeners,(void *)(intptr_t)gassServerListeners[n]);
 
     result = globus_gass_transfer_register_listen(
                                 gassServerListeners[n],
