@@ -6,6 +6,10 @@
 #include <boost/python.hpp>
 #include <boost/iterator/transform_iterator.hpp>
 
+void registerFunction(boost::python::object function, boost::python::object name);
+
+classad::ExprTree* convert_python_to_exprtree(boost::python::object value);
+
 struct ExprTreeHolder;
 
 struct AttrPairToFirst :
@@ -37,6 +41,9 @@ struct AttrPair :
 
 typedef boost::transform_iterator<AttrPair, classad::AttrList::iterator> AttrItemIter;
 
+// Look to see if a given function accepts a variable named "state".
+bool checkAcceptsState(boost::python::object pyFunc);
+
 struct ClassAdWrapper : classad::ClassAd, boost::python::wrapper<classad::ClassAd>
 {
     boost::python::object LookupWrap( const std::string &attr) const;
@@ -52,11 +59,11 @@ struct ClassAdWrapper : classad::ClassAd, boost::python::wrapper<classad::ClassA
     bool matches(boost::python::object) const;
     bool symmetricMatch(boost::python::object) const;
 
-    std::string toRepr();
+    std::string toRepr() const;
 
-    std::string toString();
+    std::string toString() const;
 
-    std::string toOldString();
+    std::string toOldString() const;
 
     AttrKeyIter beginKeys();
 
@@ -76,6 +83,10 @@ struct ClassAdWrapper : classad::ClassAd, boost::python::wrapper<classad::ClassA
 
     void update(boost::python::object source);
 
+    boost::python::list externalRefs(boost::python::object expr) const;
+
+    boost::python::list internalRefs(boost::python::object expr) const;
+
     ClassAdWrapper();
 
     ClassAdWrapper(const std::string &str);
