@@ -7,4 +7,7 @@ void export_daemon_and_ad_types();
 void export_config();
 void export_secman();
 void export_event_reader();
+void export_log_reader();
+void enable_classad_extensions();
+void enable_deprecation_warnings();
 
